/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/conf2.h: run-time configuration header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: conf2.h,v 1.18 2002/04/08 16:00:46 rageircd Exp $
 */

#ifndef __conf2_include__
#define __conf2_include__ 

#include "setup.h"
#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#ifdef ORATIMING
#include <sys/time.h>
#endif

/* SOME LIMITS */
#define INCLUDE_DEPTH	5	/* max config files to be loading at any one time */
#define MAX_CONFIG_PARAMS 10	/* max config params */

/* TYPE DEFINITIONS */
typedef struct _configfile ConfigFile;			/* configuration files */
typedef struct _configcommand ConfigCommand;		/* configuration commands (parsers) */
typedef struct _configbantype ConfigBanType;		/* configuration ban types */
typedef struct _configflag ConfigFlag;			/* configuration entry flags */
typedef struct _configitem ConfigItem;			/* generic configuration structure */
typedef struct _configitem_servinfo ConfigItem_servinfo; /* servinfo structure */
typedef struct _configitem_admin ConfigItem_admin;	/* admin structure */
typedef struct _configitem_class ConfigItem_class;	/* class structure */
typedef struct _configitem_allow ConfigItem_allow;	/* allow structure */
typedef struct _configitem_password ConfigItem_password; /* password structure */
typedef struct _configitem_oper ConfigItem_oper;	/* oper structure */
typedef struct _configitem_uline ConfigItem_uline;	/* uline structure */
typedef struct _configitem_link ConfigItem_link;	/* link structure */
typedef struct _configitem_ban ConfigItem_ban;		/* ban structure */
typedef struct _configitem_listen ConfigItem_listen;	/* listen structure */
typedef struct _configitem_except ConfigItem_except;	/* except structure */
typedef struct _configitem_vhost ConfigItem_vhost;	/* vhost structure */
typedef struct _configitem_set ConfigItem_set;		/* set structure */

/* STRUCTURES */
struct _configfile {
	char *filename;		/* name of config file */
	int linenum;		/* current line */
};

struct _configcommand {
	char *cmd;		/* name of command */
	void (*func)();		/* pointer to command parser */
	int paras;		/* number of required parameters */
};

struct _configbantype {
	char *name;
	int type;
};

struct _configflag {
	short temporary : 1;
};

struct _configitem {
	ConfigItem *next, *prev;
	ConfigFlag flag;
};

/* old M line */
struct _configitem_servinfo {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *name;		/* server name */
	char *info;		/* server info */
};

/* old A line */
struct _configitem_admin {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *line;		/* line */
};

/* old Y line */
struct _configitem_class {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *name;		/* name of class */
	int pingfreq;		/* class ping frequency */
	int connfreq;		/* class connect frequency */
	int maxclients;		/* max clients in class */
	int sendq;		/* max sendq for class */
	int clients;		/* internal: total clients in class */
};

/* old I line */
struct _configitem_allow {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *hostname;		/* allowed hostname */
	char *ip;		/* allowed ip */
	int maxperip;		/* max clients per ip */
	ConfigItem_class *class; /* class to put client in */
	char *password;		/* optional connection password */
};

/* old X line */
struct _configitem_password {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *die;		/* die password */
	char *restart;		/* restart password */
};

/* old O line */
struct _configitem_oper {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *name;		/* name of oper */
	char *from;		/* user@host */
	char *password;		/* password */
	ConfigItem_class *class; /* class to put oper in */
	long flags;		/* oper flags */
};

/* old U line */
struct _configitem_uline {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *servername;	/* name of server */
};

/* old C/N line */
struct _configitem_link {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *servername;	/* name of server */
	char *fromip;		/* ip of server */
	char *bindip;		/* ip to bind to when linking */
	char *password;		/* link password */
	int port;		/* port to link on */
	ConfigItem_class *class; /* class to put server in */
	int options;		/* link options */
	int refcount;		/* internal: number of servers using this link */
	struct in_addr ipnum;	/* internal: in_addr for socket/bindip of link */
	time_t hold;		/* internal: time before retrying link */
};

/* old K/Z/G/F/B line */
struct _configitem_ban {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	int type;		/* type of ban (kline, akill, zline, etc) */
	char *mask;		/* ban mask */
	char *reason;		/* reason for ban */
};

/* old P line */
struct _configitem_listen {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *ip;		/* ip address of port */
	int port;		/* the port */
	long options;		/* port options */
	long clients;		/* internal: clients connected via port */
};

/* old E line */
struct _configitem_except {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *mask;		/* except mask */
};

/* old VHOST config */
struct _configitem_vhost {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *vhost;		/* the vhost */
	char *login;		/* login */
	char *password;		/* password */
	char *from;		/* user@host */
};

/* dynamic configuration options */
struct _configitem_set {
	short shortmotd;	/* short motd on connect */
	short showopermotd;	/* show opermotd on oper up */
	short hideulines;	/* hide ulined servers to non-opers */
	short encryptedpasswords; /* password encryption */
	short doidentd;		/* check ident on connect */
	short showheaders;	/* show headers on connect */
	short nomixedcase;	/* no mixed case username */
	short ignoreident;	/* ignore ident responce */
	short failopernotice;	/* warn opers about failed oper attempts */
	short nooperflood;	/* stop opers from being able to flood */
	short ignorefirstchar;	/* ignore first char of ident */
	short dynchans;		/* allow dynamic channels */
	short automaskopers;	/* mask opers on operup */
	short hub;		/* server is a hub */
	short logging;		/* enable server logging */
	int pacetime;		/* seconds between /MOTD(etc) attempts */
	int maxchansperuser;	/* max channels per user */
	int floodcount;		/* flood control */
	int linkdepth;		/* depth of link */
	int maxclients;		/* maximum clients */
	int knockdelay;		/* seconds between /knock attempts */
	int acceptnoticetime;	/* seconds between accept notices */
	long connectmodes;	/* modes given to user upon connection */
	char *localklineaddress; /* local kline e-mail address */
	char *ircnetwork;	/* network name */
	char *klineaddress;	/* network kline e-mail address */
	char *helpchannel;	/* network chat/help channel */
	char *defaultserver;	/* network random/default server */
	char *services;		/* network services server */
	char *stats;		/* network stats server */
	char *chanserv;		/* chanserv nickname */
	char *nickserv;		/* nickserv nickname */
	char *memoserv;		/* memoserv nickname */
	char *operserv;		/* operserv nickname */
	char *statserv;		/* statserv nickname */
	char *netadminmask;	/* network administrator masked host */
	char *adminmask;	/* server administrator masked host */
	char *coadminmask;	/* server co-administrator masked host */
	char *opermask;		/* irc operator masked host */
	char *usermask;		/* user masked host prefix */
	long mask_key1;		/* set->maskingkeys */
	long mask_key2;
	long mask_key3;
	long mask_crc;
	char *dns_nameserver;	/* set->dns */
	time_t dns_timeout;
	int dns_retries;
	int throttle_trigcount;	/* set->throttle */
	time_t throttle_trigtime;
};

/* Link Options */
#define LINK_AUTOCONNECT	0x0001	/* autoconnect to server */
#define LINK_ZIP		0x0002	/* data to link is to be compressed (zipped) */
#define LINK_DKEY		0x0004	/* link is encrypted */

/* Ban Types */
#define BAN_USERHOST		0x0001	/* ban is user@host (K line) */
#define BAN_IP			0x0002	/* ban is ip (Z line) */
#define BAN_NICKNAME		0x0004	/* ban is nickname (Q line) */
#define BAN_NICKNAME_S		0x0008	/* ban is nickname (SQLINE) */
#define BAN_GCOS		0x0010	/* ban is gcos (G line) */
#define BAN_GCOS_S		0x0020	/* ban is gcos (SGLINE) */
#define BAN_SERVER		0x0010	/* ban is server (new) */
#define BAN_FILE		0x0020	/* ban is file (B line) */
#define BAN_CHANNEL		0x0040	/* ban is channel (new) */
#define BAN_AUTOKILL		0x0080	/* ban is autokill via services */

/* Listen Options */
#define LISTEN_BOUND		0x0001	/* listener bound */
#define LISTEN_MAINPORT		0x0002	/* listener is the main port (protection) */
#define LISTEN_CLIENTONLY	0x0004	/* listener is for clients only */
#define LISTEN_SERVERONLY	0x0008	/* listener is for servers only */
#define LISTEN_OPENSSL		0x0010	/* listener is for openssl clients */

#endif /* __conf2_include__ */
