/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/dh.h: Diffie-Helman key exchange header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: dh.h,v 1.7 2002/04/08 16:00:46 rageircd Exp $
 */

#include "setup.h"

#ifdef HAVE_OPENSSL_ON

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <openssl/rand.h>
#include <openssl/err.h>
#include <openssl/bn.h>
#include <openssl/dh.h>

typedef struct session_info SessionInfo;
typedef struct rc4_state RC4_state;
typedef unsigned char RC4_byte;
typedef unsigned int RC4_word;

struct session_info {
	DH *dh;
	char *session_shared;
	int session_shared_length;
};

struct rc4_state {
	RC4_byte mstate[256];
	RC4_byte x;
	RC4_byte y;
};

/* Do not change KEY_BITS unless you also change PRIME_BITS */

#define KEY_BITS 512
#define RAND_BITS KEY_BITS
#define RAND_BYTES (RAND_BITS / 8)
#define RAND_BYTES_HEX ((RAND_BYTES * 2) + 1)

#define PRIME_BITS 1024
#define PRIME_BYTES (PRIME_BITS / 8)
#define PRIME_BYTES_HEX ((PRIME_BYTES * 2) + 1)

extern int dh_init();
extern void dh_end_session(void *);
extern void *dh_start_session();
extern char *dh_get_s_public(char *, int, void *);
extern int dh_get_s_shared(char *, int *, void *);
extern int dh_generate_shared(void *, char *);
extern int dh_hexstr_to_raw(char *, unsigned char *, int *);
extern void rc4_process_stream_to_buf(void *, const unsigned char *, unsigned char *, unsigned int);
extern void rc4_process_stream(void *, unsigned char *, unsigned int);
extern void *rc4_initstate(unsigned char *, int);
extern void rc4_destroystate(void *);

#endif /* HAVE_OPENSSL_ON */
