/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/h.h: prototypes header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: h.h,v 1.45 2002/04/08 16:00:46 rageircd Exp $
 */

#include "send.h"
#include "ircsprintf.h"
#include "fdlist.h"
#include "setup.h"

extern int R_do_dns, R_fin_dns, R_fin_dnsc, R_fail_dns, R_do_id, R_fin_id, R_fail_id;
extern int R_do_socks, R_no_socks, R_good_socks, R_bad_socks, R_except_socks;
#ifdef PROXYPORT
extern int R_do_proxy, R_no_proxy, R_good_proxy, R_bad_proxy, R_bad_proxyc;
#endif

/* Configuration */
extern ConfigItem_servinfo *config_servinfo;
extern ConfigItem_admin *config_admin;
extern ConfigItem_admin *config_admin_tail;
extern ConfigItem_class *config_class;
extern ConfigItem_allow *config_allow;
extern ConfigItem_password *config_password;
extern ConfigItem_oper *config_oper;
extern ConfigItem_uline *config_uline;
extern ConfigItem_link *config_link;
extern ConfigItem_ban *config_ban;
extern ConfigItem_listen *config_listen;
/*extern ConfigItem_except *config_except;*/
extern ConfigItem_vhost *config_vhost;
extern ConfigItem_set ConfigSet;

extern ConfigItem_class *find_class(char *);
extern ConfigItem_allow *find_allow(char *, char *);
extern ConfigItem_oper *find_oper(char *);
extern ConfigItem_uline *find_uline(char *);
extern ConfigItem_link *find_link(char *, char *);
extern ConfigItem_ban *find_ban(int, char *);
extern ConfigItem_listen *find_listen(char *, int);
/*extern ConfigItem_except *find_except(int, int, char *);*/
extern ConfigItem_vhost *find_vhost(char *);

extern void do_maskedhost(aClient *);

extern aMotd *load_file(aMotd **, struct tm *, char [34], char *, int);
extern void send_file(aClient *, aMotd **, struct tm *, char [34], char *, char *, int);
extern void free_file(aMotd **, char [34]);

/* HTM */
extern int lifesux;
extern int currlife;
extern int LCF;
extern int LRV;
extern int noisy_htm;
extern int HTMLOCK;

extern fdlist serv_fdlist;
extern fdlist busycli_fdlist;
extern fdlist default_fdlist;
extern fdlist oper_fdlist;
extern struct Counter Count;

extern time_t NOW;
extern time_t last_stat_save;
extern time_t nextconnect, nextdnscheck, nextping, timeofday;
extern aClient *client, me, *local[];
extern aChannel *channel;
extern struct stats *ircstp;
extern int bootopt;

extern char *canonize(char *);
extern void check_fdlists();
extern aChannel *find_channel(char *, aChannel *);
extern Link *find_channel_link(Link *, aChannel *);
extern aBan *is_banned(aClient *, aChannel *);
extern void remove_user_from_channel(aClient *, aChannel *);
extern void send_user_joins(aClient *, aClient *);
extern int can_send(aClient *, aChannel *);
extern int is_chan_op(aClient *, aChannel *);
extern int is_half_op(aClient *, aChannel *);
extern int is_chan_admin(aClient *, aChannel *);
extern int has_voice(aClient *, aChannel *);
extern int count_channels(aClient *);
extern char *pretty_mask(char *);
extern int check_channelname(aClient *, unsigned char *);
extern void add_invite(aClient *, aChannel *);
extern void del_invite(aClient *, aChannel *);
extern int add_ban_tolist(aClient *, aChannel *, char *, aBan **);
extern int del_ban_fromlist(aChannel *, char *, aBan **);
extern void channel_modes(aClient *, char *, char *, aChannel *);

extern aClient *find_chasing(aClient *, char *, int *);

extern int find_restrict(aClient *);
extern int rehash(aClient *, aClient *, int);

extern void clear_scache_hash_table(void);
extern char *find_or_add(char *);
extern void count_scache(int *, u_long *);
extern void list_scache(aClient *, aClient *, int, char **);

extern char *my_itoa(int);

#ifndef STATIC_MODULES
extern void _module_load(void);
extern void _module_unload(void);
#endif
extern aCommand *CommandHash[256];
extern void init_CommandHash();
extern void add_Command(char *, int (*)(), unsigned int, int, int, int, int, unsigned long);
extern void del_Command(char *, int (*)());

extern Hook *HookTable[MAXHOOKTYPES];
extern Hook *ghook;
extern void init_HookTable();
extern void addHook(int, int (*)(), void (*)());
extern void delHook(int, int (*)(), void (*)());

extern char *debugmode, *configfile, *sbrk0;
extern void get_sockhost(aClient *, char *);
extern char *rpl_str(int);
extern char *err_str(int);
extern char *getreply(int);
extern char *strerror(int);
extern int dgets(int, char *, int);
extern char *inetntoa(char *);
extern int dbufalloc, dbufblocks, debuglevel, errno, h_errno;
extern int highest_fd, debuglevel, debugtty, maxusersperchannel;
extern int readcalls, udpfd, resfd;
extern aClient *add_connection(aClient *, int);
extern int add_listener(ConfigItem_listen *);
extern void add_local_domain(char *, int);
extern int check_client(aClient *);
extern void close_connection(aClient *);
extern void close_listeners();
extern int connect_server(ConfigItem_link *, aClient *, struct hostent *);
extern void get_my_name(aClient *, char *, int);
extern int get_sockerr(aClient *);
extern int inetport(aClient *, char *, int);
extern void init_sys();
extern int read_message(time_t, fdlist *);
extern void report_error(char *, aClient *);
extern void set_non_blocking(int, aClient *);
extern int set_blocking(int);
extern int setup_ping(void);
extern void summon(aClient *, char *, char *, char *);
extern int unixport(aClient *, char *, int);
extern int utmp_open(void);
extern int utmp_read(int, char *, char *, char *, int);
extern int utmp_close(int);

extern void start_auth(aClient *);
extern void read_authports(aClient *);
extern void send_authports(aClient *);

#ifdef PROXYPORT
extern void start_proxy(aClient *);
extern void read_proxyports(aClient *);
extern void send_proxyports(aClient *);
#endif

extern void start_socks(aClient *);
extern void read_socks(aClient *);
extern void send_socksquery(aClient *);

extern void restart(char *);
extern void send_channel_modes(aClient *, aChannel *);
extern void server_reboot(void);
extern void terminate(void), write_pidfile(void);

extern int match(char *, char *);
extern char *collapse(char *);

extern int writecalls, writeb[];
extern int deliver_it(aClient *, char *, int);

extern int check_registered(aClient *);
extern int check_registered_user(aClient *);
extern char *check_string(char *);
extern char *make_nick_user_host(char *, char *, char *);
extern char *make_user_host(char *, char *);
extern char *get_client_name(aClient *, int);
extern char *get_client_host(aClient *);
extern char *myctime(time_t), *date(time_t);
extern int exit_client(aClient *, aClient *, aClient *, char *);
extern void initstats(void), tstats(aClient *, char *);
extern void serv_info(aClient *, char *);

extern int parse(aClient *, char *, char *);

extern int do_numeric(int, aClient *, aClient *, int, char **);
extern int hunt_server(aClient *, aClient *, char *, int, int, char **);
extern aClient *next_client(aClient *, char *);
extern aClient *next_client_double(aClient *, char *);

extern int send_lusers(aClient *, aClient *, int, char **);
extern void send_umode(aClient *, aClient *, int, int, char *);
extern void send_umode_out(aClient *, aClient *, int);
extern int del_silence(aClient *, char *);

extern void free_client(aClient *);
extern void free_link(Link *);
extern void free_chanmember(chanMember *);
extern void free_user(anUser *, aClient *);
extern void free_channel(aChannel *);
#ifdef HAVE_THROTTLE_ON
extern void free_throttle(throttle *);
extern void free_hashent(hashent *);
extern throttle *make_throttle();
extern hashent *make_hashent();
#endif
extern aChannel *make_channel();
extern Link *make_link(void);
extern chanMember *make_chanmember(void);
extern anUser *make_user(aClient *);
extern aServer *make_server(aClient *);
extern aClient *make_client(aClient *, aClient *);
extern chanMember *find_user_member(chanMember *, aClient *);
extern Link *find_str_link(Link *, char *);
extern void add_client_to_list(aClient *);
extern void checklist(void);
extern void remove_client_from_list(aClient *);

extern struct hostent *get_res(char *);
extern struct hostent *gethost_byaddr(char *, Link *);
extern struct hostent *gethost_byname(char *, Link *);
extern void flush_cache(void);
extern int init_resolver(int);
extern time_t timeout_query_list(time_t);
extern time_t expire_cache(time_t);
extern void del_queries(char *);

extern void clear_channel_hash_table(void);
extern void clear_client_hash_table(void);
extern int add_to_client_hash_table(char *, aClient *);
extern int del_from_client_hash_table(char *, aClient *);
extern int add_to_channel_hash_table(char *, aChannel *);
extern int del_from_channel_hash_table(char *, aChannel *);
extern aChannel *hash_find_channel(char *, aChannel *);
extern aClient *hash_find_client(char *, aClient *);
extern aClient *hash_find_nickserver(char *, aClient *);
extern aClient *hash_find_server(char *, aClient *);

extern void add_history(aClient *, int);
extern aClient *get_history(char *, time_t);
extern void off_history(aClient *);
extern void init_whowas();

extern int dopacket(aClient *, char *, int);
extern int client_dopacket(aClient *, char *, int);

extern void debug(int level, char *pattern, ...);
extern void ircdlog(char *, char *, ...);

#ifdef DEBUGMODE
extern void send_usage(aClient *, char *);
extern void send_listinfo(aClient *, char *);
extern void count_memory(aClient *, char *);
#endif

extern void send_list(aClient *, int);
extern aChannel *hash_get_chan_bucket(int);

#ifdef DUMP_DEBUG
extern FILE *dumpfp;
#endif

extern void remove_one_ip(unsigned long);
extern void clear_ip_hash_table(void);
extern void rehash_ip_hash(void);

extern u_long inet_addr(char *);

#ifdef FLUD
int check_for_flood();
void free_fluders();
void free_fludees();
#define MyFludConnect(x)	(((x)->fd >= 0) || ((x)->fd == -2))
#endif /* FLUD */

#ifdef ANTI_SPAMBOT
#define MIN_SPAM_NUM 5
#define MIN_SPAM_TIME 60
#endif

extern void addEvent(const char *, void (*)(), void *, time_t, int);
extern void delEvent(void (*)(), void *);
extern void checkEvent(void);
extern time_t next_checkEvent(void);
extern void show_events(aClient *);

#define MAXKILLS 20
extern void count_watch_memory(int *, u_long *);
extern void clear_watch_hash_table(void);
extern int add_to_watch_hash_table(char *, aClient *);
extern int del_from_watch_hash_table(char *, aClient *);
extern int hash_check_watch(aClient *, int);
extern int hash_del_watch_list(aClient  *);
extern aWatch *hash_get_watch(char *);
#define MAXWATCH 128

extern time_t atime(char *);
extern void strip(char *);

#include "find.h"
