/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/struct.h: structures header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: struct.h,v 1.91 2002/04/08 16:00:46 rageircd Exp $
 */

#ifndef	__struct_include__
#define __struct_include__

#include "config.h"
#ifndef CONFIG_H_LEVEL_13
#error Incorrect config.h for this revision of ircd.
#endif

#include "conf2.h"
#include "queue.h"
#include "setup.h"
#include <stdio.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#ifdef HAVE_STDDEF_H
#include <stddef.h>
#endif
#ifdef ORATIMING
#include <sys/time.h>
#endif

#define REPORT_DO_DNS_		":%s NOTICE AUTH :*** Looking up your hostname...\n"
#define REPORT_FIN_DNS_		":%s NOTICE AUTH :*** Found your hostname\n"
#define REPORT_FIN_DNSC_	":%s NOTICE AUTH :*** Found your hostname (cached)\n"
#define REPORT_FAIL_DNS_	":%s NOTICE AUTH :*** Couldn't resolve your hostname; using IP address instead\n"
#define REPORT_DO_ID_		":%s NOTICE AUTH :*** Checking ident...\n"
#define REPORT_FIN_ID_		":%s NOTICE AUTH :*** Received identd responce\n"
#define REPORT_FAIL_ID_		":%s NOTICE AUTH :*** No identd responce; username prefixed with ~\n"
extern char REPORT_DO_DNS[256], REPORT_FIN_DNS[256], REPORT_FIN_DNSC[256], REPORT_FAIL_DNS[256];
extern char REPORT_DO_ID[256], REPORT_FIN_ID[256], REPORT_FAIL_ID[256];

#include "hash.h"
#include "openssl.h"

typedef struct Client aClient;
typedef struct Channel aChannel;
typedef struct User anUser;
typedef struct Server aServer;
typedef struct SLink Link;
typedef struct ChanLink chanMember;
typedef struct SMode Mode;
typedef struct Watch aWatch;
typedef struct Ban aBan;
typedef struct ListOptions LOpts;
typedef struct Motd aMotd;
typedef struct Command aCommand;
typedef struct Whowas aWhowas;
typedef struct Ctab aCtab;
typedef struct ip_entry IP_ENTRY;
typedef struct moduletable ModuleT;
typedef struct module Module;
typedef struct event Event;
typedef struct hook Hook;
typedef struct scache_entry SCACHE;
#ifdef HAVE_THROTTLE_ON
typedef struct hashent_t hashent;
typedef struct hashent_list_t hashent_list;
typedef struct hash_table_t hash_table;
typedef struct throttle_t throttle;
#endif

typedef long ts_val;

#include "dbuf.h"		/* THIS REALLY SHOULDN'T BE HERE!!! --msa */

#define	HOSTLEN			63	/* Length of hostname.  Updated to comply with RFC1123 */
#define HOSTIPLEN		15	/* Length of dotted quad form of IP */
#define	NICKLEN			30	/* Length of nickname */
#define MAX_DATE_STRING		32	/* Length of date string */
#define	USERLEN			10	/* Length of username */
#define	REALLEN			50	/* Length of realname */
#define	TOPICLEN		307	/* Length of topic */
#define	KILLLEN			400	/* Length of kill */
#define	CHANNELLEN		32	/* Length of channel name */
#define	PASSWDLEN		63	/* Length of passwords */
#define	KEYLEN			23	/* Length of channel key */
#define	BUFSIZE			512	/* Length of internal buffers */
#define MOTDLINELEN		90	/* Length of motd lines */
#define	MAXRECIPIENTS		20
#define	MAXBANS			45
#define	MAXSILES		5
#define MAXSILELENGTH		128
#define MAXDCCALLOW		5
#define MAXACCEPTS		10
#define MAXWHOREPLIES		200
#define MAXEVENTS		50
#define MAXHOOKTYPES		10
#define LOADCFREQ		5	/* HTM: every 5s */
#define LOADRECV		40	/* HTM: 40k/s */
#define LUSERS_CACHE_TIME	180
#define SCACHE_HASH_SIZE	257	/* SCACHE */
#define	USERHOST_REPLYLEN	(NICKLEN + HOSTLEN + USERLEN + 5)

#define DCC_LINK_ME	0x01	/* This is my dcc allow */
#define DCC_LINK_REMOTE 0x02    /* I need to remove these dcc allows from these clients when I die */

#define ACCEPT_LINK_ME	0x01	/* This is my message accept list */
#define ACCEPT_LINK_REMOTE 0x02	/* I need to remove these accpets from these clients when I die */

#ifdef HAVE_THROTTLE_ON
#define HASH_FL_NOCASE	0x1
#define HASH_FL_STRING	0x2
#endif

/* 'offsetof' is defined in ANSI-C. The following definition
 * is not absolutely portable (I have been told), but so far
 * it has worked on all machines I have needed it. The type
 * should be size_t but... --msa
 */

#ifndef offsetof
#define	offsetof(t,m) (int)((&((t *)0L)->m))
#endif

#define	elementsof(x) (sizeof(x)/sizeof(x[0]))
#define WHO_HOPCOUNT(s, a) ((IsULine((a)) && !IsOper((s))) ? 0 : a->hopcount)

/* flags for bootup options (command line flags) */

#define	BOOT_CONSOLE		1
#define	BOOT_QUICK		2
#define	BOOT_DEBUG		4
#define	BOOT_INETD		8
#define	BOOT_TTY		16
#define	BOOT_OPER		32
#define	BOOT_AUTODIE		64
#define BOOT_STDERR		128

#define	STAT_LOG		-6
#define	STAT_MASTER		-5
#define	STAT_CONNECTING		-4
#define	STAT_HANDSHAKE		-3
#define	STAT_ME			-2
#define	STAT_UNKNOWN		-1
#define	STAT_SERVER		0
#define	STAT_CLIENT		1

/* status macros. */

#define	IsRegisteredUser(x)	((x)->status == STAT_CLIENT)
#define	IsRegistered(x)		((x)->status >= STAT_SERVER)
#define	IsConnecting(x)		((x)->status == STAT_CONNECTING)
#define	IsHandshake(x)		((x)->status == STAT_HANDSHAKE)
#define	IsMe(x)			((x)->status == STAT_ME)
#define	IsUnknown(x)		((x)->status == STAT_UNKNOWN || (x)->status == STAT_MASTER)
#define	IsServer(x)		((x)->status == STAT_SERVER)
#define	IsClient(x)		((x)->status == STAT_CLIENT)
#define	IsLog(x)		((x)->status == STAT_LOG)

#define	SetMaster(x)		((x)->status = STAT_MASTER)
#define	SetConnecting(x)	((x)->status = STAT_CONNECTING)
#define	SetHandshake(x)		((x)->status = STAT_HANDSHAKE)
#define	SetMe(x)		((x)->status = STAT_ME)
#define	SetUnknown(x)		((x)->status = STAT_UNKNOWN)
#define	SetServer(x)		((x)->status = STAT_SERVER)
#define	SetClient(x)		((x)->status = STAT_CLIENT)
#define	SetLog(x)		((x)->status = STAT_LOG)

#define	FLAGS_PINGSENT		0x000001	/* Unreplied ping sent */
#define	FLAGS_DEADSOCKET	0x000002	/* Local socket is dead--Exiting soon */
#define	FLAGS_KILLED		0x000004	/* Prevents "QUIT" from being sent for this */
#define	FLAGS_BLOCKED		0x000008	/* socket is in a blocked condition */
#define	FLAGS_CLOSING		0x000010	/* set when closing to suppress errors */
#define	FLAGS_LISTEN		0x000020	/* used to mark clients which we listen() on */
#define	FLAGS_CHKACCESS		0x000040	/* ok to check clients access if set */
#define	FLAGS_DOINGDNS		0x000080	/* client is waiting for a DNS response */
#define	FLAGS_AUTH		0x000100	/* client is waiting on rfc931 response */
#define	FLAGS_WRAUTH		0x000200	/* set if we havent writen to ident server */
#define	FLAGS_LOCAL		0x000400	/* set for local clients */
#define	FLAGS_GOTID		0x000800	/* successful ident lookup achieved */
#define	FLAGS_DOID		0x001000	/* I-lines say must use ident return */
#define	FLAGS_NONL		0x002000	/* No \n in buffer */
#define FLAGS_NORMALEX		0x004000	/* Client exited normally */
#define FLAGS_SENDQEX		0x008000	/* Sendq exceeded */
#define FLAGS_IPHASH		0x010000	/* iphashed this client */
#define FLAGS_ULINE		0x020000	/* client is U-lined */
#define FLAGS_USERBURST		0x040000	/* server in nick/channel netburst */
#define FLAGS_TOPICBURST	0x080000	/* server in topic netburst */
#define FLAGS_BURST		(FLAGS_USERBURST | FLAGS_TOPICBURST)
#define FLAGS_SOBSENT		0x100000	/* we've sent an SOB, just have to send an EOB */
#define FLAGS_EOBRECV		0x200000	/* we're waiting on an EOB */
#define FLAGS_BAD_DNS		0x400000	/* spoofer-guy */
#define FLAGS_SERV_NEGO		0x800000	/* This is a server that has passed connection tests, but is a stat < 0 for handshake purposes */
#define FLAGS_RC4IN		0x1000000	/* This link is rc4 encrypted. */
#define FLAGS_RC4OUT		0x2000000	/* This link is rc4 encrypted. */
#define FLAGS_ZIPPED_IN		0x4000000	/* This link is gzipped. */
#define FLAGS_ZIPPED_OUT	0x8000000	/* This link is gzipped. */
#define FLAGS_MAP		0x10000000	/* Map */
#define FLAGS_SEENDCCNOTICE	0x20000000	/* Seen DCC Notice */
#define FLAGS_OPENSSL		0x40000000	/* Client OpenSSL connections */

/* DKEY */
#define DKEY_GOTIN		0x01
#define DKEY_GOTOUT		0x02
#define DKEY_DONE(x)		(((x) & (DKEY_GOTIN|DKEY_GOTOUT)) == (DKEY_GOTIN|DKEY_GOTOUT))

/* Capabilities of the ircd or clients */
#define CAPAB_NOQUIT		0x0000004	/* server supports NOQUIT */
#define CAPAB_BURST		0x0000008	/* server supports BURST */
#define CAPAB_UNCONN		0x0000010	/* server supports UNCONNECT */
#define CAPAB_ZIP		0x0000020	/* server supports ZIP */
#define CAPAB_DOZIP		0x0000040	/* output to this link shall be zipped */
#define CAPAB_DKEY		0x0000080	/* server supports DKEY */
#define CAPAB_DODKEY		0x0000100	/* output to this link shall be encrypted */
#define CAPAB_SSJOIN		0x0000200	/* server supports SSJOIN */
#define CAPAB_SSJv2		0x0000400	/* server supports SSJv2 */
#define CAPAB_NICKIP		0x0000800	/* server supports NICKIP */
#define CAPAB_SENDVHOST		0x0001000	/* server supports SENDVHOST */

#define SetNoQuit(x)		((x)->capabilities |= CAPAB_NOQUIT)
#define IsNoQuit(x)		((x)->capabilities & CAPAB_NOQUIT)

#define SetBurst(x)		((x)->capabilities |= CAPAB_BURST)
#define IsBurst(x)		((x)->capabilities & CAPAB_BURST)

#define SetUnconnect(x)		((x)->capabilities |= CAPAB_UNCONN)
#define IsUnconnect(x)		((x)->capabilities & CAPAB_UNCONN)

#define SetDKEY(x)		((x)->capabilities |= CAPAB_DKEY)
#define CanDoDKEY(x)		((x)->capabilities & CAPAB_DKEY)
#define WantDKEY(x)		((x)->capabilities & CAPAB_DODKEY)

#define SetZipCapable(x)	((x)->capabilities |= CAPAB_ZIP)
#define IsZipCapable(x)		((x)->capabilities & CAPAB_ZIP)
#define DoZipThis(x)		((x)->capabilities & CAPAB_DOZIP)

#define SetSSJOIN(x)		((x)->capabilities |= CAPAB_SSJOIN)
#define IsSSJOIN(x)		((x)->capabilities & CAPAB_SSJOIN)

#define SetSSJv2(x)		((x)->capabilities |= CAPAB_SSJv2)
#define IsSSJv2(x)		((x)->capabilities & CAPAB_SSJv2)

#define SetNICKIP(x)		((x)->capabilities |= CAPAB_NICKIP)
#define IsNICKIP(x)		((x)->capabilities & CAPAB_NICKIP)

#define SetSENDVHOST(x)		((x)->capabilities |= CAPAB_SENDVHOST)
#define IsSENDVHOST(x)		((x)->capabilities & CAPAB_SENDVHOST)

/* flag macros. */
#define IsULine(x)		((x)->flags & FLAGS_ULINE)

/* User Modes */
#define UMODE_o		0x00001		/* umode +o - Oper */
#define UMODE_i		0x00002		/* umode +i - Invisible */
#define UMODE_w		0x00004		/* umode +w - Get wallops */
#define UMODE_s		0x00008		/* umode +s - Server notices */
#define UMODE_c		0x00010		/* umode +c - Client connections/exits */
#define UMODE_r		0x00020		/* umode +r - registered nick */
#define UMODE_k		0x00040		/* umode +k - Server kill messages */
#define UMODE_f		0x00080		/* umode +f - Server flood messages */
#define UMODE_y		0x00100		/* umode +y - Stats/links */
#define UMODE_d		0x00200		/* umode +d - Debug info */
#define UMODE_g		0x00400		/* umode +g - Globops */
#define UMODE_b		0x00800		/* umode +b - Chatops */
#define UMODE_a		0x01000		/* umode +a - Services Admin */
#define UMODE_A		0x02000		/* umode +A - Server Admin */
#define UMODE_n		0x04000		/* umode +n - Routing Notices */
#define UMODE_h		0x08000		/* umode +h - Helper */
#define UMODE_m		0x10000		/* umode +m - spambot notices */
#define UMODE_R		0x20000		/* umode +R - Routing staff */
#define UMODE_e		0x40000		/* umode +e - oper notices for the above +D */
#define UMODE_z		0x80000		/* umode +z - masked hostnames */
#define UMODE_N		0x100000	/* umode +N - Net Admin */
#define UMODE_C		0x200000	/* umode +C - Server CoAdmin */
#define UMODE_p		0x400000	/* umode +p - private */
#define UMODE_S		0x800000	/* umode +S - services protection */

/* for sendto_ops_lev */

#define CCONN_LEV	1
#define REJ_LEV		2
#define SKILL_LEV	3
#define SPY_LEV		4
#define DEBUG_LEV	5
#define FLOOD_LEV 	6
#define SPAM_LEV 	7
#define DCCSEND_LEV	8

#define	SEND_UMODES	(UMODE_i|UMODE_o|UMODE_w|UMODE_r|UMODE_a|UMODE_A|UMODE_h|UMODE_R|UMODE_z|UMODE_N|UMODE_C|UMODE_p|UMODE_S)
#define ALL_UMODES	(SEND_UMODES|UMODE_s|UMODE_c|UMODE_k|UMODE_f|UMODE_y|UMODE_d|UMODE_g|UMODE_b|UMODE_n|UMODE_h|UMODE_m|UMODE_e)
#define OPER_UMODES	(UMODE_o|UMODE_w|UMODE_s|UMODE_y|UMODE_d|UMODE_g|UMODE_n|UMODE_h|UMODE_y)

#define	FLAGS_ID (FLAGS_DOID|FLAGS_GOTID)

#define	IsOper(x)		((x)->umode & UMODE_o)
#define	IsInvisible(x)		((x)->umode & UMODE_i)
#define IsARegNick(x)		((x)->umode & UMODE_r)
#define IsRegNick(x)		((x)->umode & UMODE_r)
#define IsSAdmin(x)		((x)->umode & UMODE_a)
#define IsAdmin(x)		((x)->umode & UMODE_A)
#define IsNetAdmin(x)		((x)->umode & UMODE_N)
#define IsCoAdmin(x)		((x)->umode & UMODE_C)
#define IsUmodef(x)		((x)->umode & UMODE_f)
#define IsUmodec(x)		((x)->umode & UMODE_c)
#define IsUmodey(x)		((x)->umode & UMODE_y)
#define IsUmoded(x)		((x)->umode & UMODE_d)
#define IsUmodeb(x)		((x)->umode & UMODE_b)
#define IsUmoden(x)		((x)->umode & UMODE_n)
#define IsUmodem(x)		((x)->umode & UMODE_m)
#define IsUmodeh(x)		((x)->umode & UMODE_h)
#define IsUmodee(x)		((x)->umode & UMODE_e)
#define IsUmodep(x)		((x)->umode & UMODE_p)
#define IsMasked(x)		((x)->umode & UMODE_z)
#define IsRouting(x)		((x)->umode & UMODE_R)
#define IsServices(x)		((x)->umode & UMODE_S)
#define	IsPerson(x)		((x)->user && IsClient(x))
#define	IsPrivileged(x)		(IsOper(x) || IsServer(x))
#define	SendWallops(x)		((x)->umode & UMODE_w)
#define	SendServNotice(x)	((x)->umode & UMODE_s)
#define SendCConnNotice(x)	((x)->umode & UMODE_c)
#define SendRejNotice(x)	((x)->umode & UMODE_c)
#define SendSkillNotice(x)	((x)->umode & UMODE_k)
#define SendSpyNotice(x)	((x)->umode & UMODE_y)
#define SendDCCNotice(x)	((x)->umode & UMODE_e)
#define SendFloodNotice(x)	((x)->umode & UMODE_f)
#define SendSpamNotice(x)	((x)->umode & UMODE_m)
#define SendDebugNotice(x)	((x)->umode & UMODE_d)
#define SendGlobops(x)		((x)->umode & UMODE_g)
#define SendChatops(x)		((x)->umode & UMODE_b)
#define SendRnotice(x)		((x)->umode & UMODE_n)
#define	IsListening(x)		((x)->flags & FLAGS_LISTEN)
#define	DoAccess(x)		((x)->flags & FLAGS_CHKACCESS)
#define	IsLocal(x)		((x)->flags & FLAGS_LOCAL)
#define IsRestricted(x)		((x)->flags & FLAGS_RESTRICTED)
#define	IsDead(x)		((x)->flags & FLAGS_DEADSOCKET)
#define	SetOper(x)		((x)->umode |= UMODE_o)
#define SetRegNick(x)		((x)->umode |= UMODE_r)
#define SetRouting(x)		((x)->umode |= UMODE_R)
#define SetSAdmin(x)		((x)->umode |= UMODE_a)
#define SetAdmin(x)		((x)->umode |= UMODE_A)
#define SetNetAdmin(x)		((x)->umode |= UMODE_N)
#define SetCoAdmin(x)		((x)->umode |= UMODE_C)
#define	SetInvisible(x)		((x)->umode |= UMODE_i)
#define	SetWallops(x)  		((x)->umode |= UMODE_w)
#define SetServices(x)		((x)->umode |= UMODE_S)
#define SetRestricted(x)	((x)->flags |= FLAGS_RESTRICTED)
#define	SetDNS(x)		((x)->flags |= FLAGS_DOINGDNS)
#define	DoingDNS(x)		((x)->flags & FLAGS_DOINGDNS)
#define	SetAccess(x)		((x)->flags |= FLAGS_CHKACCESS)
#define	DoingAuth(x)		((x)->flags & FLAGS_AUTH)
#define	NoNewLine(x)		((x)->flags & FLAGS_NONL)

#define SetNegoServer(x)	((x)->flags |= FLAGS_SERV_NEGO)
#define IsNegoServer(x)		((x)->flags & FLAGS_SERV_NEGO)
#define ClearNegoServer(x)	((x)->flags &= ~FLAGS_SERV_NEGO)
#define IsRC4OUT(x)		((x)->flags & FLAGS_RC4OUT)
#define SetRC4OUT(x)		((x)->flags |= FLAGS_RC4OUT)
#define IsRC4IN(x)		((x)->flags & FLAGS_RC4IN)
#define SetRC4IN(x)		((x)->flags |= FLAGS_RC4IN)
#define RC4EncLink(x)		(((x)->flags & (FLAGS_RC4IN|FLAGS_RC4OUT)) == (FLAGS_RC4IN|FLAGS_RC4OUT))

#define ZipIn(x)		((x)->flags & FLAGS_ZIPPED_IN)
#define SetZipIn(x)		((x)->flags |= FLAGS_ZIPPED_IN)
#define ZipOut(x)		((x)->flags & FLAGS_ZIPPED_OUT)
#define SetZipOut(x)		((x)->flags |= FLAGS_ZIPPED_OUT)

#define ClearSAdmin(x)		((x)->umode &= ~UMODE_a)
#define ClearAdmin(x)		((x)->umode &= ~UMODE_A)
#define ClearNetAdmin(x)	((x)->umode &= ~UMODE_N)
#define ClearCoAdmin(x)		((x)->umode &= ~UMODE_C)
#define ClearUmodef(x)		((x)->umode &= ~UMODE_f)
#define ClearUmodem(x)		((x)->umode &= ~UMODE_m)
#define ClearUmodec(x)		((x)->umode &= ~UMODE_c)
#define ClearUmodey(x)		((x)->umode &= ~UMODE_y)
#define ClearUmoded(x)		((x)->umode &= ~UMODE_d)
#define ClearUmodeb(x)		((x)->umode &= ~UMODE_b)
#define ClearUmoden(x)		((x)->umode &= ~UMODE_n)
#define ClearUmodeh(x)		((x)->umode &= ~UMODE_h)
#define ClearUmodee(x)		((x)->umode &= ~UMODE_e)
#define ClearUmodep(x)		((x)->umode &= ~UMODE_p)
#define ClearMasked(x)		((x)->umode &= ~UMODE_z)
#define ClearRouting(x)		((x)->umode &= ~UMODE_R)
#define ClearServices(x)	((x)->umode &= ~UMODE_S)
#define	ClearOper(x)		((x)->umode &= ~UMODE_o)
#define	ClearInvisible(x)	((x)->umode &= ~UMODE_i)
#define	ClearWallops(x)		((x)->umode &= ~UMODE_w)
#define	ClearDNS(x)		((x)->flags &= ~FLAGS_DOINGDNS)
#define	ClearAuth(x)		((x)->flags &= ~FLAGS_AUTH)
#define	ClearAccess(x)		((x)->flags &= ~FLAGS_CHKACCESS)

/* defined operator access levels */ 
#define OFLAG_REHASH	0x00000001  /* Oper can /rehash server */
#define OFLAG_DIE	0x00000002  /* Oper can /die the server */
#define OFLAG_RESTART	0x00000004  /* Oper can /restart the server */
/* access 0x00000008 available */
#define OFLAG_GLOBOP	0x00000020  /* Oper can send /GlobOps */
#define OFLAG_WALLOP	0x00000040  /* Oper can send /WallOps */
#define OFLAG_LOCOP	0x00000080  /* Oper can send /LocOps */
#define OFLAG_LROUTE	0x00000100  /* Oper can do local routing */
#define OFLAG_GROUTE	0x00000200  /* Oper can do global routing */
#define OFLAG_LKILL	0x00000400  /* Oper can do local kills */
#define OFLAG_GKILL	0x00000800  /* Oper can do global kills */
#define OFLAG_KLINE	0x00001000  /* Oper can /kline users */
#define OFLAG_UNKLINE	0x00002000  /* Oper can /unkline users */
#define OFLAG_LNOTICE	0x00004000  /* Oper can send local serv notices */
#define OFLAG_GNOTICE	0x00008000  /* Oper can send global notices */
#define OFLAG_ADMIN	0x00010000  /* Admin */ 
#define OFLAG_UMODEC	0x00020000  /* Oper can set umode +c */
#define OFLAG_UMODEF	0x00040000  /* Oper can set umode +f */
#define OFLAG_SADMIN    0x00080000  /* Oper can be a services admin */
#define OFLAG_ZLINE	0x00100000  /* Oper can use /zline and /unzline */
/* access 0x00200000 available */
/* access 0x00400000 available */
/* access 0x00800000 available */
#define OFLAG_RSTAFF    0x01000000  /* Oper can view server IPs (routing staff) */
#define OFLAG_NETADMIN  0x02000000  /* Net Admin */
#define OFLAG_COADMIN   0x04000000  /* Server CoAdmin */

#define OPCanZline(x)	((x)->oflag & OFLAG_SADMIN)
#define OPCanRehash(x)	((x)->oflag & OFLAG_REHASH)
#define OPCanDie(x)	((x)->oflag & OFLAG_DIE)
#define OPCanRestart(x)	((x)->oflag & OFLAG_RESTART)
#define OPCanGlobOps(x)	((x)->oflag & OFLAG_GLOBOP)
#define OPCanWallOps(x)	((x)->oflag & OFLAG_WALLOP)
#define OPCanLocOps(x)	((x)->oflag & OFLAG_LOCOP)
#define OPCanLRoute(x)	((x)->oflag & OFLAG_LROUTE)
#define OPCanGRoute(x)	((x)->oflag & OFLAG_GROUTE)
#define OPCanLKill(x)	((x)->oflag & OFLAG_LKILL)
#define OPCanGKill(x)	((x)->oflag & OFLAG_GKILL)
#define OPCanKline(x)	((x)->oflag & OFLAG_KLINE)
#define OPCanUnKline(x)	((x)->oflag & OFLAG_UNKLINE)
#define OPCanLNotice(x)	((x)->oflag & OFLAG_LNOTICE)
#define OPCanGNotice(x)	((x)->oflag & OFLAG_GNOTICE)
#define OPIsAdmin(x)	((x)->oflag & OFLAG_ADMIN)
#define OPIsNetAdmin(x)	((x)->oflag & OFLAG_NETADMIN)
#define OPIsCoAdmin(x)	((x)->oflag & OFLAG_COADMIN)
#define OPIsSAdmin(x)	((x)->oflag & OFLAG_SADMIN)
#define OPIsRStaff(x)   ((x)->oflag & OFLAG_RSTAFF)
#define OPCanUModeC(x)	((x)->oflag & OFLAG_UMODEC)
#define OPCanUModeF(x)	((x)->oflag & OFLAG_UMODEF)

#define OPSetRehash(x)	((x)->oflag |= OFLAG_REHASH)
#define OPSetDie(x)	((x)->oflag |= OFLAG_DIE)
#define OPSetRestart(x)	((x)->oflag |= OFLAG_RESTART)
#define OPSetGlobOps(x)	((x)->oflag |= OFLAG_GLOBOP)
#define OPSetWallOps(x)	((x)->oflag |= OFLAG_WALLOP)
#define OPSetLocOps(x)	((x)->oflag |= OFLAG_LOCOP)
#define OPSetLRoute(x)	((x)->oflag |= OFLAG_LROUTE)
#define OPSetGRoute(x)	((x)->oflag |= OFLAG_GROUTE)
#define OPSetLKill(x)	((x)->oflag |= OFLAG_LKILL)
#define OPSetGKill(x)	((x)->oflag |= OFLAG_GKILL)
#define OPSetKline(x)	((x)->oflag |= OFLAG_KLINE)
#define OPSetUnKline(x)	((x)->oflag |= OFLAG_UNKLINE)
#define OPSetLNotice(x)	((x)->oflag |= OFLAG_LNOTICE)
#define OPSetGNotice(x)	((x)->oflag |= OFLAG_GNOTICE)
#define OPSSetAdmin(x)	((x)->oflag |= OFLAG_ADMIN)
#define OPSetNetAdmin(x) ((x)->oflag |= OFLAG_NETADMIN)
#define OPSetCoAdmin(x) ((x)->oflag |= OFLAG_COADMIN)
#define OPSetSAdmin(x)	((x)->oflag |= OFLAG_SADMIN)
#define OPSetRStaff(x)  ((x)->oflag |= OFLAG_RSTAFF)
#define OPSetUModeC(x)	((x)->oflag |= OFLAG_UMODEC)
#define OPSetUModeF(x)	((x)->oflag |= OFLAG_UMODEF)
#define OPSetZLine(x)	((x)->oflag |= OFLAG_ZLINE)

#define OPClearRehash(x)	((x)->oflag &= ~OFLAG_REHASH)
#define OPClearDie(x)		((x)->oflag &= ~OFLAG_DIE)  
#define OPClearRestart(x)	((x)->oflag &= ~OFLAG_RESTART)
#define OPClearGlobOps(x)	((x)->oflag &= ~OFLAG_GLOBOP)
#define OPClearWallOps(x)	((x)->oflag &= ~OFLAG_WALLOP)
#define OPClearLocOps(x)	((x)->oflag &= ~OFLAG_LOCOP)
#define OPClearLRoute(x)	((x)->oflag &= ~OFLAG_LROUTE)
#define OPClearGRoute(x)	((x)->oflag &= ~OFLAG_GROUTE)
#define OPClearLKill(x)		((x)->oflag &= ~OFLAG_LKILL)
#define OPClearGKill(x)		((x)->oflag &= ~OFLAG_GKILL)
#define OPClearKline(x)		((x)->oflag &= ~OFLAG_KLINE)
#define OPClearUnKline(x)	((x)->oflag &= ~OFLAG_UNKLINE)
#define OPClearLNotice(x)	((x)->oflag &= ~OFLAG_LNOTICE)
#define OPClearGNotice(x)	((x)->oflag &= ~OFLAG_GNOTICE)
#define OPClearAdmin(x)		((x)->oflag &= ~OFLAG_ADMIN)
#define OPClearNetAdmin(x)	((x)->oflag &= ~OFLAG_NETADMIN)
#define OPClearCoAdmin(x)	((x)->oflag &= ~OFLAG_COADMIN)
#define OPClearSAdmin(x)	((x)->oflag &= ~OFLAG_SADMIN)
#define OPClearRStaff(x) 	((x)->oflag &= ~OFLAG_RSTAFF)
#define OPClearUModeC(x)	((x)->oflag &= ~OFLAG_UMODEC)
#define OPClearUModeF(x)	((x)->oflag &= ~OFLAG_UMODEF)
#define OPClearZLine(x)		((x)->oflag &= ~OFLAG_ZLINE)

/* defined debugging levels */
#define	DEBUG_FATAL  0
#define	DEBUG_ERROR  1 /* report_error() and other errors that are found */
#define	DEBUG_NOTICE 3
#define	DEBUG_DNS    4 /* used by all DNS related routines - a *lot* */
#define	DEBUG_INFO   5 /* general usful info */
#define	DEBUG_NUM    6 /* numerics */
#define	DEBUG_SEND   7 /* everything that is sent out */
#define	DEBUG_DEBUG  8 /* anything to do with debugging, ie unimportant :) */
#define	DEBUG_MALLOC 9 /* malloc/free calls */
#define	DEBUG_LIST  10 /* debug list use */

/* defines for curses in client */
#define	DUMMY_TERM	0
#define	CURSES_TERM	1
#define	TERMCAP_TERM	2

/* Hook definitions */
#define HOOKTYPE_LOCAL_CONNECT	0
#define HOOKTYPE_REHASH		1
#define HOOKTYPE_OPER		2

struct Counter {
	int server;		/* servers */
	int myserver;		/* my servers */
	int myulined;		/* my ulined servers */
	int oper;		/* opers */
	int chan;		/* channels */
	int local;		/* local clients */
	int total;		/* total clients */
	int invisi;		/* invisible clients */
	int unknown;		/* unknown connections */
	int max_loc;		/* max local clients */
	int max_tot;		/* max global clients */
	ts_val start;		/* when we started collecting info */
	u_long today;		/* client connections today */
	ts_val day;		/* when today started */
	u_long weekly;		/* connections this week */
	ts_val week;		/* when this week started */
	u_long monthly;		/* connections this month */
	ts_val month;		/* when this month started */
	u_long yearly;		/* connections this year (this is gunna be be big!) */
	ts_val year;		/* when this year started */
};

struct Motd {
	char line[MOTDLINELEN];
	aMotd *next;
};

struct Command {
	aCommand *next, *prev;
	char *cmd;
	int (*func)();
	unsigned int count;
	int parameters;
	int flags;
	int unreg_use;
	int reset_idle;
	unsigned long bytes;
};

struct Whowas {
	int hashv;
	char name[NICKLEN + 1];
	char username[USERLEN + 1];
	char hostname[HOSTLEN + 1];
	char maskedhost[HOSTLEN + 1];
	char realname[REALLEN + 1];
	char *servername;
	time_t logoff;
	aClient *online;
	aWhowas *next, *prev;
	aWhowas *cnext, *cprev;
};

/* channel mode and flag tab structure */
struct Ctab {
	u_int mode;
	char flag;
	int halfop;
	int param;
};

/* iphash entry structure */
struct ip_entry {
	unsigned long ip;
	int count;
	IP_ENTRY *next;
};

/* module table structure */
struct moduletable {
	ModuleT *next, *prev;
	Module *header;
};

/* module structure */
struct module {
	char *name;
	char *info;
	int version;
	void *ptr;
	void (*unloadfunc)();
};

/* event structure */
struct event {
	const char *name;
	void (*func)();
	void *arg;
	time_t frequency;
	time_t when;
	Event *next;
	int active;
	int loop;
};

/* hook structure */
struct hook {
	int (*ifunc)();
	void (*vfunc)();
	Hook *next, *prev;
};

/* sache structure */
struct scache_entry {
	char name[HOSTLEN + 1];
	SCACHE *next;
};

#ifdef HAVE_THROTTLE_ON
struct hashent_t {
	void *ent;
	SLIST_ENTRY(hashent_t) lp;
};

struct hash_table_t {
	int size;
	hashent_list *table;
	size_t keyoffset;
	size_t keylen;
	int flags;
	int (*cmpfunc)(void *, void *);
};

struct throttle_t {
	char addr[HOSTIPLEN + 1];
	int conns;
	time_t first;
	time_t last;
	time_t zline_start;
	int stage;
	int re_zlines;
	LIST_ENTRY(throttle_t) lp;
};
#endif

/* Client structures */
struct User {
	Link *channel;			/* chain of channel pointer blocks */
	Link *invited;			/* chain of invite pointer blocks */
	char *away;			/* pointer to away message */
	time_t last;
	int joined;			/* number of channels joined */
	char username[USERLEN + 1];	/* username */
	char host[HOSTLEN + 1];		/* hostname */
	char *server;			/* pointer to scached server name */
#ifdef OS_SOLARIS2
	uint_t servicestamp;
#else
	u_int32_t servicestamp;
#endif
	char *maskedhost;		/* masked host */
	Link *silence;			/* chain of silenced users */
	LOpts *lopt;			/* Saved /list options */
	aWhowas *whowas;		/* Pointers to whowas structs */
        Link *dccallow;			/* chain of dcc send allowed users */
	Link *accept;			/* accept list for umode +p */
	time_t last_accept_notice;	/* time since the last accept notice was given */
	time_t first_message_time;
	int number_of_messages;
	int flood_notice;
};

struct Server {
   char *up;				/* Pointer to scache name */
   char bynick[NICKLEN + 1];
   char byuser[USERLEN + 1];
   char byhost[HOSTLEN + 1];
   int dkey_flags;			/* dkey flags */
#ifdef HAVE_OPENSSL_ON
   void *sessioninfo_in;		/* pointer to opaque sessioninfo structure */
   void *sessioninfo_out;		/* pointer to opaque sessioninfo structure */
   void *rc4_in;			/* etc */
   void *rc4_out;			/* etc */
#endif
   void *zip_out;			/* inbound zipped data */
   void *zip_in;			/* outbound zipped data */
   ConfigItem_link *conf;		/* config */
};

struct Client {
	struct Client *next, *prev, *hnext;
	anUser *user;			/* ...defined, if this is a User */
	aServer *serv;			/* ...defined, if this is a server */
	time_t lasttime;		/* ...should be only LOCAL clients? --msa */
	time_t firsttime;		/* time client was created */
	time_t since;			/* last time we parsed something */
	ts_val tsinfo;			/* TS on the nick, SVINFO on servers */
	long flags;			/* client flags */
	long flags2;			/* client flags-II */
	long umode;			/* We can illeviate overflow this way */
	aClient *from;			/* == self, if Local Client, *NEVER* NULL! */
	aClient *uplink;		/* this client's uplink to the network */
	int fd;				/* >= 0, for local clients */
#ifdef HAVE_OPENSSL_ON
	struct SSL *openssl;		/* openssl socket */
#endif
  	int hopcount;			/* number of servers to this 0 = local */
	short status;			/* Client type */
	char nicksent;
	char name[HOSTLEN + 1];		/* Unique name of the client, nick or host */
	char info[REALLEN + 1];		/* Free form additional client information */
#ifdef FLUD
	Link *fludees;
#endif
	int count;			/* Amount of data in buffer */
#ifdef FLUD
	time_t fludblock;
	struct fludbot *fluders;
#endif
#ifdef ANTI_SPAMBOT
	time_t last_join_time;		/* when this client last joined a channel */
	time_t last_leave_time;		/* when this client last left a channel */
	int join_leave_count;		/* count of JOIN/LEAVE in less than MIN_JOIN_LEAVE_TIME seconds */
	int oper_warn_count_down;	/* * warn opers of this possible spambot every time this gets to 0 */
#endif
	char buffer[BUFSIZE];		/* Incoming message buffer */
	short lastsq;			/* # of 2k blocks when sendqueued called last */
	DBuf sendQ;			/* Outgoing message queue--if socket full */
	DBuf recvQ;			/* Hold for data incoming yet to be parsed */
	long sendM;			/* Statistics: protocol messages send */
	long sendK;			/* Statistics: total k-bytes send */
	long receiveM;			/* Statistics: protocol messages received */
	long receiveK;			/* Statistics: total k-bytes received */
	u_short sendB;  /* counters to count upto 1-k lots of bytes */
	u_short receiveB;		/* sent and received. */
	long lastrecvM;			/* to check for activity --Mika */
	int priority;
  	Link *confs;			/* Configuration record associated */
  	int         authfd;	   /* fd for rfc931 authentication */
	char        username[USERLEN + 1]; /* username here now for auth stuff */
	struct in_addr ip;		 /* keep real ip# too */
	char        hostip[HOSTIPLEN + 1];	/* Keep real ip as string too - Dianora */
	unsigned short port;	 /* and the remote port# too :-) */
	struct hostent *hostp;
#ifdef ANTI_NICK_FLOOD
	time_t      last_nick_change;
	int         number_of_nick_changes;
#endif
#ifdef NO_AWAY_FLUD
        time_t	    alas;	/* last time of away set */
	int	    acount;	/* count of away settings */
#endif
	char        sockhost[HOSTLEN + 1];	/* This is the host name from the socket and after which the 
						 * connection was accepted. */
	char        passwd[PASSWDLEN + 1];
	/* try moving this down here to prevent weird problems... ? */
	int         oflag;    /* Operator Flags */
	Link *watch; /* user's watch list */
        int sockerr; /* what was the last error returned for this socket? */
	int watches; /* how many watches this user has set */
        int capabilities; /* what this server/client supports */
	int pingval;	  /* cache client class ping value here */
        int sendqlen;	  /* cache client max sendq here */
	aClient *listener;		/* port */
	ConfigItem_class *class;	/* class assigned */
};

#define	CLIENT_LOCAL_SIZE sizeof(aClient)
#define	CLIENT_REMOTE_SIZE offsetof(aClient, count)
#define USER_LOCAL_SIZE sizeof(anUser)
#define USER_REMOTE_SIZE offsetof(anUser, last)

/* statistics structures */
struct stats {
	unsigned int is_cl;     /* number of client connections */
	unsigned int is_sv;     /* number of server connections */
	unsigned int is_ni;     /* connection but no idea who it was */
	unsigned short is_cbs;  /* bytes sent to clients */
	unsigned short is_cbr;  /* bytes received to clients */
	unsigned short is_sbs;	/* bytes sent to servers */
	unsigned short is_sbr;  /* bytes received to servers */
	unsigned long is_cks;	  /* k-bytes sent to clients */
	unsigned long is_ckr;	  /* k-bytes received to clients */
	unsigned long is_sks;	  /* k-bytes sent to servers */
	unsigned long is_skr;	  /* k-bytes received to servers */
	time_t      is_cti;		  /* time spent connected by clients */
	time_t      is_sti;		  /* time spent connected by servers */
	unsigned int is_ac;		  /* connections accepted */
	unsigned int is_ref;	  /* accepts refused */
	unsigned int is_unco;	  /* unknown commands */
	unsigned int is_wrdi;	  /* command going in wrong direction */
	unsigned int is_unpf;	  /* unknown prefix */
	unsigned int is_empt;	  /* empty message */
	unsigned int is_num;	  /* numeric message */
	unsigned int is_kill;  	/* number of kills generated on collisions */
	unsigned int is_fake;	  /* MODE 'fakes' */
	unsigned int is_asuc;	  /* successful auth requests */
	unsigned int is_abad;	  /* bad auth requests */
	unsigned int is_udp;		/* packets recv'd on udp port */
	unsigned int is_loc;		/* local connections made */
#ifdef FLUD
	unsigned int is_flud;	  /* users/channels flood protected */
#endif	                  /* FLUD */
};

/* mode structure for channels */

struct SMode {
	unsigned int mode;
	int         limit;
	char        key[KEYLEN + 1];
};

struct Ban {
	char       *banstr;
	char       *who;
	time_t     when;
	aBan 	   *next;
};

/* channel member link structure, used for chanmember chains */
struct ChanLink {
        struct ChanLink *next;
        aClient *cptr;
        int flags;
};

/* general link structure used for chains */

struct SLink {
	struct SLink *next;
	union {
		aClient    *cptr;
		aChannel   *chptr;
		ConfigItem *conf;
		aBan       *banptr;
		aWatch     *wptr;
		char       *cp;
	} value;
	int         flags;
};

/* channel structure */

struct Channel {
	struct Channel *nextch, *prevch, *hnextch;
	int         hashv;		/* raw hash value */
	Mode        mode;
	char        topic[TOPICLEN + 1];
        char        topic_nick[NICKLEN + 1];
	time_t      topic_time;
	int         users;
	chanMember       *members;
	Link       *invites;
	aBan       *banlist;
	aBan	   *exbanlist;
	ts_val      channelts;
#ifdef FLUD
	time_t      fludblock;
	struct fludbot *fluders;
#endif
	char        chname[CHANNELLEN+1];
	/* Channel Flood Control */
	time_t last_knock;
	time_t first_message_time;
	int number_of_messages;
	int flood_notice;
};

/* TS Definitions
 *
 * TS_*     - TS versions used in SVINFO when parc > 5 (rage2<->rage2)
 * TSORA_*  - TS versions used in SVINFO when parc == 5 (bahamut-compatible<->bahamut-compatible)
 */
#define TS_CURRENT	5		/* Current rage2 TS protocol version */
#define TSORA_CURRENT	3		/* Current bahamut-compatible TS protocol version */
#define TS_MIN		3		/* Minimum rage2 TS protocol version */
#define TSORA_MIN	1		/* Current bahamut-compatible TS protocol version */
#define TS_DOESTS	0x20000000
#define DoesTS(x)	((x)->tsinfo == TS_DOESTS)

#define CHFL_CHANADMIN	0x0001	/* Channel Administrator */
#define	CHFL_CHANOP	0x0002	/* Channel Operator */
#define CHFL_HALFOP	0x0004	/* Channel Half Operator */
#define	CHFL_VOICE	0x0008	/* the power to speak */
#define	CHFL_DEOPPED	0x0010	/* deopped by us, modes need to be bounced */

#define MODE_CHANADMIN		CHFL_CHANADMIN
#define	MODE_CHANOP		CHFL_CHANOP
#define MODE_HALFOP		CHFL_HALFOP
#define	MODE_VOICE		CHFL_VOICE
#define	MODE_DEOPPED		CHFL_DEOPPED
#define	MODE_PRIVATE		0x0020
#define	MODE_SECRET		0x0040
#define	MODE_MODERATED		0x0080
#define	MODE_TOPICLIMIT		0x0100
#define	MODE_INVITEONLY		0x0200
#define	MODE_NOPRIVMSGS		0x0400
#define	MODE_KEY		0x0800
#define	MODE_BAN		0x1000
#define	MODE_LIMIT		0x2000
#define MODE_REGISTERED		0x4000
#define MODE_REGONLY		0x8000
#define MODE_NOCOLOR		0x10000
#define MODE_OPERONLY		0x20000
#define MODE_ADMINONLY		0x40000
#define MODE_STRIPCOLOR		0x80000
#define MODE_EXBAN		0x100000
#define MODE_NOCTCP		0x200000
#define MODE_NONICKCH		0x400000

/* mode flags which take another parameter (With PARAmeterS) */
#define	MODE_WPARAS	(MODE_CHANOP|MODE_VOICE|MODE_BAN|MODE_KEY|MODE_LIMIT|MODE_HALFOP|MODE_CHANADMIN|MODE_EXBAN)

#define	HoldChannel(x)		(!(x))
#define	SecretChannel(x)	((x) && ((x)->mode.mode & MODE_SECRET))
#define	HiddenChannel(x)	((x) && ((x)->mode.mode & MODE_PRIVATE))
#define	ShowChannel(v,c)	(PubChannel(c) || IsMember((v),(c)))
#define	PubChannel(x)		((!x) || ((x)->mode.mode & (MODE_PRIVATE | MODE_SECRET)) == 0)
#define IsMember(blah, chan)	((blah && blah->user && find_channel_link((blah->user)->channel, chan)) ? 1 : 0)
#define	IsChannelName(name)	((name) && (*(name) == '#' || *(name) == '&'))

/* Misc macros */

#define	BadPtr(x)	(!(x) || (*(x) == '\0'))
#define	isvalid(c)	(((c) >= 'A' && (c) < '~') || isdigit(c) || (c) == '-')
#define	MyConnect(x)	((x)->fd >= 0)
#define	MyClient(x)	(MyConnect(x) && IsClient(x))
#define	MyOper(x)	(MyConnect(x) && IsOper(x))
#define IsSendable(x)	(DBufLength(&x->sendQ) < 16384)
#define DoList(x)	(((x)->user) && ((x)->user->lopt))
#define strncpyzt(x, y, N)	do { strncpy(x, y, N); x[N - 1] = '\0'; } while (0)

#define	MODE_NULL      0
#define	MODE_ADD       0x40000000
#define	MODE_DEL       0x20000000

/* return values for hunt_server() */

#define	HUNTED_NOSUCH	(-1)	/* if the hunted server is not found */
#define	HUNTED_ISME	0	/* if this server should execute the command */
#define	HUNTED_PASS	1	/* if message passed onwards successfully */

/* used when sending to #mask or $mask */

#define	MATCH_SERVER  1
#define	MATCH_HOST    2

/* used for async dns values */

#define	ASYNC_NONE	(-1)
#define	ASYNC_CLIENT	0
#define	ASYNC_CONNECT	1
#define	ASYNC_CONF	2
#define	ASYNC_SERVER	3

/* misc variable externs */

extern char version[128], *build;
extern char *infotext[], *dalinfotext[];
extern char *generation, *creation;

/* misc defines */

#define ZIP_NEXT_BUFFER -4
#define RC4_NEXT_BUFFER -3
#define	FLUSH_BUFFER	-2
#define	UTMP		"/etc/utmp"
#define	COMMA		","

#ifdef ORATIMING

/*
 * Timing stuff (for performance measurements): compile with
 * -DORATIMING and put a TMRESET where you want the counter of time
 * spent set to 0, a TMPRINT where you want the accumulated results,
 * and TMYES/TMNO pairs around the parts you want timed -orabidoo
 */

extern struct timeval tsdnow, tsdthen;
extern unsigned long tsdms;

#define TMRESET	tsdms = 0;
#define TMYES	gettimeofday(&tsdthen, NULL);
#define TMNO	gettimeofday(&tsdnow, NULL); if (tsdnow.tv_sec != tsdthen.tv_sec) tsdms += 1000000 * (tsdnow.tv_sec - tsdthen.tv_sec); tsdms += tsdnow.tv_usec; tsdms -= tsdthen.tv_usec;
#define TMPRINT	sendto_ops("Time spent: %ld ms", tsdms);
#else
#define TMRESET
#define TMYES
#define TMNO
#define TMPRINT
#endif

#ifdef FLUD
struct fludbot {
	struct Client *fluder;
	int         count;
	time_t      first_msg, last_msg;
	struct fludbot *next;
};

#endif /* FLUD */

struct Watch {
	   aWatch  *hnext;
	   time_t   lasttime;
	   Link  *watch;
	   char  nick[1];
};

struct ListOptions {
	   LOpts *next;
	   Link  *yeslist, *nolist;
	   int   starthash;
	   short int   showall;
	   unsigned short usermin;
	   int   usermax;
	   time_t   currenttime;
	   time_t   chantimemin;
	   time_t   chantimemax;
	   time_t   topictimemin;
	   time_t   topictimemax;
};

typedef struct SearchOptions {
	int umodes;
	char *nick;
	char *user;
	char *host;
	char *gcos;
	aChannel *channel;
	aClient *server;
	char umode_plus:1;
	char nick_plus:1;
	char user_plus:1;
	char host_plus:1;
	char gcos_plus:1;
	char chan_plus:1;
	char serv_plus:1;
	char away_plus:1;
	char check_away:1;
	char check_umode:1;
        char show_chan:1;
        char search_chan:1;
	char spare:4; /* spare space for more stuff(?) */
} SOpts;

/* internal defines for cptr->sockerr */
#define IRCERR_BUFALLOC		-11
#define IRCERR_ZIP		-12

#define runHook0(hooktype)	for (ghook = HookTable[hooktype]; ghook; ghook = ghook->next) { \
					if (ghook->ifunc != NULL) ghook->ifunc(); \
					if (ghook->vfunc != NULL) ghook->vfunc(); \
				}

#define runHook1(hooktype, a)	for (ghook = HookTable[hooktype]; ghook; ghook = ghook->next) { \
					if (ghook->ifunc != NULL) ghook->ifunc(a); \
					if (ghook->vfunc != NULL) ghook->vfunc(a); \
				}

#endif /* __struct_include__ */
