/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/aliases.c: services' aliases module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: aliases.c,v 1.10 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_chanserv(aClient *, aClient *, int, char **);
int m_nickserv(aClient *, aClient *, int, char **);
int m_memoserv(aClient *, aClient *, int, char **);
int m_operserv(aClient *, aClient *, int, char **);
int m_statserv(aClient *, aClient *, int, char **);
int m_services(aClient *, aClient *, int, char **);
int m_identify(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/aliases.so",
	"Service Aliases",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("CHANSERV", m_chanserv, 0, 1, 1, 0, 0, 0L);
	add_Command("CS", m_chanserv, 0, 1, 1, 0, 0, 0L);
	add_Command("NICKSERV", m_nickserv, 0, 1, 1, 0, 0, 0L);
	add_Command("NS", m_nickserv, 0, 1, 1, 0, 0, 0L);
	add_Command("MEMOSERV", m_memoserv, 0, 1, 1, 0, 0, 0L);
	add_Command("MS", m_memoserv, 0, 1, 1, 0, 0, 0L);
	add_Command("OPERSERV", m_operserv, 0, 1, 1, 0, 0, 0L);
	add_Command("OS", m_operserv, 0, 1, 1, 0, 0, 0L);
	add_Command("STATSERV", m_statserv, 0, 1, 1, 0, 0, 0L);
	add_Command("SS", m_statserv, 0, 1, 1, 0, 0, 0L);
	add_Command("SERVICES", m_services, 0, 1, 1, 0, 0, 0L);
	add_Command("IDENTIFY", m_identify, 0, 1, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("CHANSERV", m_chanserv);
	del_Command("CS", m_chanserv);
	del_Command("NICKSERV", m_nickserv);
	del_Command("NS", m_nickserv);
	del_Command("MEMOSERV", m_memoserv);
	del_Command("MS", m_memoserv);
	del_Command("OPERSERV", m_operserv);
	del_Command("OS", m_operserv);
	del_Command("STATSERV", m_statserv);
	del_Command("SS", m_statserv);
	del_Command("SERVICES", m_services);
	del_Command("IDENTIFY", m_identify);
}
#endif

/*
 * m_chanserv
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_chanserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
		return 0;
	}

	acptr = find_person(ConfigSet.chanserv, NULL);
	if (!acptr) {
		sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], ConfigSet.chanserv);
		return 0;
	}
	sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", parv[0], ConfigSet.chanserv, ConfigSet.services,
		parv[1]);
	return 0;
}

/*
 * m_nickserv
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_nickserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
		return 0;
	}

	acptr = find_person(ConfigSet.nickserv, NULL);
	if (!acptr) {
		sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], ConfigSet.nickserv);
		return 0;
	}
	sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", parv[0], ConfigSet.nickserv, ConfigSet.services,
		parv[1]);
	return 0;
}

/*
 * m_memoserv
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_memoserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
		return 0;
	}

	acptr = find_person(ConfigSet.memoserv, NULL);
	if (!acptr) {
		sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], ConfigSet.memoserv);
		return 0;
	}
	sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", parv[0], ConfigSet.memoserv, ConfigSet.services,
		parv[1]);
	return 0;
}

/*
 * m_operserv
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_operserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
		return 0;
	}

	acptr = find_person(ConfigSet.operserv, NULL);
	if (!acptr) {
		sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], ConfigSet.operserv);
		return 0;
	}
	sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", parv[0], ConfigSet.operserv, ConfigSet.services,
		parv[1]);
	return 0;
}

/*
 * m_statserv
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_statserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
		return 0;
	}

	acptr = find_person(ConfigSet.statserv, NULL);
	if (!acptr) {
		sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0], ConfigSet.statserv);
		return 0;
	}
	sendto_one(acptr, ":%s PRIVMSG %s@%s :%s", parv[0], ConfigSet.statserv, ConfigSet.stats,
		parv[1]);
	return 0;
}

/*
 * m_services
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_services(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *tmp = NULL;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
		return 0;
	}

	if (strlen(parv[1]) >= 4 && !myncmp(parv[1], "help", 4)) {
		sendto_one(sptr, ":services!service@%s NOTICE %s :For ChanServ help, use /chanserv help",
			ConfigSet.services, parv[0]);
		sendto_one(sptr, ":services!service@%s NOTICE %s :For NickServ help, use /nickserv help",
			ConfigSet.services, parv[0]);
		sendto_one(sptr, ":services!service@%s NOTICE %s :For Memoserv help, use /memoserv help",
			ConfigSet.services, parv[0]);
		sendto_one(sptr, ":services!service@%s NOTICE %s :For StatServ help, use /statserv help",
			ConfigSet.stats, parv[0]);
		return 0;
	}

	tmp = strchr(parv[1], ' ');
	if (tmp) {
		for (; *tmp == ' '; tmp++) {
			if (*tmp == '#') {
				return m_chanserv(cptr, sptr, parc, parv);
			}
			else {
				return m_nickserv(cptr, sptr, parc, parv);
			}
		}
	}
	return m_nickserv(cptr, sptr, parc, parv);
}

/*
 * m_identify
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_identify(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;
	int gate = 0;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NOTEXTTOSEND), me.name, parv[0]);
		return 0;
	}

	if (*parv[1] == '#' && strchr(parv[1], ' ')) {
		acptr = find_person(ConfigSet.chanserv, NULL);
		gate = 1;
	}
	else {
		acptr = find_person(ConfigSet.nickserv, NULL);
		gate = 0;
	}
	if (!acptr) {
		sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name, parv[0],
			gate ? ConfigSet.chanserv : ConfigSet.nickserv);
		return 0;
	}
	sendto_one(acptr, ":%s PRIVMSG %s@%s :IDENTIFY %s", parv[0],
		gate ? ConfigSet.chanserv : ConfigSet.nickserv,
		ConfigSet.services, parv[1]);
	return 0;
}
