/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/commands.c: commands module wrapper
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: commands.c,v 1.6 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern void init_motd(int);
extern void init_opermotd(int);
extern void init_rules(int);

Module _module_header = {
	"modules/commands.so",
	"Commands Module",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("AWAY", m_away, 0, 15, 1, 0, 1, 0L);
	add_Command("MAP", m_map, 0, 15, 1, 0, 0, 0L);
	add_Command("MKPASSWD", m_mkpasswd, 0, 15, 1, 0, 0, 0L);
	add_Command("OPER", m_oper, 0, 15, 1, 0, 0, 0L);
	add_Command("VERSION", m_version, 0, 15, 1, 1, 0, 0L);
	add_Command("SVSMODE", m_svsmode, 0, 15, 1, 0, 0, 0L);
	add_Command("SVSNICK", m_svsnick, 0, 15, 1, 0, 0, 0L);
	add_Command("JOIN", m_join, 0, 15, 1, 0, 1, 0L);
	add_Command("PART", m_part, 0, 15, 1, 0, 1, 0L);
	add_Command("KLINE", m_kline, 0, 15, 1, 0, 0, 0L);
	add_Command("UNKLINE", m_unkline, 0, 15, 1, 0, 0, 0L);
	add_Command("DIE", m_die, 0, 15, 1, 0, 0, 0L);
	add_Command("RESTART", m_restart, 0, 15, 1, 0, 0, 0L);
	add_Command("ZLINE", m_zline, 0, 15, 1, 0, 0, 0L);
	add_Command("SQLINE", m_sqline, 0, 15, 1, 0, 0, 0L);
	add_Command("UNSQLINE", m_unsqline, 0, 15, 1, 0, 0, 0L);
	add_Command("WHOIS", m_whois, 0, 15, 1, 0, 0, 0L);
	add_Command("WHOWAS", m_whowas, 0, 15, 1, 0, 0, 0L);
	add_Command("ADMIN", m_admin, 0, 15, 1, 1, 0, 0L);
	add_Command("INFO", m_info, 0, 15, 1, 0, 0, 0L);
	add_Command("DALINFO", m_dalinfo, 0, 15, 1, 0, 0, 0L);
	add_Command("LINKS", m_links, 0, 15, 1, 0, 0, 0L);
	add_Command("KICK", m_kick, 0, 15, 1, 0, 0, 0L);
	add_Command("SENDVHOST", m_sendvhost, 0, 15, 1, 0, 0, 0L);
	add_Command("VHOST", m_vhost, 0, 15, 1, 0, 0, 0L);
	add_Command("MODE", m_mode, 0, 15, 1, 0, 0, 0L);
	add_Command("TOPIC", m_topic, 0, 15, 1, 0, 0, 0L);
	add_Command("INVITE", m_invite, 0, 15, 1, 0, 0, 0L);
	add_Command("NAMES", m_names, 0, 15, 1, 0, 0, 0L);
	add_Command("TIME", m_time, 0, 15, 1, 0, 0, 0L);
	add_Command("PING", m_ping, 0, 15, 1, 0, 0, 0L);
	add_Command("PONG", m_pong, 0, 15, 1, 0, 0, 0L);
	add_Command("QUIT", m_quit, 0, 15, 1, 0, 0, 0L);
	add_Command("PASS", m_pass, 0, 15, 1, 1, 0, 0L);
	add_Command("USERHOST", m_userhost, 0, 15, 1, 1, 0, 0L);
	add_Command("ISON", m_ison, 0, 15, 1, 0, 0, 0L);
	add_Command("MOTD", m_motd, 0, 15, 1, 0, 0, 0L);
	add_Command("RULES", m_rules, 0, 15, 1, 0, 0, 0L);
	add_Command("OPERMOTD", m_opermotd, 0, 15, 1, 0, 0, 0L);
	add_Command("SVSNOOP", m_svsnoop, 0, 15, 1, 0, 0, 0L);
	add_Command("LIST", m_list, 0, 15, 1, 0, 0, 0L);
	add_Command("SAMODE", m_samode, 0, 15, 1, 0, 0, 0L);
	add_Command("CAPAB", m_capab, 0, 15, 1, 1, 0, 0L);
	add_Command("SJOIN", m_sjoin, 0, 15, 1, 0, 0, 0L);
	add_Command("REHASH", m_rehash, 0, 15, 1, 0, 0, 0L);
	add_Command("HTM", m_htm, 0, 15, 1, 0, 0, 0L);
	add_Command("BURST", m_burst, 0, 15, 1, 0, 0, 0L);
	add_Command("SVINFO", m_svinfo, 0, 15, 1, 1, 0, 0L);
	add_Command("USERS", m_users, 0, 15, 1, 0, 0, 0L);
	add_Command("ERROR", m_error, 0, 15, 1, 1, 0, 0L);
	add_Command("LUSERS", m_lusers, 0, 15, 1, 0, 0, 0L);
	add_Command("CONNECT", m_connect, 0, 15, 1, 0, 0, 0L);
	add_Command("WALLOPS", m_wallops, 0, 15, 1, 0, 0, 0L);
	add_Command("LOCOPS", m_locops, 0, 15, 1, 0, 0, 0L);
	add_Command("GLOBOPS", m_globops, 0, 15, 1, 0, 0, 0L);
	add_Command("GOPER", m_goper, 0, 15, 1, 0, 0, 0L);
	add_Command("GNOTICE", m_gnotice, 0, 15, 1, 0, 0, 0L);
	add_Command("SQUIT", m_squit, 0, 15, 1, 0, 0, 0L);
	add_Command("CHATOPS", m_chatops, 0, 15, 1, 0, 0, 0L);
	add_Command("CLOSE", m_close, 0, 15, 1, 0, 0, 0L);
	add_Command("NICK", m_nick, 0, 15, 1, 1, 0, 0L);
	add_Command("USER", m_user, 0, 15, 1, 1, 0, 0L);
	add_Command("PRIVMSG", m_private, 0, 15, 1, 0, 1, 0L);
	add_Command("NOTICE", m_notice, 0, 15, 1, 0, 0, 0L);
	add_Command("KILL", m_kill, 0, 15, 1, 0, 0, 0L);
	add_Command("SVSKILL", m_svskill, 0, 15, 1, 0, 0, 0L);
	add_Command("AKILL", m_akill, 0, 15, 1, 0, 0, 0L);
	add_Command("RAKILL", m_rakill, 0, 15, 1, 0, 0, 0L);
	add_Command("DNS", m_dns, 0, 15, 1, 0, 0, 0L);
	add_Command("DKEY", m_dkey, 0, 15, 1, 1, 0, 0L);
	add_Command("TRACE", m_trace, 0, 15, 1, 0, 0, 0L);
	add_Command("WATCH", m_watch, 0, 15, 1, 0, 0, 0L);
	add_Command("DCCALLOW", m_dccallow, 0, 15, 1, 1, 0, 0L);
	add_Command("SILENCE", m_silence, 0, 15, 1, 0, 0, 0L);
	add_Command("STATS", m_stats, 0, 15, 1, 0, 0, 0L);
	add_Command("WHO", m_who, 0, 15, 1, 0, 0, 0L);
	add_Command("SERVER", m_server, 0, 15, 1, 1, 0, 0L);
	add_Command("KNOCK", m_knock, 0, 15, 1, 0, 0, 0L);
	add_Command("SET", m_set, 0, 15, 1, 0, 0, 0L);
	add_Command("SGLINE", m_sgline, 0, 15, 1, 0, 0, 0L);
	add_Command("UNSGLINE", m_unsgline, 0, 15, 1, 0, 0, 0L);
	add_Command("ACCEPT", m_accept, 0, 15, 1, 1, 0, 0L);
	init_motd(1);
	init_opermotd(1);
	init_rules(1);
	return;
}
void _module_unload(void) {
	del_Command("AWAY", m_away);
	del_Command("MAP", m_map);
	del_Command("MKPASSWD", m_mkpasswd);
	del_Command("OPER", m_oper);
	del_Command("VERSION", m_version);
	del_Command("SVSMODE", m_svsmode);
	del_Command("SVSNICK", m_svsnick);
	del_Command("JOIN", m_join);
	del_Command("PART", m_part);
	del_Command("KLINE", m_kline);
	del_Command("UNKLINE", m_unkline);
	del_Command("DIE", m_die);
	del_Command("RESTART", m_restart);
	del_Command("ZLINE", m_zline);
	del_Command("SQLINE", m_sqline);
	del_Command("UNSQLINE", m_unsqline);
	del_Command("WHOIS", m_whois);
	del_Command("WHOWAS", m_whowas);
	del_Command("ADMIN", m_admin);
	del_Command("INFO", m_info);
	del_Command("DALINFO", m_dalinfo);
	del_Command("LINKS", m_links);
	del_Command("KICK", m_kick);
	del_Command("SENDVHOST", m_sendvhost);
	del_Command("VHOST", m_vhost);
	del_Command("MODE", m_mode);
	del_Command("TOPIC", m_topic);
	del_Command("INVITE", m_invite);
	del_Command("NAMES", m_names);
	del_Command("TIME", m_time);
	del_Command("PING", m_ping);
	del_Command("PONG", m_pong);
	del_Command("QUIT", m_quit);
	del_Command("PASS", m_pass);
	del_Command("USERHOST", m_userhost);
	del_Command("ISON", m_ison);
	del_Command("MOTD", m_motd);
	del_Command("RULES", m_rules);
	del_Command("OPERMOTD", m_opermotd);
	del_Command("SVSNOOP", m_svsnoop);
	del_Command("LIST", m_list);
	del_Command("SAMODE", m_samode);
	del_Command("CAPAB", m_capab);
	del_Command("SJOIN", m_sjoin);
	del_Command("REHASH", m_rehash);
	del_Command("HTM", m_htm);
	del_Command("BURST", m_burst);
	del_Command("SVINFO", m_svinfo);
	del_Command("USERS", m_users);
	del_Command("ERROR", m_error);
	del_Command("LUSERS", m_lusers);
	del_Command("CONNECT", m_connect);
	del_Command("WALLOPS", m_wallops);
	del_Command("LOCOPS", m_locops);
	del_Command("GLOBOPS", m_globops);
	del_Command("GOPER", m_goper);
	del_Command("GNOTICE", m_gnotice);
	del_Command("SQUIT", m_squit);
	del_Command("CHATOPS", m_chatops);
	del_Command("CLOSE", m_close);
	del_Command("NICK", m_nick);
	del_Command("USER", m_user);
	del_Command("PRIVMSG", m_private);
	del_Command("NOTICE", m_notice);
	del_Command("KILL", m_kill);
	del_Command("SVSKILL", m_svskill);
	del_Command("AKILL", m_akill);
	del_Command("RAKILL", m_rakill);
	del_Command("DNS", m_dns);
	del_Command("DKEY", m_dkey);
	del_Command("TRACE", m_trace);
	del_Command("WATCH", m_watch);
	del_Command("DCCALLOW", m_dccallow);
	del_Command("SILENCE", m_silence);
	del_Command("STATS", m_stats);
	del_Command("WHO", m_who);
	del_Command("SERVER", m_server);
	del_Command("KNOCK", m_knock);
	del_Command("SET", m_set);
	del_Command("UNSGLINE", m_unsgline);
	del_Command("ACCEPT", m_accept);
	init_motd(0);
	init_opermotd(0);
	init_rules(0);
	return;
}
