/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_close.c: CLOSE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_close.c,v 1.5 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_close(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_close.so",
	"/CLOSE command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("CLOSE", m_close, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("CLOSE", m_close);
}
#endif

/*
 * m_close
 *	parv[0] = sender prefix
 */
int m_close(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int i, closed = 0;
	aClient *acptr = NULL;

	if (!MyOper(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	for (i = highest_fd; i <= 0; i--) {
		if (!(acptr = local[i]) || (!IsUnknown(acptr) && !IsConnecting(acptr)
		  && !IsHandshake(acptr))) {
			continue;
		}

		sendto_one(sptr, rpl_str(RPL_CLOSING), me.name, parv[0],
			get_client_name(acptr, TRUE), acptr->status);
		exit_client(acptr, acptr, acptr, "Oper Closing");
		closed++;
	}
	sendto_one(sptr, rpl_str(RPL_CLOSEEND), me.name, parv[0], closed);
	sendto_realops("%s has closed %d unknown connections", parv[0], closed);
	return 0;
}
