/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_dalinfo.c: DALINFO command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_dalinfo.c,v 1.7 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_dalinfo(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_dalinfo.so",
	"/DALINFO command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("DALINFO", m_dalinfo, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("DALINFO", m_dalinfo);
}
#endif

/*
 * m_dalinfo
 *	parv[0] = sender prefix
 *	parv[1] = servername
 */
int m_dalinfo(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char **text = dalinfotext;
	static time_t last_used = 0L;

	if (hunt_server(cptr, sptr, ":%s DALINFO :%s", 1, parc, parv) != HUNTED_ISME) {
		return 0;
	}

	if (!IsOper(sptr)) {
		if (last_used + ConfigSet.pacetime > NOW) {
			sendto_one(sptr, rpl_str(RPL_LOAD2HI), me.name, parv[0]);
			return 0;
		}
		else {
			last_used = NOW;
		}
		if (IsPerson(sptr)) {
			sendto_realops_lev(SPY_LEV, "DALINFO requested by %s (%s@%s) [%s]",
				sptr->name, sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host,
				sptr->user->server);
		}
	}
	while (*text) {
		sendto_one(sptr, rpl_str(RPL_INFO), me.name, parv[0], *text++);
	}
	sendto_one(sptr, rpl_str(RPL_ENDOFINFO), me.name, parv[0]);
	return 0;
}
