/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_die.c: DIE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_die.c,v 1.9 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_die(aClient *, aClient *, int, char **);

extern VOIDSIG s_die();

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_die.so",
	"/DIE command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("DIE", m_die, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("DIE", m_die);
}
#endif

/*
 * m_die
 *	parv[0] = sender prefix
 *	parv[1] = password
 */
int m_die(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;
	int i;

	if (!IsOper(sptr) || !OPCanDie(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	if (config_password && config_password->die) {
		if (parc < 2 || *parv[1] == '\0') {
			sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "DIE");
			return 0;
		}
		if (strcmp(config_password->die, parv[1])) {
			sendto_one(sptr, err_str(ERR_PASSWDMISMATCH), me.name, parv[0]);
			ircdlog(LOG_DEFAULT, "%s attempted to terminate server: password mismatch",
				get_client_name(sptr, FALSE));
			return 0;
		}
	}

	for (i = 0; i <= highest_fd; i++) {
		if (!(acptr = local[i])) {
			continue;
		}

		if (IsClient(acptr)) {
			sendto_one(acptr, ":%s NOTICE %s :Server terminating by request of %s.",
				me.name, acptr->name, get_client_name(sptr, FALSE));
		}
		else if (IsServer(acptr)) {
			sendto_one(acptr, ":%s ERROR :Server terminating by request of %s.",
				me.name, acptr->name, get_client_name(sptr, TRUE));
		}
	}

	ircdlog(LOG_DEFAULT, "server terminating by request of %s", get_client_name(sptr, FALSE));
	s_die();
	return 0;
}
