/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/m_dns.c: DNS command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_dns.c,v 1.7 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "res.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_dns(aClient *, aClient *, int, char **);

extern aCache *cachetop;

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_dns.so",
	"/DNS command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("DNS", m_dns, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("DNS", m_dns);
}
#endif

/*
 * m_dns
 *	parv[0] = sender prefix
 *	parv[1] = optional command
 */
int m_dns(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aCache *cp;
	int i = 0;

	if (MyOper(sptr) && parc > 1 && *parv[1] == 'l') {
		sendto_realops_lev(SPY_LEV, "DNS cache list requested by %s (%s@%s)",
			sptr->name, sptr->user->username,
			IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
		for (cp = cachetop; cp; cp = cp->list_next) {
			sendto_one(sptr, "NOTICE %s :Ext %d ttl %d host %s(%s)", sptr->name,
				cp->expireat - timeofday, cp->ttl, cp->he.h_name,
				inetntoa(cp->he.h_addr));
			for (i = 0; cp->he.h_aliases[i]; i++) {
				sendto_one(sptr, "NOTICE %s : %s = %s (CN)", sptr->name, cp->he.h_name,
					cp->he.h_aliases[i]);
			}
			for (i = 0; cp->he.h_addr_list[i]; i++) {
				sendto_one(sptr, "NOTICE %s : %s = %s (IP)", sptr->name, cp->he.h_name,
					inetntoa(cp->he.h_addr_list[i]));
			}
		}
		return 0;
	}
	sendto_one(sptr, "NOTICE %s :Ca %d Cd %d Ce %d Cl %d Ch %d:%d Cu %d", sptr->name, cainfo.ca_adds,
		cainfo.ca_dels, cainfo.ca_expires, cainfo.ca_lookups, cainfo.ca_na_hits, cainfo.ca_nu_hits,
		cainfo.ca_updates);
	sendto_one(sptr, "NOTICE %s :Re %d Rl %d/%d Rp %d Rq %d", sptr->name, reinfo.re_errors,
		reinfo.re_nu_look, reinfo.re_na_look, reinfo.re_replies, reinfo.re_requests);
	sendto_one(sptr, "NOTICE %s :Ru %d Rsh %d Rs %d(%d) Rt %d", sptr->name, reinfo.re_unkrep,
		reinfo.re_shortttl, reinfo.re_sent, reinfo.re_resends, reinfo.re_timeouts);
	return 0;
}
