/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_error.c: ERROR protocol module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_error.c,v 1.5 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_error(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_error.so",
	"ERROR protocol",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("ERROR", m_error, 0, 15, 1, 1, 0, 0L);
}
void _module_unload(void) {
	del_Command("ERROR", m_error);
}
#endif

/*
 * m_error
 *	parv[0] = sender prefix
 *	parv[1] = message
 */
int m_error(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *para = NULL;

	if (IsPerson(cptr) || IsUnknown(cptr)) {
		return 0;
	}

	para = (parc > 1 && *parv[1] != '\0') ? parv[1] : "<>";
	Debug((DEBUG_ERROR, "Received ERROR message from %s: %s", sptr->name, para));

	if (cptr == sptr) {
		sendto_ops("ERROR :from %s -- %s", get_client_name(cptr, HIDEME), para);
	}
	else {
		sendto_ops("ERROR :from %s via %s -- %s", sptr->name,
			get_client_name(cptr, HIDEME), para);
	}
	return 0;
}
