/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_join.c: JOIN command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_join.c,v 1.17 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_join(aClient *, aClient *, int, char **);

#ifdef ANTI_SPAMBOT
/* SPAMBOT stuff, defined in s_serv.c */
extern int spam_num;
extern int spam_time;
#endif /* ANTI_SPAMBOT */

extern int can_join(aClient *, aChannel *, char *);
extern aChannel *get_channel(aClient *, char *, int);
extern void add_user_to_channel(aChannel *chptr, aClient *who, int flags);
extern void send_names(aClient *, aChannel *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_join.so",
	"/JOIN command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("JOIN", m_join, 0, 15, 1, 0, 1, 0L);
}
void _module_unload(void) {
	del_Command("JOIN", m_join);
}
#endif

/*
 * m_join
 *	parv[0] = sender prefix
 *	parv[1] = channel name
 *	parv[2] = channel password (key)
 */
int m_join(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	static char jbuf[BUFSIZE];
	char *name, *key = NULL, *p, *p2;
	Link *lp;
	aChannel *chptr;
	ConfigItem_ban *ban = NULL;
	int i, flags = 0, chanlen = 0;
#ifdef ANTI_SPAMBOT
	int successful_join_count = 0;
#endif

	if (!sptr->user) {
		return 0;
	}
	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "JOIN");
		return 0;
	}

	jbuf[0] = '\0';
	for (i = 0, name = strtoken(&p, parv[1], ","); name; name = strtoken(&p, NULL, ",")) {
		if (!check_channelname(sptr, (unsigned char *)name)) {
			continue;
		}

		chanlen = strlen(name);
		if (chanlen > CHANNELLEN) {
			name[CHANNELLEN] = '\0';
			chanlen = CHANNELLEN;
		}
		if (*name == '&' && !MyConnect(sptr)) {
			continue;
		}
		if (*name == '0' && !atoi(name)) {
			*jbuf = '\0';
		}
		else if (!IsChannelName(name)) {
			if (MyClient(sptr)) {
				sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL), me.name, parv[0], name);
			}
			continue;
		}

		if (*jbuf) {
			strcat(jbuf, ",");
		}
		strncat(jbuf, name, sizeof(jbuf) - i - 1);
		i += chanlen + 1;
	}

	p = NULL;
	if (parv[2]) {
		key = strtoken(&p2, parv[2], ",");
	}
	parv[2] = NULL;

	for (name = strtoken(&p, jbuf, ","); name; key = key ? strtoken(&p2, NULL, ",")  : NULL,
	  name = strtoken(&p, NULL, ",")) {
		if (*name == '0' && !atoi(name)) {
			if (!sptr->user->channel) {
				continue;
			}

			while ((lp = sptr->user->channel)) {
				chptr = lp->value.chptr;
				sendto_channel_butserv(chptr, sptr, ":%s PART %s", parv[0], chptr->chname);
				remove_user_from_channel(sptr, chptr);
			}

#ifdef ANTI_SPAMBOT
			if (MyConnect(sptr) && !IsOper(sptr)) {
				if (sptr->join_leave_count >= spam_num) {
					sendto_ops_lev(SPAM_LEV, "User %s (%s@%s) is a possible spambot",
						sptr->name, sptr->user->username,
						IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
					sptr->oper_warn_count_down = OPER_SPAM_COUNTDOWN;
				}
				else {
					int t_delta = (NOW - sptr->last_leave_time);
					if (t_delta > JOIN_LEAVE_COUNT_EXPIRE_TIME) {
						int decrement_count = (t_delta / JOIN_LEAVE_COUNT_EXPIRE_TIME);
						if (decrement_count > sptr->join_leave_count) {
							sptr->join_leave_count = 0;
						}
						else {
							sptr->join_leave_count -= decrement_count;
						}
					}
					else {
						if ((NOW - sptr->last_join_time) < spam_time) {
							sptr->join_leave_count++;
						}
					}
					sptr->last_leave_time = NOW;
				}
			}
#endif /* ANTI_SPAMBOT */

			sendto_match_servs(NULL, cptr, ":%s JOIN 0", parv[0]);
			continue;
		}

		if (MyConnect(sptr)) {
			flags = ChannelExists(name) ? 0 : CHFL_CHANOP;

			if (!ConfigSet.dynchans && !ChannelExists(name) && !IsOper(sptr)) {
				sendto_one(sptr, ":%s NOTICE %s :Dynamic channels are not allowed "
					"on this server - join %s for more information.", me.name,
					parv[0], ConfigSet.helpchannel);
				return 0;
			}

			ban = find_ban(BAN_CHANNEL, name);
			if (ban && !IsCoAdmin(sptr) && !IsAdmin(sptr)) {
				sendto_one(sptr, ":%s NOTICE %s :Channel %s is disallowed by configuration (%s)",
				me.name, parv[0], name, ban->reason);
				return 0;
			}

			if ((sptr->user->joined >= ConfigSet.maxchansperuser) && (!IsOper(sptr) ||
			  (sptr->user->joined >= ConfigSet.maxchansperuser * 3))) {
				sendto_one(sptr, err_str(ERR_TOOMANYCHANNELS), me.name, parv[0], name);
#ifdef ANTI_SPAMBOT
				if (successful_join_count) {
					sptr->last_join_time = NOW;
				}
#endif /* ANTI_SPAMBOT */
				return 0;
			}

#ifdef ANTI_SPAMBOT
			if (!flags) {
				successful_join_count++;
			}
			if (sptr->join_leave_count >= spam_num) {
				if (sptr->oper_warn_count_down > 0) {
					sptr->oper_warn_count_down--;
				}
				else {
					sptr->oper_warn_count_down = 0;
				}
				if (!sptr->oper_warn_count_down) {
					sendto_ops_lev(SPAM_LEV, "User %s (%s@%s) trying to join %s is a "
						"possible spambot", sptr->name, sptr->user->username,
						IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host,
						name);
					sptr->oper_warn_count_down = OPER_SPAM_COUNTDOWN;
				}
#ifndef ANTI_SPAMBOT_WARN_OUT
				return 0;
#endif /* ANTI_SPAMBOT_WARN_OUT */
			}
#endif /* ANTI_SPAMBOT */
		}
		else if (!ChannelExists(name)) {
			ts_warn("User on %s is remotely JOINing new channel", sptr->user->server);
		}

		chptr = get_channel(sptr, name, CREATE);
		if (!chptr || (MyConnect(sptr) && (i = can_join(sptr, chptr, key)))) {
			sendto_one(sptr, getreply(i), me.name, parv[0], name);
#ifdef ANTI_SPAMBOT
			if (successful_join_count > 0) {
				successful_join_count--;
			}
#endif /* ANTI_SPAMBOT */
			continue;
		}

		if (IsMember(sptr, chptr)) {
			continue;
		}

		add_user_to_channel(chptr, sptr, flags);

		if (MyClient(sptr)) {
			if (flags == CHFL_CHANOP) {
				chptr->channelts = timeofday;
				sendto_ssjoin_servs_butone(chptr, cptr, ":%s SJOIN %ld %s + <none> :@%s",
					me.name, chptr->channelts, name, parv[0]);
			}
			else {
				sendto_ssjoin_servs_butone(chptr, cptr, ":%s SJOIN %ld :%s", parv[0],
					chptr->channelts, name);
			}
		}
		else {
			sendto_match_servs(chptr, sptr, ":%s JOIN :%s", parv[0], name);
		}
		sendto_channel_butserv(chptr, sptr, ":%s JOIN :%s", parv[0], name);

		if (MyClient(sptr)) {
			del_invite(sptr, chptr);
			if (*chptr->topic != '\0') {
				sendto_one(sptr, rpl_str(RPL_TOPIC), me.name, parv[0], name, chptr->topic);
				sendto_one(sptr, rpl_str(RPL_TOPICWHOTIME), me.name, parv[0], name,
					chptr->topic_nick, chptr->topic_time);
			}
			send_names(sptr, chptr);
		}
	}

#ifdef ANTI_SPAMBOT
	if (MyConnect(sptr) && successful_join_count) {
		sptr->last_join_time = NOW;
	}
#endif /* ANTI_SPAMBOT */
	return 0;
}
