/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_kick.c: KICK command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_kick.c,v 1.8 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_kick(aClient *, aClient *, int, char **);

extern aChannel *get_channel(aClient *, char *, int);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_kick.so",
	"/KICK command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("KICK", m_kick, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("KICK", m_kick);
}
#endif

/*
 * m_kick
 *	parv[0] = sender prefix
 *	parv[1] = channel list
 *	parv[2] = client list
 *	parv[3] = comment
 */
int m_kick(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;
	aChannel *chptr = NULL;
	int chasing = 0, sakick = 0;
	char *comment, *name, *user, *p = NULL, *p2 = NULL;

	if (parc < 3 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "KICK");
		return 0;
	}

	comment = parc > 3 ? parv[3] : parv[0];
	if (strlen(comment) > TOPICLEN) {
		comment[TOPICLEN] = '\0';
	}

	for (; (name = strtoken(&p, parv[1], ",")); parv[1] = NULL) {
		chptr = get_channel(sptr, name, !CREATE);
		if (!chptr) {
			sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL), me.name, parv[0], name);
			continue;
		}

		if (!IsServer(sptr) && !is_chan_op(sptr, chptr) && !is_half_op(sptr, chptr) &&
		  !IsULine(sptr) && !IsOper(sptr)) {
			if (MyConnect(sptr)) {
				sendto_one(sptr, err_str(ERR_CHANOPRIVSNEEDED), me.name, parv[0],
					chptr->chname);
				continue;
			}
			if (chptr->channelts == 0) {
				sendto_one(sptr, err_str(ERR_CHANOPRIVSNEEDED), me.name, parv[0],
					chptr->chname);
				continue;
			}
		}

		for (; (user = strtoken(&p2, parv[2], ",")); parv[2] = NULL) {
			acptr = find_chasing(sptr, user, &chasing);
			if (!acptr) {
				continue;
			}

			if (IsMember(acptr, chptr)) {
				if (!IsServer(sptr) && !IsULine(sptr)) {
					if (IsSAdmin(sptr)) {
						if (!is_chan_op(sptr, chptr) && !is_half_op(sptr, chptr)) {
							sakick = 1;
						}
						if (!is_chan_op(sptr, chptr) && is_half_op(sptr, chptr) &&
						  (is_chan_op(acptr, chptr) || is_chan_admin(acptr, chptr))) {
							sakick = 1;
						}
						else if (is_chan_op(sptr, chptr) && is_chan_admin(acptr, chptr)) {
							sakick = 1;
						}
						if (sakick) {
							sendto_ops_lev(SPY_LEV, "%s used SAKICK %s %s (%s)",
								parv[0], chptr->chname, acptr->name, comment);
						}
					}
					else {
						if (!is_chan_admin(sptr, chptr) && is_chan_admin(acptr, chptr)) {
							sendto_one(sptr, ":%s NOTICE %s :*** Cannot kick %s - "
								"channel administrator", me.name, parv[0], user);
							continue;
						}
						if (is_half_op(sptr, chptr) && !is_chan_op(sptr, chptr) &&
						  is_chan_op(acptr, chptr)) {
							sendto_one(sptr, ":%s NOTICE %s :*** Cannot kick %s - "
								"you are only a halfop", me.name, parv[0], user);
							continue;
						}
					}
				}
				sendto_channel_butserv(chptr, sptr, ":%s KICK %s %s :%s", parv[0], name,
					acptr->name, comment);
				sendto_match_servs(chptr, cptr, ":%s KICK %s %s :%s", parv[0], name,
					acptr->name, comment);
				remove_user_from_channel(acptr, chptr);
			}
			else {
				sendto_one(sptr, err_str(ERR_USERNOTINCHANNEL), me.name, parv[0], user, name);
			}
		}
	}
	return 0;
}
