/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_knock.c: KNOCK command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_knock.c,v 1.4 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_knock(aClient *, aClient *, int, char **);

extern void clean_channelname(unsigned char *);
extern int check_channelmask(aClient *, aClient *, char *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_knock.so",
	"/KNOCK command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("KNOCK", m_knock, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("KNOCK", m_knock);
}
#endif

/*
 * m_knock
 *	parv[0] = sender prefix
 *	parv[1] = channel name
 */
int m_knock(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;

	if (parc < 2 || BadPtr(parv[1])) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "KNOCK");
		return 0;
	}
	if (MyConnect(sptr)) {
		clean_channelname(parv[1]);
	}
	if (check_channelmask(sptr, cptr, parv[1])) {
		return 0;
	}

	if (!IsChannelName(parv[1])) {
		sendto_one(sptr, ":%s NOTICE %s :*** %s is not a valid channel name", me.name,
			parv[0], parv[1]);
		return 0;
	}

	chptr = find_channel(parv[1], NullChn);
	if (!chptr) {
		sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL), me.name, parv[0], parv[1]);
		return 0;
	}

	if (IsMember(sptr, chptr)) {
		sendto_one(sptr, ":%s NOTICE %s :*** Already on channel %s", me.name, parv[0],
			chptr->chname);
		return 0;
	}

	if (!(chptr->mode.mode & MODE_INVITEONLY) && !*chptr->mode.key && (!chptr->mode.limit
	  || (chptr->mode.limit && chptr->users < chptr->mode.limit))) {
		sendto_one(sptr, ":%s NOTICE %s :*** Channel %s is open!", me.name, parv[0],
			chptr->chname);
		return 0;
	}

	if ((chptr->mode.mode & MODE_PRIVATE) || is_banned(sptr, chptr)) {
		sendto_one(sptr, err_str(ERR_CANNOTSENDTOCHAN), me.name, parv[0], chptr->chname);
		return 0;
	}

	if (chptr->last_knock + ConfigSet.knockdelay > NOW) {
		sendto_one(sptr, ":%s NOTICE %s :*** Wait %d seconds before another knock to %s",
			me.name, parv[0], (int)ConfigSet.knockdelay - (NOW - chptr->last_knock),
			chptr->chname);
		return 0;
	}

	chptr->last_knock = NOW;
	sendto_channelflag_butone(CHFL_CHANOP, cptr, sptr, chptr, ":%s NOTICE @%s :KNOCK: %s "
		"(%s@%s) has requested an invite", me.name, chptr->chname, parv[0],
		sptr->user->username, IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
	sendto_one(sptr, ":%s NOTICE %s :*** Your knock has been delivered.", me.name, parv[0]);
	return 0;
}
