/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_links.c: LINKS command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_links.c,v 1.7 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_links(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_links.so",
	"/LINKS command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("LINKS", m_links, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("LINKS", m_links);
}
#endif

/*
 * m_links
 *	parv[0] = sender prefix
 *	parv[1] = servername mask
 * OR
 *	parv[0] = sender prefix
 *	parv[1] = server to query
 *	parv[2] = servername mask
 */
int m_links(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	static time_t last_used = 0L;
	char *mask, *s, *d;
	char clean_mask[(2 * HOSTLEN) + 1];
	aClient *acptr;
	int n;

	if (parc > 2) {
		if (hunt_server(cptr, sptr, ":%s LINKS %s :%s", 1, parc, parv) != HUNTED_ISME) {
			return 0;
		}
		mask = parv[2];
	}
	else {
		mask = parc < 2 ? NULL : parv[1];
	}

	if (!IsOper(sptr)) {
		if (!MyConnect(sptr)) {
			return 0;
		}
		if (last_used + ConfigSet.pacetime > NOW) {
			sendto_one(sptr, rpl_str(RPL_LOAD2HI), me.name, parv[0]);
			return 0;
		}
		else {
			last_used = NOW;
		}
		if (IsPerson(sptr)) {
			sendto_realops_lev(SPY_LEV, "LINKS requested by %s (%s@%s) [%s]",
				sptr->name, sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host,
				sptr->user->server);
		}
	}

	if (mask) {
		s = mask;
		d = clean_mask;
		n = (2 * HOSTLEN) - 2;

		while (*s && n > 0) {
			if ((unsigned char)*s < (unsigned char)' ') {
				*d++ = '^';
				*d++ = (char)((unsigned char)*s + 0x40);
				s++;
				n -= 2;
			}
			else if ((unsigned char)*s > (unsigned char)'~') {
				*d++ = '.';
				s++;
				n--;
			}
			else {
				*d++ = *s++;
				n--;
			}
		}
		*d = '\0';
	}

	for (acptr = client, (void)collapse(mask); acptr; acptr = acptr->next) {
		if (!IsServer(acptr) && !IsMe(acptr)) {
			continue;
		}
		if (!BadPtr(mask) && match(mask, acptr->name)) {
			continue;
		}
		if (ConfigSet.hideulines && !IsOper(sptr) && !IsULine(acptr)) {
			continue;
		}
		sendto_one(sptr, rpl_str(RPL_LINKS), me.name, parv[0], acptr->name,
			acptr->serv->up, acptr->hopcount,
			acptr->info[0] ? acptr->info : "IRCers United");
	}
	sendto_one(sptr, rpl_str(RPL_ENDOFLINKS), me.name, parv[0],
		BadPtr(mask) ? "*" : clean_mask);
	return 0;
}
