/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_map.c: MAP command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_map.c,v 1.12 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_map(aClient *, aClient *, int, char **);

static void dump_map(aClient *, aClient *, char *, int, int);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_map.so",
	"/MAP command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("MAP", m_map, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("MAP", m_map);
}
#endif

/*
 * m_map
 *	parv[0] = sender prefix
 *	parv[1] = server mask
 */
int m_map(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	int longest = strlen(me.name);
	static time_t last_used = 0L;

	if (parc < 2) {
		parv[1] = "*";
	}

	if (!IsOper(sptr)) {
		if (last_used + ConfigSet.pacetime > NOW) {
			sendto_one(sptr, rpl_str(RPL_LOAD2HI), me.name, parv[0]);
			return 0;
		}
		else {
			last_used = NOW;
		}
		if (IsPerson(sptr)) {
			sendto_realops_lev(SPY_LEV, "MAP requested by %s (%s@%s) [%s]", sptr->name,
				sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host,
				sptr->user->server);
		}
	}

	for (acptr = client; acptr; acptr = acptr->next) {
		if (IsServer(acptr) && (strlen(acptr->name) + acptr->hopcount * 2) > longest) {
			longest = strlen(acptr->name) + acptr->hopcount * 2;
		}
	}

	if (longest > 60) {
		longest = 60;
	}
	longest += 2;
	dump_map(sptr, &me, parv[1], 0, longest);
	sendto_one(sptr, rpl_str(RPL_MAPEND), me.name, parv[0]);
	return 0;
}

static void dump_map(aClient *cptr, aClient *server, char *mask, int prompt_length, int length)
{
	static char prompt[64];
	char *p = &prompt[prompt_length];
	int count = 0, local = 0;
	aClient *acptr;

	*p = '\0';

	if (prompt_length > 60) {
		sendto_one(cptr, rpl_str(RPL_MAPMORE), me.name, cptr->name, prompt, server->name);
	}
	else {
		for (acptr = client; acptr; acptr = acptr->next) {
			if (IsPerson(acptr)) {
				++count;
				if (!strcasecmp(acptr->user->server, server->name)) {
					++local;
				}
			}
		}
		sendto_one(cptr, rpl_str(RPL_MAP), me.name, cptr->name, prompt, length, server->name, local,
			(local * 100) / count);
		count = 0;
	}

	if (prompt_length > 0) {
		p[-1] = ' ';
		if (p[-2] == '`') {
			p[-2] = ' ';
		}
	}
	if (prompt_length > 60) {
		return;
	}

	strcpy(p, "|-");

	for (acptr = client; acptr; acptr = acptr->next) {
		if (!IsServer(acptr) || strcasecmp(acptr->serv->up, server->name)) {
			continue;
		}
		if (match(mask, acptr->name)) {
			acptr->flags &= ~FLAGS_MAP;
		}
		else {
			acptr->flags |= FLAGS_MAP;
			count++;
		}
	}

	for (acptr = client; acptr; acptr = acptr->next) {
		if (!(acptr->flags & FLAGS_MAP) || !IsServer(acptr) || strcasecmp(acptr->serv->up, server->name)) {
			continue;
		}
		if (--count == 0) {
			*p = '`';
		}
		dump_map(cptr, acptr, mask, prompt_length + 2, length - 2);
	}

	if (prompt_length > 0) {
		p[-1] = '-';
	}
}
