/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_mkpasswd.c: MKPASSWD command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_mkpasswd.c,v 1.9 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_mkpasswd(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_mkpasswd.so",
	"/MKPASSWD command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("MKPASSWD", m_mkpasswd, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("MKPASSWD", m_mkpasswd);
}
#endif

int m_mkpasswd(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (!ConfigSet.encryptedpasswords) {
		sendto_one(sptr, ":%s NOTICE %s :Encryption is not enabled on this server.", me.name, parv[0]);
	}
	else {
		static char saltChars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";
		extern char *crypt();
		char salt[3];
		int usable = 0;

		if (parc > 1 && strlen(parv[1]) >= 1) {
			usable = 1;
		}
		if (!usable) {
			sendto_one(sptr, ":%s NOTICE %s :Text to encrypt must be atleast one character in length.",
				me.name, parv[0]);
			return 0;
		}

		srandom(time(0));
		salt[0] = saltChars[random() % 64];
		salt[1] = saltChars[random() % 64];
		salt[2] = 0;

		if (!strchr(saltChars, salt[0]) || !strchr(saltChars, salt[1])) {
			sendto_one(sptr, ":%s NOTICE %s :%s is not a legal encryption salt.", me.name, parv[0], salt);
			return 0;
		}

		sendto_one(sptr, ":%s NOTICE %s :Encryption for \"%s\": %s", me.name, parv[0], parv[1],
			crypt(parv[1], salt));
	}
	return 0;
}
