/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_names.c: NAMES command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_names.c,v 1.7 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_names(aClient *, aClient *, int, char **);

extern void send_names(aClient *, aChannel *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_names.so",
	"/NAMES command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("NAMES", m_names, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("NAMES", m_names);
}
#endif

/*
 * m_names
 *	parv[0] = sender prefix
 *	parv[1] = channel
 */
int m_names(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;
	char *s, *para = parc > 1 ? parv[1] : NULL;

	if (parc < 2 || !MyConnect(sptr)) {
		sendto_one(sptr, rpl_str(RPL_ENDOFNAMES), me.name, parv[0], "*");
		return 0;
	}

	for (s = para; *s; s++) {
		if (*s == ',') {
			if (strlen(para) > 64) {
				para[64] = '\0';
			}
			sendto_realops_lev(FLOOD_LEV, "Possible names abuser: %s (%s@%s) target(s): %s",
				sptr->name, sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host,
				para);
			sendto_one(sptr, err_str(ERR_TOOMANYTARGETS), me.name, sptr->name, "NAMES");
			return 0;
		}
	}

	if (!check_channelname(sptr, (unsigned char *)para)) {
		return 0;
	}

	chptr = find_channel(para, NullChn);
	if (!chptr || !ShowChannel(sptr, chptr)) {
		sendto_one(sptr, rpl_str(RPL_ENDOFNAMES), me.name, parv[0], para);
		return 0;
	}
	send_names(sptr, chptr);
	return 0;
}
