/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_opermotd.c: OPERMOTD command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_opermotd.c,v 1.6 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_opermotd(aClient *, aClient *, int, char **);

extern void init_opermotd(int);
extern aMotd *opermotd;
extern char opermotd_changed_date[34];
extern struct tm *opermotd_tm;

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_opermotd.so",
	"/OPERMOTD command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	init_opermotd(1);
	add_Command("OPERMOTD", m_opermotd, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("OPERMOTD", m_opermotd);
	init_opermotd(0);
}
#endif

/*
 * m_opermotd
 *	parv[0] = sender prefix
 *	parv[1] = server
 */
int m_opermotd(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (hunt_server(cptr, sptr, ":%s OPERMOTD :%s", 1, parc, parv) != HUNTED_ISME) {
		return 0;
	}

	if (!IsOper(sptr) && MyClient(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	send_file(sptr, &opermotd, opermotd_tm, opermotd_changed_date, "OPERMOTD",
		"IRC Operator Message of the Day", 0);
	return 0;
}
