/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_part.c: PART command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_part.c,v 1.8 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_part(aClient *, aClient *, int, char **);

#ifdef ANTI_SPAMBOT
/* SPAMBOT stuff, defined in src/s_serv.c */
extern int spam_num;
extern int spam_time;
#endif

extern aChannel *get_channel(aClient *, char *, int);
extern int can_send(aClient *, aChannel *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_part.so",
	"/PART command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("PART", m_part, 0, 15, 1, 0, 1, 0L);
}
void _module_unload(void) {
	del_Command("PART", m_part);
}
#endif

/*
 * m_part
 *	parv[0] = sender prefix
 *	parv[1] = channel name
 *	parv[2] = message
 */
int m_part(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;
	char *p, *name, *reason = (parc > 2 && parv[2]) ? parv[2] : NULL;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "PART");
		return 0;
	}

	name = strtoken(&p, parv[1], ",");
#ifdef ANTI_SPAMBOT
	if (name && MyConnect(sptr) && !IsOper(sptr)) {
		if (sptr->join_leave_count >= spam_num) {
			sendto_ops_lev(SPAM_LEV, "User %s (%s@%s) is a possible spambot",
				sptr->name, sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
			sptr->oper_warn_count_down = OPER_SPAM_COUNTDOWN;
		}
		else {
			int t_delta = (NOW - sptr->last_leave_time);
			if (t_delta > JOIN_LEAVE_COUNT_EXPIRE_TIME) {
				int decrement_count = (t_delta / JOIN_LEAVE_COUNT_EXPIRE_TIME);
				if (decrement_count > sptr->join_leave_count) {
					sptr->join_leave_count = 0;
				}
				else {
					sptr->join_leave_count -= decrement_count;
				}
			}
			else if ((NOW - sptr->last_join_time) < spam_time) {
				sptr->join_leave_count++;
			}
			sptr->last_leave_time = NOW;
		}
	}
#endif /* ANTI_SPAMBOT */

	while (name) {
		chptr = get_channel(sptr, name, 0);
		if (!chptr) {
			sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL), me.name, parv[0], name);
			name = strtoken(&p, NULL, ",");
			continue;
		}
		if (!IsMember(sptr, chptr)) {
			sendto_one(sptr, err_str(ERR_NOTONCHANNEL), me.name, parv[0], name);
			name = strtoken(&p, NULL, ",");
			continue;
		}

		if (parc < 3 || can_send(sptr, chptr)) {
			sendto_match_servs(chptr, cptr, ":%s PART %s", parv[0], name);
			sendto_channel_butserv(chptr, sptr, ":%s PART %s", parv[0], name);
		}
		else {
			sendto_match_servs(chptr, cptr, ":%s PART %s :%s", parv[0], name, reason);
			sendto_channel_butserv(chptr, sptr, ":%s PART %s :%s", parv[0], name, reason);
		}
		remove_user_from_channel(sptr, chptr);
		name = strtoken(&p, NULL, ",");
	}
	return 0;
}
