/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_pass.c: PASS protocol module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_pass.c,v 1.8 2002/04/08 16:00:47 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_pass(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_pass.so",
	"PASS protocol",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("PASS", m_pass, 0, 15, 1, 1, 0, 0L);
}
void _module_unload(void) {
	del_Command("PASS", m_pass);
}
#endif

/*
 * m_pass
 *	parv[0] = sender prefix
 *	parv[1] = password
 *	parv[2] = optional version information
 */
int m_pass(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *password = parc > 1 ? parv[1] : NULL;

	if (BadPtr(password)) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "PASS");
		return 0;
	}
	if (!MyConnect(sptr) || (!IsUnknown(cptr) && !IsHandshake(cptr))) {
		sendto_one(cptr, err_str(ERR_ALREADYREGISTRED), me.name, parv[0]);
		return 0;
	}

	strncpyzt(cptr->passwd, password, sizeof(cptr->passwd));

	if (parc > 2) {
		int len = strlen(parv[2]);
		if (len > 1) {
			if (parv[2][0] == 'T' && parv[2][1] == 'S') {
				cptr->tsinfo = (ts_val)TS_DOESTS;
			}
		}
	}
	return 0;
}
