/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_pong.c: PONG command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_pong.c,v 1.11 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_pong(aClient *, aClient *, int, char **);

extern void send_topic_burst(aClient *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_pong.so",
	"/PONG command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("PONG", m_pong, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("PONG", m_pong);
}
#endif

/*
 * m_pong
 *	parv[0] = sender prefix
 *	parv[1] = origin
 *	parv[2] = destination
 */
int m_pong(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	char *origin, *destination;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NOORIGIN), me.name, parv[0]);
		return 0;
	}

	origin = parv[1];
	destination = parv[2];
	cptr->flags &= ~FLAGS_PINGSENT;
	sptr->flags &= ~FLAGS_PINGSENT;

	if (sptr == cptr && IsServer(cptr)) {
		if (sptr->flags & FLAGS_USERBURST) {
			sptr->flags &= ~FLAGS_USERBURST;
			sendto_gnotice("%s has processed user/channel burst, sending topic burst.",
				sptr->name);
			send_topic_burst(sptr);
			sptr->flags |= FLAGS_PINGSENT|FLAGS_SOBSENT;
			sendto_one(sptr, "PING :%s", me.name);
		}
		else if (sptr->flags & FLAGS_TOPICBURST) {
			sptr->flags &= ~FLAGS_TOPICBURST;
			sendto_gnotice("%s has processed topic burst, synch complete. (capabilities:%s%s%s)",
				sptr->name,
				IsSSJv2(sptr) ? " SSJv2" : IsSSJOIN(cptr) ? " SSJOIN" : "",
				IsNICKIP(sptr) ? " NICKIP" : "",
				IsSENDVHOST(sptr) ?  " SENDVHOST" : "");
			if (ConfigSet.hub) {
				sendto_serv_butone(sptr, ":%s GNOTICE :%s has processed topic burst, synch "
					"completed. (capabilities:%s%s%s)", me.name, sptr->name,
					IsSSJv2(sptr) ? " SSJv2" : IsSSJOIN(cptr) ? " SSJOIN" : "",
					IsNICKIP(sptr) ? " NICKIP" : "",
					IsSENDVHOST(cptr) ? " SENDVHOST" : "");
			}
		}
	}

	if (!BadPtr(destination) && mycmp(destination, me.name) != 0 && IsRegistered(sptr)) {
		acptr = find_client(destination, NULL);
		if (!acptr) {
			acptr = find_server(destination, NULL);
			if (!acptr) {
				sendto_one(sptr, err_str(ERR_NOSUCHSERVER), me.name, parv[0],
					destination);
				return 0;
			}
		}
		sendto_one(acptr, ":%s PONG %s %s", parv[0], origin, destination);
	}

#ifdef DEBUGMODE
	else {
		Debug((DEBUG_NOTICE, "PONG: %s %s", origin, destination ? destination : "*"));
	}
#endif /* DEBUGMODE */
	return 0;
}
