/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_quit.c: QUIT command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_quit.c,v 1.6 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_quit(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_quit.so",
	"/QUIT command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("QUIT", m_quit, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("QUIT", m_quit);
}
#endif

/*
 * m_quit
 *	parv[0] = sender prefix
 *	parv[1] = comment
 */
int m_quit(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *reason = (parc > 1 && *parv[1]) ? parv[1] : parv[0];
	char comment[TOPICLEN + 1];

	sptr->flags |= FLAGS_NORMALEX;

	if (!IsServer(cptr)) {
		ircsprintf(comment, "Quit: ");
		strncpy(comment + 6, reason, TOPICLEN - 6);
		comment[TOPICLEN] = '\0';
		return exit_client(cptr, sptr, sptr, comment);
	}
	else {
		return exit_client(cptr, sptr, sptr, reason);
	}
	return 0;
}
