/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_rakill.c: RAKILL command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_rakill.c,v 1.7 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_rakill(aClient *, aClient *, int, char **);

extern void send_globops(char *, ...);
extern ConfigItem *del_ConfigItem(ConfigItem *, ConfigItem **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_rakill.so",
	"/RAKILL command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("RAKILL", m_rakill, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("RAKILL", m_rakill);
}
#endif

/*
 * m_rakill
 *	parv[0] = sender prefix
 *	parv[1] = host
 *	parv[2] = user
 */
int m_rakill(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char mask[USERLEN + HOSTLEN + 2];
	ConfigItem_ban *ban;

	if (!IsServer(sptr) || parc < 3) {
		return 0;
	}
	if (!IsULine(sptr)) {
		send_globops("Non-ULined server %s is trying to RAKILL!", parv[0]);
		sendto_serv_butone(&me, ":%s GLOBOPS :Non-ULined server %s is trying "
			"to RAKILL!", me.name, parv[0]);
		return 0;
	}

	ircsprintf(mask, "%s@%s", parv[2], parv[1]);
	ban = find_ban(BAN_AUTOKILL, mask);
	if (ban) {
		del_ConfigItem((ConfigItem *)ban, (ConfigItem **)&config_ban);
		MyFree(ban->mask);
		MyFree(ban->reason);
		MyFree(ban);
	}
	sendto_serv_butone(cptr, ":%s RAKILL %s %s", parv[0], parv[1], parv[2]);
	return 0;
}
