/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_rehash.c: REHASH command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_rehash.c,v 1.7 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "nameser.h"
#include "resolv.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_rehash(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_rehash.so",
	"/REHASH command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("REHASH", m_rehash, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("REHASH", m_rehash);
}
#endif

/*
 * m_rehash
 *	parv[0] = sender prefix
 *	parv[1] = options
 */
int m_rehash(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int all = 0;

	if (!OPCanRehash(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	if (parc > 1) {
		if (!mycmp(parv[1], "-dns")) {
			sendto_one(sptr, rpl_str(RPL_REHASHING), me.name, parv[0], "DNS");
			sendto_ops("%s is rehashing DNS CACHE", parv[0]);
			flush_cache();
			res_init();
		}
		else if (!mycmp(parv[1], "-iphash")) {
			sendto_one(sptr, rpl_str(RPL_REHASHING), me.name, parv[0], "IPHASH");
			sendto_ops("%s is rehashing IPHASH", parv[0]);
			rehash_ip_hash();
		}
		else {
			sendto_one(sptr, ":%s NOTICE %s :*** %s is not a known rehash option, reloading everything.",
				me.name, parv[0], parv[1]);
			all = 1;
		}
	}
	else {
		all = 1;
	}

	if (all) {
		sendto_one(sptr, rpl_str(RPL_REHASHING), me.name, parv[0], "Server Configuration Files");
		sendto_ops("%s is rehashing Server Configuration Files", parv[0]);
		ircdlog(LOG_DEFAULT, "rehashing by request of %s", get_client_name(cptr, FALSE));
		return (rehash(cptr, sptr, (parc > 1) ? ((*parv[1] == 'q') ? 2 : 0) : 0));
	}
	return 0;
}
