/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_restart.c: RESTART command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_restart.c,v 1.10 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_restart(aClient *, aClient *, int, char **);
static char buf[BUFSIZE];

extern void restart(char *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_restart.so",
	"/RESTART command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("RESTART", m_restart, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("RESTART", m_restart);
}
#endif

/*
 * m_restart
 *	parv[0] = sender prefix
 *	parv[1] = password
 */
int m_restart(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (!IsOper(sptr) || !OPCanRestart(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	if (config_password && config_password->restart) {
		if (parc < 2 || *parv[1] == '\0') {
			sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "RESTART");
			return 0;
		}
		if (strcmp(config_password->restart, parv[1])) {
			sendto_one(sptr, err_str(ERR_PASSWDMISMATCH), me.name, parv[0]);
			ircdlog(LOG_DEFAULT, "%s attempted to restart server: password mismatch",
				get_client_name(sptr, FALSE));
			return 0;
		}
	}
	ircsprintf(buf, "Server restarting by request of %s.", get_client_name(sptr, HIDEME));
	ircdlog(LOG_DEFAULT, "server restarting by request of %s", get_client_name(sptr, FALSE));
	restart(buf);
	return 0;
}
