/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_sendvhost.c: SENDVHOST protocol module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_sendvhost.c,v 1.6 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_sendvhost(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_sendvhost.so",
	"SENDVHOST protocol",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("SENDVHOST", m_sendvhost, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("SENDVHOST", m_sendvhost);
}
#endif

/*
 * m_sendvhost
 *	parv[0] = sender prefix
 *	parv[1] = masked hostname to send
 */
int m_sendvhost(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (!IsServer(cptr)) {
		return 0;
	}
	if (*parv[1] != '*') {
		if (sptr->user->maskedhost) {
			MyFree(sptr->user->maskedhost);
		}
		sptr->user->maskedhost = MyMalloc(strlen(parv[1]) + 1);
		strcpy(sptr->user->maskedhost, parv[1]);
	}
	else {
		do_maskedhost(sptr);
	}
	sendto_serv_butone(cptr, ":%s SENDVHOST :%s", sptr->name, *parv[1] != '*' ? parv[1] : "*");
	return 0;
}
