/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_set.c: SET command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_set.c,v 1.8 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_set(aClient *, aClient *, int, char **);

#ifdef FLUD
extern int flud_num, flud_time, flud_block;
#endif
#ifdef ANTI_SPAMBOT
extern int spam_num, spam_time;
#endif

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_set.so",
	"/SET command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("SET", m_set, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("SET", m_set);
}
#endif

/*
 * m_set
 *	parv[0] = sender prefix
 *	parv[1] = directive
 *	parv[2] = optional new value
 */
int m_set(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (!MyConnect(sptr) && !IsCoAdmin(sptr) && !IsAdmin(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}
	if (parc > 2) {
		if (!strcasecmp(parv[1], "MAXCLIENTS")) {
			int val = atoi(parv[2]);
			if (val < 32) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set MAXCLIENTS < 32 (%d:%d)", me.name,
					parv[0], ConfigSet.maxclients, highest_fd);
				return 0;
			}
			if (val > MASTER_MAX) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set MAXCLIENTS > MASTER_MAX (%d)", me.name,
					parv[0], MASTER_MAX);
				return 0;
			}
			ConfigSet.maxclients = val;
			sendto_realops("%s (%s@%s) has changed MAXCLIENTS to %d", parv[0], sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host, ConfigSet.maxclients);
			return 0;
		}
		if (!strcasecmp(parv[1], "MAXCHANSPERUSER")) {
			int val = atoi(parv[2]);
			if (val < 1) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set MAXCHANSPERUSER < 1", me.name,
					parv[0]);
				return 0;
			}
			ConfigSet.maxchansperuser = val;
			sendto_realops("%s (%s@%s) has changed MAXCHANSPERUSER to %d", parv[0], sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host, ConfigSet.maxchansperuser);
			return 0;
		}
		if (!strcasecmp(parv[1], "PACETIME")) {
			time_t val = atime(parv[2]);
			if (val < 0) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set PACETIME < 0", me.name, parv[0]);
				return 0;
			}
			ConfigSet.pacetime = val;
			if (ConfigSet.pacetime == 0) {
				sendto_realops("%s (%s@%s) has disabled COMMAND flood protection",
					parv[0], sptr->user->username,
					IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
			}
			else {
				sendto_realops("%s (%s@%s) has changed PACETIME to %s", parv[0], sptr->user->username,
					IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host,
					parv[2]);
			}
			return 0;
		}
		if (!strcasecmp(parv[1], "KNOCKDELAY")) {
			time_t val = atime(parv[2]);
			if (val < 0) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set KNOCKDELAY < 0", me.name, parv[0]);
				return 0;
			}
			ConfigSet.knockdelay = val;
			if (ConfigSet.knockdelay == 0) {
				sendto_realops("%s (%s@%s) has disabled KNOCK flood protection", parv[0],
					sptr->user->username,
					IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
			}
			else {
				sendto_realops("%s (%s@%s) has changed KNOCKDELAY to %s", parv[0],
					sptr->user->username,
					IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
			}
			return 0;
		}
		if (!strcasecmp(parv[1], "FLOODCOUNT")) {
			int val = atoi(parv[2]);
			if (val < 0) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set FLOODCOUNT < 0", me.name, parv[0]);
				return 0;
			}
			ConfigSet.floodcount = val;
			if (ConfigSet.floodcount == 0) {
				sendto_realops("%s (%s@%s) has disabled MESSAGE flood protection", parv[0],
					sptr->user->username,
					IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
			}
			else {
				sendto_realops("%s (%s@%s) has changed FLOODCOUNT to %d", parv[0],
					sptr->user->username,
					IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host,
					ConfigSet.floodcount);
			}
			return 0;
		}
#ifdef FLUD
		if (!strcasecmp(parv[1], "FLUDNUM")) {
			int val = atoi(parv[2]);
			if (val < 1) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set FLUDNUM < 1", me.name, parv[0]);
				return 0;
			}
			flud_num = val;
			sendto_realops("%s (%s@%s) has changed FLUDNUM to %d", parv[0], sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host,
				flud_num);
			return 0;
		}
		if (!strcasecmp(parv[1], "FLUDTIME")) {
			int val = atoi(parv[2]);
			if (val < 1) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set FLUDTIME < 1", me.name, parv[0]);
				return 0;
			}
			flud_time = val;
			sendto_realops("%s (%s@%s) has changed FLUDTIME to %d", parv[0], sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host,
				flud_time);
			return 0;
		}
		if (!strcasecmp(parv[1], "FLUDBLOCK")) {
			int val = atoi(parv[2]);
			if (val < 0) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set FLUDBLOCK < 0", me.name, parv[0]);
				return 0;
			}
			flud_block = val;
			if (flud_block == 0) {
				sendto_realops("%s (%s@%s) has disabled FLUD protection", parv[0], sptr->user->username,
					IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
			}
			else {
				sendto_realops("%s (%s@%s) has changed FLUDBLOCK to %d", parv[0], sptr->user->username,
					IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
			}
			return 0;
		}
#endif
#ifdef ANTI_SPAMBOT
		if (!strcasecmp(parv[1], "SPAMNUM")) {
			int val = atoi(parv[2]);
			if (val < MIN_SPAM_NUM) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set SPAMNUM < %d", me.name, parv[0],
					MIN_SPAM_NUM);
				return 0;
			}
			spam_num = val;
			sendto_realops("%s (%s@%s) has changed SPAMNUM to %d", parv[0], sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host, spam_num);
			return 0;
		}
		if (!strcasecmp(parv[1], "SPAMTIME")) {
			int val = atoi(parv[2]);
			if (val < MIN_SPAM_TIME) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set SPAMTIME < %d", me.name, parv[0],
					MIN_SPAM_NUM);
				return 0;
			}
			spam_time = val;
			sendto_realops("%s (%s@%s) has changed SPAMTIME to %d", parv[0], sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host, spam_time);
			return 0;
		}
#endif
	}
	sendto_one(sptr, ":%s NOTICE %s :The following options are available:", me.name, parv[0]);
	sendto_one(sptr, ":%s NOTICE %s :MAXCLIENTS(%d) MAXCHANSPERUSER(%d)", me.name, parv[0],
		ConfigSet.maxclients, ConfigSet.maxchansperuser);
	sendto_one(sptr, ":%s NOTICE %s :PACETIME(%ld) KNOCKDELAY(%ld) FLOODCOUNT(%d)", me.name,
		parv[0], ConfigSet.pacetime, ConfigSet.knockdelay, ConfigSet.floodcount);
#ifdef FLUD
	sendto_one(sptr, ":%s NOTICE %s :FLUDNUM(%d) FLUDTIME(%d) FLUDBLOCK(%d)", me.name, parv[0],
		flud_num, flud_time, flud_block);
#endif
#ifdef ANTI_SPAMBOT
	sendto_one(sptr, ":%s NOTICE %s :SPAMNUM(%d) SPAMTIME(%d)", me.name, parv[0], spam_num,
		spam_time);
#endif
	return 0;
}
