/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_silence.c: SILENCE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_silence.c,v 1.5 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_silence(aClient *, aClient *, int, char **);

extern int add_silence(aClient *, char *);
extern int del_silence(aClient *, char *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_silence.so",
	"/SILCENCE command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("SILENCE", m_silence, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("SILENCE", m_silence);
}
#endif

/*
 * m_silence
 *	parv[0] = sender prefix
 *	If local client:
 *		parv[1] = mask (NULL sends the list)
 *	If remote client:
 *		parv[1] = nick to be silenced
 *		parv[2] = mask
 */
int m_silence(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;
	Link *lp;
	char c, *cp;

	if (check_registered_user(sptr)) {
		return 0;
	}

	if (MyClient(sptr)) {
		acptr = sptr;
		if (parc < 2 || *parv[1] == '\0' || (acptr = find_person(parv[1], NULL))) {
			if (!acptr->user) {
				return 0;
			}
			for (lp = acptr->user->silence; lp; lp = lp->next) {
				sendto_one(sptr, rpl_str(RPL_SILELIST), me.name, sptr->name,
					acptr->name, lp->value.cp);
			}
			sendto_one(sptr, rpl_str(RPL_ENDOFSILELIST), me.name, acptr->name);
			return 0;
		}

		cp = parv[1];
		c = *cp;

		if (c == '-' || c == '+') {
			cp++;
		}
		else if (!(strchr(cp, '@') || strchr(cp, '.') || strchr(cp, '!') || strchr(cp, '*'))) {
			sendto_one(sptr, err_str(ERR_NOSUCHNICK), me.name, parv[0], parv[1]);
			return 0;
		}
		else {
			c = '+';
		}
		cp = pretty_mask(cp);

		if ((c == '-' && !del_silence(sptr, cp)) || (c != '-' && !add_silence(sptr, cp))) {
			sendto_prefix_one(sptr, sptr, ":%s SILENCE %c%s", parv[0], c, cp);
			if (c == '-') {
				sendto_serv_butone(NULL, ":%s SILENCE * -%s", sptr->name, cp);
			}
		}
	}
	else if (parc < 3 || *parv[2] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "SILENCE");
	}
	else if ((c = *parv[2]) == '-' || (acptr = find_person(parv[1], NULL))) {
		if (c == '-') {
			if (!del_silence(sptr, parv[2] + 1)) {
				sendto_serv_butone(cptr, ":%s SILENCE %s :%s", parv[0], parv[1],
					parv[2]);
			}
		}
		else {
			add_silence(sptr, parv[2]);
			if (!MyClient(acptr)) {
				sendto_one(acptr, ":%s SILENCE %s :%s", parv[0], parv[1], parv[2]);
			}
		}
	}
	else {
		sendto_one(sptr, err_str(ERR_NOSUCHNICK), me.name, parv[0], parv[1]);
	}
	return 0;
}
