/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_sqline.c: SQLINE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_sqline.c,v 1.11 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_sqline(aClient *, aClient *, int, char **);

extern void send_globops(char *, ...);
extern void add_ConfigItem(ConfigItem *, ConfigItem **);
extern char *collapse(char *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_sqline.so",
	"/SQLINE command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("SQLINE", m_sqline, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("SQLINE", m_sqline);
}
#endif

/*
 * m_sqline
 *	parv[0] = sender prefix
 *	parv[1] = nickname
 *	parv[2] = reason
 */
int m_sqline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	ConfigItem_ban *ban;

	if (!(IsServer(sptr) || IsULine(sptr))) {
		return 0;
	}
	if (parc < 2) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "SQLINE");
		return 0;
	}

	parv[1] = collapse(parv[1]);
	ban = find_ban(BAN_NICKNAME|BAN_NICKNAME_S, parv[1]);
	if (!ban) {
		ban = (ConfigItem_ban *)MyMalloc(sizeof(ConfigItem_ban));
		ban->type = (BAN_NICKNAME|BAN_NICKNAME_S);
		DupString(ban->mask, parv[1]);
		DupString(ban->reason, !BadPtr(parv[2]) ? parv[2] : "Reserved");
		ban->flag.temporary = 0;
		add_ConfigItem((ConfigItem *)ban, (ConfigItem **)&config_ban);
	}
	sendto_serv_butone(cptr, ":%s SQLINE %s :%s", parv[0], parv[1], parv[2]);
	return 0;
}
