/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_squit.c: SQUIT command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_squit.c,v 1.7 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_squit(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_squit.so",
	"/SQUIT command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("SQUIT", m_squit, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("SQUIT", m_squit);
}
#endif

/*
 * m_squit
 *	parv[0] = sender prefix
 *	parv[1] = servername
 *	parv[2] = optional comment
 */
int m_squit(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *server, *comment = (parc > 2) ? parv[2] : sptr->name;
	aClient *acptr = NULL;

	if (!IsPrivileged(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	if (parc > 1) {
		server = parv[1];
		for (acptr = client; (acptr = next_client(acptr, server)); acptr = acptr->next) {
			if (IsServer(acptr) || IsMe(acptr)) {
				break;
			}
		}
		if (acptr && IsMe(acptr)) {
			acptr = cptr;
			server = cptr->sockhost;
		}
	}
	else {
		server = cptr->sockhost;
		acptr = cptr;
	}
	if (!acptr) {
		sendto_one(sptr, err_str(ERR_NOSUCHSERVER), me.name, parv[0], server);
		return 0;
	}
	if (MyClient(sptr) && ((!OPCanLRoute(sptr) && MyConnect(acptr)) || (!OPCanGRoute(sptr) &&
	  !MyConnect(acptr)))) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	if (MyConnect(acptr)) {
		sendto_gnotice("Received SQUIT %s from %s (%s)", acptr->name,
			get_client_name(sptr, HIDEME), comment);
		sendto_serv_butone(NULL, ":%s GNOTICE :Received SQUIT %s from %s (%s)", me.name,
			acptr->name, get_client_name(sptr, HIDEME), comment);
		ircdlog(LOG_SERVER, "SQUIT from %s: %s (%s)", parv[0], server, comment);

		if (cptr == sptr) {
			exit_client(&me, acptr, sptr, comment);
			return FLUSH_BUFFER;
		}
		return exit_client(&me, acptr, sptr, comment);
	}

	if (sptr->from == acptr->from) {
		sendto_ops_lev(DEBUG_LEV, "Exiting server %s due to upstream SQUIT by %s [%s]",
			acptr->name, sptr->name, comment);
		return exit_client(cptr, acptr, sptr, comment);
	}
	if (!IsUnconnect(acptr->from)) {
		sendto_ops_lev(DEBUG_LEV, "Exiting server %s due to non-unconnect server %s [%s]",
			acptr->name, acptr->from->name, comment);
		return exit_client(&me, acptr, sptr, comment);
	}

	sendto_ops_lev(DEBUG_LEV, "Passing along SQUIT for %s by %s [%s]", acptr->name, sptr->name,
		comment);
	sendto_one(acptr->from, ":%s SQUIT %s :%s", parv[0], acptr->name, comment);
	return 0;
}
