/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_stats.c: STATS command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_stats.c,v 1.23 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_stats(aClient *, aClient *, int, char **);

extern int max_connection_count;
extern int max_client_count;
extern void show_oper_serv_fdlists(aClient *);
extern void count_memory(aClient *, char *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_stats.so",
	"/STATS command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
        add_Command("STATS", m_stats, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
        del_Command("STATS", m_stats);
}
#endif

/*
 * m_stats
 *	parv[0] = sender prefix
 *	parv[1] = stat
 *	parv[2] = server name
 */
int m_stats(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	static char Lformat[] = ":%s %d %s %s %u %u %u %u %u :%u %u %s";
	static char Sformat[] = ":%s %d %s Name SendQ SendM SendBytes RcveM RcveBytes :OpenSince Idle TS";

	aClient *acptr;
	char stat = parc > 1 ? parv[1][0] : '\0';
	char *name;
	int i, doall = 0, wilds = 0;
	time_t sincetime = 0L;   
	static time_t last_used = 0L;
	char options[10];

	if (hunt_server(cptr, sptr, ":%s STATS %s :%s", 2, parc, parv) != HUNTED_ISME) {
		return 0;
	}
	if (!IsOper(sptr) && !IsULine(sptr)) {
		if (last_used + ConfigSet.pacetime > NOW) {
			sendto_one(sptr, rpl_str(RPL_LOAD2HI), me.name, parv[0]);
			return 0;
		}
		else {
			last_used = NOW;
		}
		if (IsPerson(sptr)) {
			if (stat == '\0') {
				sendto_ops_lev(SPY_LEV, "STATS 'NULL' requested by %s (%s@%s) [%s]",
					sptr->name, sptr->user->username,
					IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host,
					sptr->user->server);
				return 0;
			}
			else {   
				sendto_ops_lev(SPY_LEV, "STATS '%c' requested by %s (%s@%s) [%s]",
					stat, sptr->name, sptr->user->username,
					IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host,
					sptr->user->server);
			}
		}
	}

	if (parc > 2) {
		name = parv[2];
		if (!mycmp(name, me.name)) {
			doall = 2;
		}
		else if (!match(name, me.name)) {
			doall = 1;
		}
		if (strchr(name, '*') || strchr(name, '?')) {
			wilds = 1;
		}
	}
	else {
		name = me.name;
	}

	switch (stat) {
		case 'L':
		case 'l':
		{
			sendto_one(sptr, Sformat, me.name, RPL_STATSLINKINFO, parv[0]);
			if (parc > 2 && !(doall || wilds)) {
				acptr = find_person(name, NULL);
				if (!acptr) {
					break;
				}
				sincetime = acptr->since > timeofday ? 0 : timeofday - acptr->since;
				sendto_one(sptr, Lformat, me.name, RPL_STATSLINKINFO, parv[0],
					get_client_name(acptr, TRUE), (int)DBufLength(&acptr->sendQ),
					(int)acptr->sendM, (int)acptr->sendK, (int)acptr->receiveM,
					(int)acptr->receiveK, timeofday - acptr->firsttime,
					sincetime,
					IsServer(acptr) ? DoesTS(acptr) ? "TS" : "Non-TS" : "-");
			}
			else {
				for (i = 0; i <= highest_fd; i++) {
					if (!(acptr = local[i]) || !IsServer(acptr)) {
						continue;
					}
					if (ConfigSet.hideulines && IsULine(acptr) && !IsOper(sptr)) {
						continue;
					}
					sincetime = acptr->since > timeofday ? 0 : timeofday - acptr->since;
					sendto_one(sptr, Lformat, me.name, RPL_STATSLINKINFO, parv[0],
						get_client_name(acptr, HIDEME), (int)DBufLength(&acptr->sendQ),
						(int)acptr->sendM, (int)acptr->sendK, (int)acptr->receiveM,
						(int)acptr->receiveK, timeofday - acptr->firsttime, sincetime,
						IsServer(acptr) ? DoesTS(acptr) ? "TS" : "Non-TS" : "-");
				}
			}
			break;
		}
		case 'C':
		case 'c':
		case 'H':
		case 'h':
		{
			ConfigItem_link *link = NULL;
			if (ConfigSet.hideulines && !IsOper(sptr)) {
				sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
				break;
			}
			for (link = config_link; link; link = (ConfigItem_link *)link->next) {
				ircsprintf(options, "%s%s%s",
					(link->options & LINK_AUTOCONNECT) ? "A" : "",
					(link->options & LINK_ZIP) ? "Z" : "",
					(link->options & LINK_DKEY) ? "E" : "");
				sendto_one(sptr, err_str(RPL_STATSCLINE), me.name, parv[0], "C",
					link->fromip, link->servername, link->port, link->class->name,
					options);
			}
			break;
		}
		case 'K':
		case 'k':
		case 'Q':
		case 'q':
		case 'B':
		case 'b':
		case 'G':
		case 'g':
		{
			ConfigItem_ban *ban = NULL;
			char type = '\0';
			for (ban = config_ban; ban; ban = (ConfigItem_ban *)ban->next) {
				if (stat == 'K' || stat == 'k') {
					switch (ban->type) {
						case BAN_USERHOST:
							type = ban->flag.temporary ? 'k' : 'K';
							break;
						case BAN_IP:
							type = ban->flag.temporary ? 'z' : 'Z';
							break;
						case BAN_AUTOKILL:
							type = 'A';
							break;
						default:
							continue;
					}
				}
				else if (stat == 'Q' || stat == 'q') {
					switch (ban->type) {
						case BAN_NICKNAME:
							type = 'Q';
							break;
						case BAN_SERVER:
							type = 'S';
							break;
						case BAN_CHANNEL:
							type = 'c';
							break;
						default:
							continue;
					}
				}
				else if (stat == 'B' || stat == 'b') {
					switch (ban->type) {
						case BAN_FILE:
							type = 'B';
							break;
						default:
							continue;
					}
				}
				else if (stat == 'G' || stat == 'g') {
					switch (ban->type) {
						case BAN_GCOS:
							type = ban->flag.temporary ? 'g' : 'G';
							break;
						default:
							continue;
					}
				}
				sendto_one(sptr, rpl_str(RPL_STATSKLINE), me.name, parv[0], type,
					ban->mask, ban->reason);
			}
			break;
		}
		case 'E':
		{
			if (!IsOper(sptr)) {
				sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
				break;
			}
			show_events(sptr);
			break;
		}
/*		case 'e':
		{
			ConfigItem_except *except = NULL;
			if (!IsOper(sptr)) {
				sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
				break;
			}
			for (except = config_except; except; except = (ConfigItem_except *)except->next) {
				sendto_one(sptr, rpl_str(RPL_STATSKLINE),me.name, parv[0],
					except->flag.temporary ? 'e' : 'E', except->mask, "*");
			}
			break;
		}*/
		case 'U':
		{
			ConfigItem_uline *uline = NULL;
			if (ConfigSet.hideulines && !IsOper(sptr)) {
				sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
				break;
			}
			for (uline = config_uline; uline; uline = (ConfigItem_uline *)uline->next) {
				sendto_one(sptr, rpl_str(RPL_STATSULINE), me.name, parv[0], "U",
					uline->servername);
			}
			break;
		}
		case 'u':
		{
			time_t now = timeofday - me.since;
			sendto_one(sptr, rpl_str(RPL_STATSUPTIME), me.name, parv[0], now / 86400,
				(now / 3600) % 24, (now / 60) % 60, now % 60);
			sendto_one(sptr, rpl_str(RPL_STATSCONN), me.name, parv[0], max_connection_count,
				max_client_count);
			break;
		}
		case 'I':
		case 'i':
		{
			ConfigItem_allow *allow = NULL;
			for (allow = config_allow; allow; allow = (ConfigItem_allow *)allow->next) {
				sendto_one(sptr, rpl_str(RPL_STATSILINE), me.name, parv[0], allow->ip,
					allow->hostname, allow->maxperip, allow->class->name);
			}
			break;
		}
		case 'M':
		case 'm':
		{
			int i = 0;
			aCommand *command = NULL;
			for (i = 0; i < 256; i++) {
				for (command = CommandHash[i]; command; command = command->next) {
					sendto_one(sptr, rpl_str(RPL_STATSCOMMANDS), me.name, parv[0],
						command->cmd, command->count, command->bytes);
				}
			}
			break;
		}
		case 'O':
		case 'o':
		{
			ConfigItem_oper *oper = NULL;
			if (!IsOper(sptr)) {
				sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
				break;
			}
			for (oper = config_oper; oper; oper = (ConfigItem_oper *)oper->next) {
				sendto_one(sptr, rpl_str(RPL_STATSOLINE), me.name, parv[0], "O",
					oper->from, oper->name, oper->class->name);
			}
			break;
		}
		case 'p':
		case 'P':
		{
			ConfigItem_listen *listen = NULL;
			if (!IsOper(sptr)) {
				sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
				break;
			}
			for (i = 0; i <= highest_fd; i++) {
				if (!(acptr = local[i]) || !IsListening(acptr)) {
					continue;
				}
				if (!acptr->class) {
					sendto_one(sptr, ":%s NOTICE %s :*** ERROR *** Listener without class "
						"structure", me.name, parv[0]);
					continue;
				}

				listen = (ConfigItem_listen *)acptr->class;
				sendto_one(sptr, ":%s NOTICE %s :*** Listener bound on %s:%d, %d client%s, %s.",
					me.name, parv[0], listen->ip, listen->port, listen->clients,
					listen->clients == 1 ? "" : "s",
					listen->flag.temporary ? "temporary" : "permanent");
			}
			break;
		}
		case 'T':
		case 't':
		{
			if (!IsOper(sptr)) {
				sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
				break;
			}
			tstats(sptr, parv[0]);
			break;
		}
#ifdef DEBUGMODE
		case 'R':
		case 'r':
		{
			send_usage(sptr, parv[0]);
			break;
		}
#endif
		case 'S':
		case 's':
		{
			if (!IsOper(sptr)) {
				sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
				break;
			}
			list_scache(sptr, sptr, parc, parv);
			break;
		}
		case 'N':
		case 'n':
		{
			sendto_one(sptr, rpl_str(RPL_STATSCOUNT), me.name, parv[0],
				"User Connects Today: ", Count.today);
			sendto_one(sptr, rpl_str(RPL_STATSCOUNT), me.name, parv[0],
				"User Connects past week: ", Count.weekly);
			sendto_one(sptr, rpl_str(RPL_STATSCOUNT), me.name, parv[0],
				"User Connects past month: ", Count.monthly);
			sendto_one(sptr, rpl_str(RPL_STATSCOUNT), me.name, parv[0],
				"User Conects past year: ", Count.yearly);
			break;
		}
		case 'V':
		{
			ConfigItem_vhost *vhost = NULL;
			if (!IsOper(sptr)) {
				sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
				break;
			}
			for (vhost = config_vhost; vhost; vhost = (ConfigItem_vhost *)vhost->next) {
				sendto_one(sptr, ":%s %d %s :V %s %s (%s)", me.name, RPL_TEXT, parv[0],
					vhost->vhost, vhost->login, vhost->from);
			}
			break;
		}
		case 'v':
		{
			show_oper_serv_fdlists(sptr);
			break;
		}
		case 'Y':
		case 'y':
		{
			ConfigItem_class *class = NULL;
			for (class = config_class; class; class = (ConfigItem_class *)class->next) {
				sendto_one(sptr, rpl_str(RPL_STATSYLINE), me.name, parv[0], "Y",
					class->name, class->pingfreq, class->connfreq,
					class->maxclients, class->sendq, class->clients);
			}
			break;
		}
		case 'Z':
		case 'z':
		{
			if (!IsOper(sptr)) {
				sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
				break;
			}
			count_memory(sptr, parv[0]);
			break;
		}
		case '?':
		{
			serv_info(sptr, parv[0]);
			break;
		}
		default:
		{
			if (stat == '\0') {
				sendto_one(sptr, ":%s NOTICE %s :*** Please specify a staticstic flag.",
					me.name, parv[0]);
			}
			else {
				sendto_one(sptr, ":%s NOTICE %s :*** '%c' is an unknown statistic.", me.name,
					parv[0], stat);
			}
			break;
		}
	}
	sendto_one(sptr, rpl_str(RPL_ENDOFSTATS), me.name, parv[0], stat);
	return 0;
}
