/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_svskill.c: SVSKILL command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_svskill.c,v 1.7 2002/04/08 17:05:56 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_svskill(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_svskill.so",
	"/SVSKILL command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("SVSKILL", m_svskill, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("SVSKILL", m_svskill);
}
#endif

/*
 * m_svskill
 *	parv[0] = sender prefix
 *	parv[1] = target
 *	parv[2] = target stamp
 *	parv[3] = message
 */
int m_svskill(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	char *comment, reason[TOPICLEN + 1];
	ts_val ts = 0;

	if (!IsULine(sptr)) {
		return 0;
	}
	if (parc > 3) {
		comment = parv[3] ? parv[3] : parv[0];
		ts = atol(parv[2]);
	}
	else {
		comment = (parc > 2 && parv[2]) ? parv[2] : parv[0];
	}

	acptr = find_client(parv[1], NULL);
	if (acptr && (!ts || ts == acptr->tsinfo)) {
		if (MyClient(acptr)) {
			strcpy(reason, "SVSKilled: ");
			strncpy(reason + 11, comment, TOPICLEN - 11);
			reason[TOPICLEN] = '\0';
			ircdlog(LOG_KILL, "SVSKILL for %s by %s", get_client_name(acptr, FALSE),
				get_client_name(cptr, FALSE));
			exit_client(acptr, acptr, sptr, reason);
			return (acptr == cptr) ?  FLUSH_BUFFER : 0;
		}
		if (acptr->from == cptr) {
			sendto_ops_lev(DEBUG_LEV, "Received wrong-direction SVSKILL for %s (behined "
				"%s) from %s", acptr->name, cptr->name, get_client_name(sptr, HIDEME));
		}
		else if (ts == 0) {
			sendto_one(acptr->from, ":%s SVSKILL %s :%s", parv[0], parv[1], comment);
		}
		else {
			sendto_serv_butone(acptr->from, ":%s SVSKILL %s %ld :%s", parv[0], parv[1], ts, comment);
		}
	}
	return 0;
}
