/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_svsnick.c: SVSNICK command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_svsnick.c,v 1.6 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_svsnick(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_svsnick.so",
	"/SVSNICK command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("SVSNICK", m_svsnick, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("SVSNICK", m_svsnick);
}
#endif

/*
 * m_svsnick
 *	parv[0] = sender prefix
 *	parv[1] = old nickname
 *	parv[2] = new nickname
 *	parv[3] = TS
 *
 * Implemented from DreamForge by Rastlin.
 */
int m_svsnick(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;

	if (!IsULine(sptr) || parc < 4 || strlen(parv[2]) > NICKLEN) {
		return 0;
	}

	acptr = find_person(parv[2], NULL);
	if (acptr) {
		sendto_serv_butone(cptr, ":%s KILL %s :%s (SVSNICK collision)", sptr->name,
			parv[1], sptr->name);
		sendto_one(cptr, ":%s KILL %s :%s (SVSNICK collision)", sptr->name, parv[2], sptr->name);
		return 0;
	}

	if (!hunt_server(cptr, sptr, ":%s SVSNICK %s %s :%s", 1, parc, parv) != HUNTED_ISME) {
		acptr = find_person(parv[1], NULL);
		if (acptr) {
			acptr->umode &= ~UMODE_r;
			acptr->last_nick_change = atoi(parv[3]);
			sendto_common_channels(acptr, ":%s NICK :%s", parv[1], parv[2]);

			if (IsPerson(acptr)) {
				add_history(acptr, 1);
			}
			sendto_serv_butone(NULL, ":%s NICK %s :%d", parv[1], parv[2], atoi(parv[3]));

			if (acptr->name[0]) {
				del_from_client_hash_table(acptr->name, acptr);
			}
			strcpy(acptr->name, parv[2]);
			add_to_client_hash_table(parv[2], acptr);
		}
	}
	return 0;
}
