/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_svsnoop.c: SVSNOOP command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_svsnoop.c,v 1.7 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_svsnoop(aClient *, aClient *, int, char **);

extern ConfigItem *del_ConfigItem(ConfigItem *, ConfigItem **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_svsnoop.so",
	"/SVSNOOP command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("SVSNOOP", m_svsnoop, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("SVSNOOP", m_svsnoop);
}
#endif

/*
 * m_svsnoop
 *	parv[0] = sender prefix
 *	parv[1] = server
 *	parv[2] = +/-
 */
int m_svsnoop(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	ConfigItem_oper *oper = NULL;
	ConfigItem c;

	if (!IsULine(sptr) && parc < 2) {
		return 0;
	}

	if (hunt_server(cptr, sptr, ":%s SVSNOOP %s :%s", 1, parc, parv) != HUNTED_ISME) {
		return 0;
	}

	if (parv[2][0] == '+') {
		for (oper = config_oper; oper; oper = (ConfigItem_oper *)oper->next) {
			c.next = del_ConfigItem((ConfigItem *)oper, (ConfigItem **)&config_oper);
			MyFree(oper->name);
			MyFree(oper->from);
			MyFree(oper->password);
			MyFree(oper);
			oper = (ConfigItem_oper *)&c;
		}
	}
	else {
		rehash(&me, &me, 2);
	}
	return 0;
}
