/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_unkline.c: UNKLINE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_unkline.c,v 1.9 2002/04/08 16:00:48 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_unkline(aClient *, aClient *, int, char **);

extern ConfigItem *del_ConfigItem(ConfigItem *, ConfigItem **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_unkline.so",
	"/UNKLINE command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("UNKLINE", m_unkline, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("UNKLINE", m_unkline);
}
#endif

/*
 * m_unkline
 *	parv[0] = sender prefix
 *	parv[1] = mask
 */
int m_unkline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char mask[USERLEN + HOSTLEN + 2];
	char *user, *host, *p;
	ConfigItem_ban *ban;
	int i = 0;

	if (!MyClient(sptr) || !IsOper(sptr) || !OPCanUnKline(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}
	if (parc < 2 || BadPtr(parv[1])) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "UNKLINE");
		return 0;
	}

	host = strchr(parv[1], '@');
	if (host || *parv[1] == '*') {
		if (host) {
			user = parv[1];
			*(host++) = '\0';
		}
		else {
			user = "*";
			host = parv[1];
		}

		if (!strcmp(user, "*")) {
			for (p = host; *p; p++) {
				if (*p != '*' && *p != '.') {
					i++;
				}
			}
			if (i < 1) {
				sendto_one(sptr, ":%s NOTICE %s :*** You cannot un-K:Line *@*",
					me.name, parv[0]);
				return 0;
			}
		}
	}
	else {
		user = "*";
		host = parv[1];
	}

	ircsprintf(mask, "%s@%s", user, host);
	ban = find_ban(BAN_USERHOST, mask);
	if (!ban) {
		sendto_one(sptr, ":%s NOTICE %s :*** %s is not currently on the K:Line list",
			me.name, parv[0], mask);
		return 0;
	}
	if (!ban->flag.temporary) {
		sendto_one(sptr, ":%s NOTICE %s :*** Cannot remove a permanent K:Line",
			me.name, parv[0]);
		return 0;
	}

	del_ConfigItem((ConfigItem *)ban, (ConfigItem **)&config_ban);
	sendto_realops("%s (%s@%s) has removed temporary K:Line on %s", parv[0], sptr->user->username,
		IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host, ban->mask);
	MyFree(ban->mask);
	MyFree(ban->reason);
	MyFree(ban);
	return 0;
}
