/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_user.c: USER command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_user.c,v 1.14 2002/04/08 16:00:49 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_user(aClient *, aClient *, int, char **);

extern int do_user(char *, aClient *, aClient *, char *, char *, char *, unsigned long, unsigned int, char *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_user.so",
	"/USER command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("USER", m_user, 0, 15, 1, 1, 0, 0L);
}
void _module_unload(void) {
	del_Command("USER", m_user);
}
#endif

/*
 * m_user
 *	parv[0] = sender prefix
 *	parv[1] = username
 *	parv[2] = client hostname (used only from other servers)
 *	parv[3] = server hostname (used only from other servers)
 *	parv[4] = users real name info
 */
int m_user(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *username, *host, *server, *realname;
	ConfigItem_ban *ban;

	if (sptr->listener && sptr->listener->class) {
		ConfigItem_listen *listen = (ConfigItem_listen *)sptr->listener->class;
		if (listen->options & LISTEN_SERVERONLY) {
			sendto_ops_lev(REJ_LEV, "Blocking unknown connection on server-only port %d",
				cptr->listener->port);
			return exit_client(cptr, sptr, sptr, "This port is for servers only");
		}
	}

	username = strchr(parv[1], '@');
	if (parc > 2 && username) {
		*username = '\0';
	}
	if (parc < 5 || *parv[1] == '\0' || *parv[2] == '\0' || *parv[3] == '\0' || *parv[4] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "USER");
		if (IsServer(cptr)) {
			sendto_realops_lev(DEBUG_LEV, "Bad USER param count for %s from %s", parv[0],
				get_client_name(cptr, FALSE));
		}
		else {
			return 0;
		}
	}

	username = (parc < 2 || BadPtr(parv[1])) ? "<bad-boy>" : parv[1];
	host = (parc < 3 || BadPtr(parv[2])) ? "<nohost>" : parv[2];
	server = (parc < 4 || BadPtr(parv[3])) ? "<noserver>" : parv[3];
	realname = (parc < 5 || BadPtr(parv[4])) ? "<bad-realname>" : parv[4];

	ban = find_ban(BAN_GCOS|BAN_GCOS_S, realname);
	if (ban) {
		sendto_one(sptr, ":%s NOTICE %s :*** Your GCOS (real name) is banned: %s", me.name,
			parv[0], ban->reason);
		return exit_client(cptr, sptr, sptr, "G:Lined");
	}
	return do_user(parv[0], cptr, sptr, username, host, server, 0, 0, realname);
}
