/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_userhost.c: USERHOST command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_userhost.c,v 1.7 2002/04/08 16:00:49 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_userhost(aClient *, aClient *, int, char **);
static char buf[BUFSIZE], buf2[BUFSIZE];

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_userhost.so",
	"/USERHOST command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("USERHOST", m_userhost, 0, 15, 1, 1, 0, 0L);
}
void _module_unload(void) {
	del_Command("USERHOST", m_userhost);
}
#endif

/*
 * m_userhost
 *	parv[0] = sender prefix
 *	parv[1] = user
 */
int m_userhost(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *p = NULL, *s;
	aClient *acptr;
	int i, len;

	if (parc > 2) {
		m_userhost(cptr, sptr, parc - 1, parv + 1);
	}
	if (parc < 2) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "USERHOST");
		return 0;
	}

	ircsprintf(buf, rpl_str(RPL_USERHOST), me.name, parv[0]);
	len = strlen(buf);
	*buf2 = '\0';

	for (i = 5, s = strtoken(&p, parv[1], ","); i && s; s = strtoken(&p, NULL, " "), i--) {
		acptr = find_person(s, NULL);
		if (!acptr) {
			continue;
		}

		if (*buf2) {
			strcat(buf, " ");
		}
		ircsprintf(buf2, "%s%s=%c%s@%s", acptr->name, IsOper(acptr) ? "*" : "",
			acptr->user->away ? '-' : '+', acptr->user->username,
			IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
		strncat(buf, buf2, sizeof(buf) - len);
		len += strlen(buf2);
	}
	sendto_one(sptr, "%s", buf);
	return 0;
}
