/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_vhost.c: VHOST command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_vhost.c,v 1.8 2002/04/08 16:00:49 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_vhost(aClient *, aClient *, int, char **);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_vhost.so",
	"/VHOST command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("VHOST", m_vhost, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("VHOST", m_vhost);
}
#endif

/*
 * m_vhost
 *	parv[0] = sender prefix
 *	parv[1] = command
 *	parv[2] = login
 *	parv[3] = password
 */
int m_vhost(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	ConfigItem_vhost *vhost = NULL;
	char from[USERLEN + HOSTLEN + 2];

	if (!MyClient(sptr)) {
		return 0;
	}
	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "VHOST");
		return 0;
	}

	if (!strcasecmp(parv[1], "ON")) {
		if (parc < 4) {
			sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "VHOST ON");
			return 0;
		}

		vhost = find_vhost(parv[2]);
		if (!vhost) {
			sendto_realops("%s (%s@%s) failed to login to a Vhost line (login incorrect)",
				parv[0], sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
			sendto_one(sptr, ":%s NOTICE %s :*** There are no Vhost entries matching your login",
				me.name, parv[0]);
			return 0;
		}

		ircsprintf(from, "%s@%s", sptr->user->username, sptr->user->host);
		if (match(vhost->from, from)) {
			sendto_realops("%s (%s@%s) failed to log into a Vhost line (userhost mismatch)",
				parv[0], sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
			sendto_one(sptr, ":%s NOTICE %s :*** There are no Vhost entries matching your userhost",
				me.name, parv[0]);
			return 0;
		}

		if (strcmp(vhost->password, parv[3])) {
			sendto_realops("%s (%s@%s) failed to log into a Vhost line (bad password)",
				parv[0], sptr->user->username,
				IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host);
			sendto_one(sptr, ":%s NOTICE %s :*** Password incorrect", me.name, parv[0]);
			return 0;
		}

		sendto_realops("%s (%s@%s) has successfully logged into a Vhost line (%s)",
			parv[0], sptr->user->username,
			IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host, vhost->vhost);

		if (sptr->user->maskedhost) {
			MyFree(sptr->user->maskedhost);
		}
		sptr->user->maskedhost = MyMalloc(strlen(vhost->vhost) + 1);
		strcpy(sptr->user->maskedhost, vhost->vhost);
		sptr->umode |= UMODE_z;

		sendto_serv_butone(cptr, ":%s SENDVHOST :%s", parv[0], vhost->vhost);
		sendto_one(sptr, ":%s NOTICE %s :*** Your hostname is now %s", me.name,
			parv[0], sptr->user->maskedhost);
	}
	else if (!strcasecmp(parv[1], "OFF")) {
		do_maskedhost(sptr);
		sptr->umode |= UMODE_z;
		sendto_serv_butone(cptr, ":%s SENDVHOST :*", parv[0]);
		sendto_one(sptr, ":%s NOTICE %s :*** Your hostname is now %s", me.name, parv[0],
			sptr->user->maskedhost);
	}
	else {
		sendto_one(sptr, ":%s NOTICE %s :*** Syntax: /VHOST <ON/OFF> [login] [password]",
			me.name, parv[0]);
	}
	return 0;
}
