/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_who.c: WHO command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_who.c,v 1.7 2002/04/08 16:00:49 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_who(aClient *, aClient *, int, char **);
static int (*nchkfn)(char *, char *);
static int (*uchkfn)(char *, char *);
static int (*hchkfn)(char *, char *);
static int (*gchkfn)(char *, char *);
static int build_searchopts(aClient *, int, char **);
static int chk_who(aClient *, int);
static inline char *first_visible_channel(aClient *, aClient *);
static SOpts wsopts;

extern int user_modes[];

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_who.so",
	"/WHO command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("WHO", m_who, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("WHO", m_who);
}
#endif

/*
 * m_who
 *	parv[0] = sender prefix
 *	parv[1] = options
 */
int m_who(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *ac;
	chanMember *cm;
	Link *lp;
	int shown = 0, i = 0, showall = IsOper(sptr);
	char status[4], modechar = '\0';

	if (!MyClient(sptr) || !build_searchopts(sptr, parc - 1, parv + 1)) {
		return 0;
	}

	if (wsopts.gcos && !strchr(wsopts.gcos, '?') && !strchr(wsopts.gcos, '*')) {
		gchkfn = mycmp;
	}
	else {
		gchkfn = match;
	}
	if (wsopts.nick && !strchr(wsopts.nick, '?') && !strchr(wsopts.nick, '*')) {
		nchkfn = mycmp;
	}
	else {
		nchkfn = match;
	}
	if (wsopts.user && !strchr(wsopts.user, '?') && !strchr(wsopts.user, '*')) {
		uchkfn = mycmp;
	}
	else {
		uchkfn = match;
	}
	if (wsopts.host && !strchr(wsopts.host, '?') && !strchr(wsopts.host, '*')) {
		hchkfn = mycmp;
	}
	else {
		hchkfn = match;
	}

	if (wsopts.channel) {
		if (IsMember(sptr, wsopts.channel)) {
			showall = 1;
		}
		else if (SecretChannel(wsopts.channel) && IsSAdmin(sptr)) {
			showall = 1;
		}
		else if (!SecretChannel(wsopts.channel) && IsOper(sptr)) {
			showall = 1;
		}
		else {
			showall = 0;
		}

		if (showall || !SecretChannel(wsopts.channel)) {
			for (cm = wsopts.channel->members; cm; cm = cm->next) {
				ac = cm->cptr;
				i = 0;

				if (!chk_who(ac, showall)) {
					continue;
				}
				status[i++] = !ac->user->away ? 'H' : 'G';
				status[i] = IsOper(ac) ? '*' : (IsInvisible(ac) && IsOper(sptr)) ? '%' : '\0';
				if (cm->flags & CHFL_CHANADMIN) {
					modechar = '*';
				}
				else if (cm->flags & CHFL_CHANOP) {
					modechar = '@';
				}
				else if (cm->flags & CHFL_HALFOP) {
					modechar = '%';
				}
				else if (cm->flags & CHFL_VOICE) {
					modechar = '+';
				}
				else {
					modechar = '\0';
				}
				status[(status[i] ? ++i : i)] = modechar;
				status[++i] = '\0';

				sendto_one(sptr, rpl_str(RPL_WHOREPLY), me.name, sptr->name,
					wsopts.channel->chname, ac->user->username,
					IsMasked(ac) ? ac->user->maskedhost : ac->user->host,
					ac->user->server, ac->name, status, WHO_HOPCOUNT(sptr, ac),
					ac->info);
			}
		}
		sendto_one(sptr, rpl_str(RPL_ENDOFWHO), me.name, sptr->name, wsopts.channel->chname);
		return 0;
	}
	else if (nchkfn == mycmp) {
		ac = find_person(wsopts.nick, NULL);
		if (ac) {
			if (!chk_who(ac, 1)) {
				sendto_one(sptr, rpl_str(RPL_ENDOFWHO), me.name, sptr->name,
					wsopts.host ? wsopts.host : wsopts.nick);
				return 0;
			}
			status[0] = !ac->user->away ? 'H' : 'G';
			status[1] = IsOper(ac) ? '*' : (IsInvisible(ac) && IsOper(sptr)) ? '%' : '\0';
			status[2] = '\0';
			sendto_one(sptr, rpl_str(RPL_WHOREPLY), me.name, sptr->name,
				wsopts.show_chan ? first_visible_channel(ac, sptr) : "*",
				ac->user->username,
				IsMasked(ac) ? ac->user->maskedhost : ac->user->host,
				ac->user->server, ac->name, status,
				WHO_HOPCOUNT(sptr, ac), ac->info);
			sendto_one(sptr, rpl_str(RPL_ENDOFWHO), me.name, sptr->name,
				wsopts.host ? wsopts.host : wsopts.nick);
			return 0;
		}
		sendto_one(sptr, rpl_str(RPL_ENDOFWHO), me.name, sptr->name, wsopts.host ? wsopts.host : wsopts.nick);
		return 0;
	}

	if (lifesux && !IsOper(sptr)) {
		sendto_one(sptr, rpl_str(RPL_LOAD2HI), me.name, sptr->name);
		return 0;
	}

	if (wsopts.search_chan) {
		for (lp = sptr->user->channel; lp; lp = lp->next) {
			for (cm = lp->value.chptr->members; cm; cm = cm->next) {
				ac = cm->cptr;
				if (!chk_who(ac, 1)) {
					continue;
				}
				if (shown == MAXWHOREPLIES && !IsOper(sptr)) {
					sendto_one(sptr, err_str(ERR_WHOLIMEXCEED), me.name, sptr->name,
						MAXWHOREPLIES);
					break;
				}
				i = 0;
				status[i++] = !ac->user->away ? 'H' : 'G';
				status[i] = IsOper(ac) ? '*' : (IsInvisible(ac) && IsOper(sptr)) ? '%' : '\0';
				if (cm->flags & CHFL_CHANADMIN) {
					modechar = '*';
				}
				else if (cm->flags & CHFL_CHANOP) {
					modechar = '@';
				}
				else if (cm->flags & CHFL_HALFOP) {
					modechar = '%';
				}
				else if (cm->flags & CHFL_VOICE) {
					modechar = '+';
				}
				else {
					modechar = '\0';
				}
				status[(status[i] ? ++i : i)] = modechar;
				status[++i] = '\0';

				sendto_one(sptr, rpl_str(RPL_WHOREPLY), me.name, sptr->name,
					lp->value.chptr->chname, ac->user->username,
					IsMasked(ac) ? ac->user->maskedhost : ac->user->host,
					ac->user->server, ac->name, status,
					WHO_HOPCOUNT(sptr, ac), ac->info);
				shown++;
			}
		}
	}
	else {
		for (ac = client; ac; ac = ac->next) {
			if (!chk_who(ac, showall)) {
				continue;
			}
			if (shown == MAXWHOREPLIES && !IsOper(sptr)) {
				sendto_one(sptr, err_str(ERR_WHOLIMEXCEED), me.name, sptr->name,
					MAXWHOREPLIES);
				break;
			}
			status[0] = !ac->user->away ? 'H' : 'G';
			status[1] = IsOper(ac) ? '*' : (IsInvisible(ac) && IsOper(sptr)) ? '%' : '\0';
			status[2] = '\0';
			sendto_one(sptr, rpl_str(RPL_WHOREPLY), me.name, sptr->name,
				wsopts.show_chan ? first_visible_channel(ac, sptr) : "*",
				ac->user->username,
				IsMasked(ac) ? ac->user->maskedhost : ac->user->host,
				ac->user->server, ac->name, status, WHO_HOPCOUNT(sptr, ac),
				ac->info);
			shown++;
		}
	}
	sendto_one(sptr, rpl_str(RPL_ENDOFWHO), me.name, sptr->name,
		wsopts.host ? wsopts.host :
		wsopts.nick ? wsopts.nick :
		wsopts.user ? wsopts.user :
		wsopts.gcos ? wsopts.gcos :
		wsopts.server ? wsopts.server->name : "*");
	return 0;
}

static int build_searchopts(aClient *sptr, int parc, char *parv[])
{
	static char *who_help[] = {
		"/WHO [+|-][acghmnsuCM] [args]",
		"Flags are specified like channel modes, the flags cgmnsu all have arguments.",
		"Flags are set to a positive check by +, and a negative check by -. The flags",
		"are as follows:",
		" ",
		"a - user is away",
		"c <channel> - user is on <channel>, no wildcards accepted",
		"g <realname> - user has string <realname> in their GCOS (realname), wildcards",
		"               accepted, opers only",
		"h <host> - user has string <host> in their hostname/maskedhost, wildcards accepted",
		"m <usermodes> - user has <usermodes> set on them, only o/a/C/A/N for nonopers",
		"s <server> - user is on server <server>, wildcards not accepted",
		"u <user> - user has string <user> in their username, wildcards accepted",
		" ",
		"Behaviour flags:",
		" ",
		"C - show first visible channel user is in",
		"M - check for user in channels I am a member of",
		NULL
	};
	char *flags, change = 1, *s;
	int args = 1, i;

	memset((char *)&wsopts, '\0', sizeof(SOpts));

	if (parc < 1 || *parv[0] == '?') {
		char **ptr;
		for (ptr = who_help; *ptr; ptr++) {
			sendto_one(sptr, rpl_str(RPL_COMMANDSYNTAX), me.name, sptr->name, *ptr);
		}
		sendto_one(sptr, rpl_str(RPL_ENDOFWHO), me.name, sptr->name, "?");
		return 0;
	}
	else if (*parv[0] == '\0' && parv[0][1] == '\0') {
		if (parc > 1 && *parv[1] == 'o') {
			wsopts.check_umode = 1;
			wsopts.umode_plus = 1;
			wsopts.umodes = UMODE_o;
		}
		wsopts.host_plus = 1;
		wsopts.host = "*";
		return 1;
	}
	else if (*parv[0] != '+' && *parv[0] != '-') {
		if (*parv[0] == '#' || *parv[0] == '&') {
			wsopts.channel = find_channel(parv[0], NullChn);
			if (!wsopts.channel) {
				sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL), me.name, sptr->name,
					parv[0]);
				return 0;
			}
		}
		else {
			if (strchr(parv[0], '.')) {
				wsopts.host_plus = 1;
				wsopts.host = parv[0];
			}
			else {
				wsopts.nick_plus = 1;
				wsopts.nick = parv[0];
			}
		}
		return 1;
	}

	flags = parv[0];
	while (*flags) {
		switch (*flags) {
			case '+':
			case '-':
				change = *flags == '+' ? 1 : 0;
				break;
			case 'a':
				if (change) {
					wsopts.away_plus = 1;
				}
				else {
					wsopts.away_plus = 0;
				}
				wsopts.check_away = 1;
				break;
			case 'C':
				wsopts.show_chan = change;
				break;
			case 'M':
				wsopts.search_chan = change;
				break;
			case 'c':
				if (!parv[args] || !change) {
					sendto_one(sptr, err_str(ERR_WHOSYNTAX), me.name, sptr->name);
					return 0;
				}
				wsopts.channel = find_channel(parv[args], NullChn);
				if (!wsopts.channel) {
					sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL), me.name, sptr->name,
						parv[args]);
					return 0;
				}
				wsopts.chan_plus = change;
				args++;
				break;
			case 'g':
				if (!parv[args] || !IsOper(sptr)) {
					sendto_one(sptr, err_str(ERR_WHOSYNTAX), me.name, sptr->name);
					return 0;
				}
				wsopts.gcos = parv[args];
				wsopts.gcos_plus = change;
				args++;
				break;
			case 'h':
				if (!parv[args] || !IsOper(sptr)) {
					sendto_one(sptr, err_str(ERR_WHOSYNTAX), me.name, sptr->name);
					return 0;
				}
				wsopts.host = parv[args];
				wsopts.host_plus = change;
				args++;
				break;
			case 'm':
				if (!parv[args]) {
					sendto_one(sptr, err_str(ERR_WHOSYNTAX), me.name, sptr->name);
					return 0;
				}
				s = parv[args];
				while (*s) {
					for (i = 1; user_modes[i] != 0x0; i += 2) {
						if (*s == (char)user_modes[i]) {
							wsopts.umodes |= user_modes[i - 1];
							break;
						}
					}
					s++;
				}
				if (!IsOper(sptr)) {
					wsopts.umodes = (wsopts.umodes & (UMODE_o|UMODE_a|UMODE_C|UMODE_A|UMODE_N));
				}
				wsopts.umode_plus = change;
				if (wsopts.umodes) {
					wsopts.check_umode = 1;
				}
				args++;
				break;
			case 'n':
				if (!parv[args]) {
					sendto_one(sptr, err_str(ERR_WHOSYNTAX), me.name, sptr->name);
					return 0;
				}
				wsopts.nick = parv[args];
				wsopts.nick_plus = change;
				args++;
				break;
			case 's':
				if (!parv[args] || !change) {
					sendto_one(sptr, err_str(ERR_WHOSYNTAX), me.name, sptr->name);
					return 0;
				}
				wsopts.server = find_server(parv[args], NULL);
				if (!wsopts.server) {
					sendto_one(sptr, err_str(ERR_NOSUCHSERVER), me.name, sptr->name,
						parv[args]);
					return 0;
				}
				wsopts.serv_plus = change;
				args++;
				break;
			case 'u':
				if (!parv[args]) {
					sendto_one(sptr, err_str(ERR_WHOSYNTAX), me.name, sptr->name);
					return 0;
				}
				wsopts.user = parv[args];
				wsopts.user_plus = change;
				args++;
				break;
			default:
				args++;
				break;
		}
		flags++;
	}

	if (wsopts.search_chan && !(wsopts.check_away || wsopts.gcos || wsopts.host || wsopts.check_umode ||
	  wsopts.server || wsopts.user)) {
		if (!parv[args] || wsopts.channel || wsopts.nick || *parv[args] == '#' || *parv[args] == '&') {
			sendto_one(sptr, err_str(ERR_WHOSYNTAX), me.name, sptr->name);
			return 0;
		}

		if (strchr(parv[args], '.')) {
			wsopts.host_plus = 1;
			wsopts.host = parv[args];
		}
		else {
			sendto_one(sptr, err_str(ERR_WHOSYNTAX), me.name, sptr->name);
			return 0;
		}
	}
	else if (wsopts.show_chan && !(wsopts.check_away || wsopts.gcos || wsopts.host || wsopts.check_umode ||
	  wsopts.server || wsopts.user || wsopts.nick || wsopts.channel)) {
		if (!parv[args]) {
			sendto_one(sptr, err_str(ERR_WHOSYNTAX), me.name, sptr->name);
			return 0;
		}

		if (strchr(parv[args], '.')) {
			wsopts.host_plus = 1;
			wsopts.host = parv[args];
		}
		else {
			wsopts.nick_plus = 1;
			wsopts.nick = parv[args];
		}
	}
	return 1;
}

static int chk_who(aClient *ac, int showall)
{
	if (!IsClient(ac) || (!IsInvisible(ac) && !showall)) {
		return 0;
	}
	if (wsopts.check_umode) {
		if ((wsopts.umode_plus && !((ac->umode & wsopts.umodes) == wsopts.umodes)) ||
		  (!wsopts.umode_plus && ((ac->umode & wsopts.umodes) == wsopts.umodes))) {
			return 0;
		}
	}
	if (wsopts.check_away) {
		if ((wsopts.away_plus && !ac->user->away) || (!wsopts.away_plus && ac->user->away)) {
			return 0;
		}
	}
	if (wsopts.serv_plus) {
		if (wsopts.server != ac->uplink) {
			return 0;
		}
	}
	if (wsopts.user) {
		if ((wsopts.user_plus && uchkfn(wsopts.user, ac->user->username)) ||
		  (!wsopts.user_plus && !uchkfn(wsopts.user, ac->user->username))) {
			return 0;
		}
	}
	if (wsopts.nick) {
		if ((wsopts.nick_plus && nchkfn(wsopts.nick, ac->name)) ||
		  (!wsopts.nick_plus && !nchkfn(wsopts.nick, ac->name))) {
			return 0;
		}
	}
	if (wsopts.host) {
		if ((wsopts.host_plus && hchkfn(wsopts.host, ac->user->host)) ||
		  (!wsopts.host_plus && !hchkfn(wsopts.host, ac->user->host))) {
			return 0;
		}
	}
	if (wsopts.gcos) {
		if ((wsopts.gcos_plus && gchkfn(wsopts.gcos, ac->info)) ||
		  (!wsopts.gcos && !gchkfn(wsopts.gcos, ac->info))) {
			return 0;
		}
	}
	return 1;
}

static inline char *first_visible_channel(aClient *cptr, aClient *sptr)
{
	Link *lp;
	int secret = 0;
	aChannel *chptr = NULL;
	static char chnbuf[CHANNELLEN + 2];

	if (cptr->user->channel) {
		if (IsSAdmin(sptr)) {
			chptr = cptr->user->channel->value.chptr;
			if (!ShowChannel(sptr, chptr)) {
				secret = 1;
			}
		}
		else {
			for (lp = cptr->user->channel; lp; lp = lp->next) {
				if (ShowChannel(sptr, lp->value.chptr)) {
					break;
				}
			}
			if (lp) {
				chptr = lp->value.chptr;
			}
		}
		if (chptr) {
			if (!secret) {
				return chptr->chname;
			}
			ircsprintf(chnbuf, "%%%s", chptr->chname);
			return chnbuf;
		}
	}
	return "*";
}
