/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_whois.c: WHOIS command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_whois.c,v 1.14 2002/04/08 16:00:49 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

int m_whois(aClient *, aClient *, int, char **);
static char buf[BUFSIZE];

extern char *get_mode_str(aClient *);
extern int is_chan_admin(aClient *, aChannel *);
extern int is_chan_op(aClient *, aChannel *);
extern int is_half_op(aClient *, aChannel *);
extern int has_voice(aClient *, aChannel *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_whois.so",
	"/WHOIS command",
	30000,
	NULL,
	NULL
};
void _module_load(void) {
	add_Command("WHOIS", m_whois, 0, 15, 1, 0, 0, 0L);
}
void _module_unload(void) {
	del_Command("WHOIS", m_whois);
}
#endif

/*
 * m_whois
 *	parv[0] = sender prefix
 *	parv[1] = nickname list
 */
int m_whois(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	static anUser UnknownUser = {
		NULL,		/* channel */
		NULL,		/* invited */
		NULL,		/* away */
		0,		/* last */
		0,		/* joined */
		"<Unknown>",	/* user */
		"<Unknown>",	/* host */
		"<Unknown>",	/* server */
		0,		/* servicestamp */
		NULL		/* silenced */
	};
	Link *lp;
	anUser *user;
	aClient *acptr, *a2cptr;
	aChannel *chptr;
	char *nick, *tmp, *name = NULL, *p = NULL;
	int found, len, mlen;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "WHOIS");
		return 0;
	}
	if (parc > 2) {
		if (hunt_server(cptr, sptr, ":%s WHOIS %s :%s", 1, parc, parv) != HUNTED_ISME) {
			return 0;
		}
		parv[1] = parv[2];
	}

	for (tmp = parv[1]; (nick = strtoken(&p, tmp, ",")); tmp = NULL) {
		int member;
		found = 0;

		collapse(nick);
		acptr = hash_find_client(nick, (aClient *)NULL);
		if (!acptr || !IsPerson(acptr)) {
			sendto_one(sptr, err_str(ERR_NOSUCHNICK), me.name, parv[0], nick);
			return 0;
		}

		user = acptr->user ? acptr->user : &UnknownUser;
		name = (*acptr->name) ? acptr->name : "?";
		member = (user->channel) ? 1 : 0;
		a2cptr = find_server(user->server, NULL);

		sendto_one(sptr, rpl_str(RPL_WHOISUSER), me.name, parv[0], name, user->username,
			IsMasked(acptr) ? user->maskedhost : user->host, acptr->info);

		if (IsOper(sptr) || sptr == acptr) {
			sendto_one(sptr, rpl_str(RPL_WHOISHOSTANDMODES), me.name, parv[0], name,
				acptr->user->host, get_mode_str(acptr));
		}

		if (!IsServices(acptr)) {
			mlen = strlen(me.name) + strlen(parv[0]) + 6 + strlen(name);
			for (len = 0, *buf = '\0', lp = user->channel; lp; lp = lp->next) {
				chptr = lp->value.chptr;
				if (ShowChannel(sptr, chptr) || IsSAdmin(sptr)) {
					if (len + strlen(chptr->chname) > (size_t)BUFSIZE - 4 - mlen) {
						sendto_one(sptr, ":%s %d %s %s :%s", me.name, RPL_WHOISCHANNELS,
							name, buf);
						*buf = '\0';
						len = 0;
					}
					if (!ShowChannel(sptr, chptr)) {
						*(buf + len++) = '!';
					}
					if (is_chan_admin(acptr, chptr)) {
						*(buf + len++) = '*';
					}
					if (is_chan_op(acptr, chptr)) {
						*(buf + len++) = '@';
					}
					if (is_half_op(acptr, chptr)) {
						*(buf + len++) = '%';
					}
					if (has_voice(acptr, chptr)) {
						*(buf + len++) = '+';
					}
					if (buf) {
						*(buf + len) = '\0';
					}
					strcpy(buf + len, chptr->chname);
					len += strlen(chptr->chname);
					strcat(buf + len, " ");
					len++;
				}
			}
			if (buf[0] != '\0') {
				sendto_one(sptr, rpl_str(RPL_WHOISCHANNELS), me.name, parv[0], name, buf);
			}
		}

		sendto_one(sptr, rpl_str(RPL_WHOISSERVER), me.name, parv[0], name, user->server,
			a2cptr ? a2cptr->info : "*Not On This Net*");

		if (IsRegNick(acptr)) {
			sendto_one(sptr, rpl_str(RPL_WHOISREGNICK), me.name, parv[0], name);
		}
		if (user->away) {
			sendto_one(sptr, rpl_str(RPL_AWAY), me.name, parv[0], name, user->away);
		}

		buf[0] = '\0';
		if (IsOper(acptr)) {
			strcat(buf, "an IRC Operator");
		}
		if (IsNetAdmin(acptr)) {
			strcat(buf, " - Network Administrator");
		}
		else if (IsAdmin(acptr)) {
			strcat(buf, " - Server Administrator");
		}
		else if (IsCoAdmin(acptr)) {
			strcat(buf, " - Server Co-Administrator");
		}
		if (buf[0]) {
			sendto_one(sptr, rpl_str(RPL_WHOISOPERATOR), me.name, parv[0], name, buf);
		}

		if (IsServices(acptr)) {
			sendto_one(sptr, rpl_str(RPL_WHOISSERVICECLI), me.name, parv[0], name);
		}

		if (acptr->user && MyConnect(acptr)) {
			sendto_one(sptr, rpl_str(RPL_WHOISIDLE), me.name, parv[0], name,
				timeofday - user->last, acptr->firsttime);
		}
		continue;
	}
	sendto_one(sptr, rpl_str(RPL_ENDOFWHOIS), me.name, parv[0], name);
	return 0;
}
