/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/fdlist.c: file descriptor list management
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: fdlist.c,v 1.4 2002/04/08 16:00:50 rageircd Exp $
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "h.h"
#include "config.h"
#include "fdlist.h"
#include "memory.h"

void addto_fdlist(int fd, fdlist *listp)
{
	int index = ++listp->last_entry;
	if (index > MAXCONNECTIONS) {
		--listp->last_entry;
		ircdlog(LOG_ERROR, "fdlist too big, exiting");
		abort();
	}
	else {
		listp->entry[index] = fd;
	}
	return;
}

void delfrom_fdlist(int fd, fdlist *listp)
{
	int i;

	for (i = listp->last_entry; i; i--) {
		if (listp->entry[i] == fd) {
			break;
		}
	}
	if (!i) {
		return;
	}
	if (i == listp->last_entry) {
		listp->entry[i] = 0;
		listp->last_entry--;
	}
	else {
		listp->entry[i] = listp->entry[listp->last_entry];
		listp->entry[listp->last_entry] = 0;
		listp->last_entry--;
	}
	return;
}

void init_fdlist(fdlist *listp)
{
	listp->last_entry = 0;
	memset((char *)listp->entry, '\0', sizeof(listp->entry));
	return;
}
