/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/hook.c: internal function hooks
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: hook.c,v 1.3 2002/04/08 16:00:50 rageircd Exp $
 */

#include "struct.h"
#include "setup.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "h.h"
#include "msg.h"
#include "channel.h"
#include "send.h"
#include <sys/types.h>
#include <sys/stat.h>   
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "patchlevel.h"
#include "memory.h"

Hook *HookTable[MAXHOOKTYPES];
Hook *ghook;

void init_HookTable()
{
	memset(HookTable, '\0', sizeof(HookTable));
	return;
}

void addHook(int hooktype, int (*ifunc)(), void (*vfunc)())
{
	Hook *hptr = (Hook *)MyMalloc(sizeof(Hook));
	hptr->ifunc = ifunc;
	hptr->vfunc = vfunc;
	hptr->prev = NULL;
	hptr->next = HookTable[hooktype];
	if (hptr->next) {
		hptr->next->prev = hptr;
	}
	HookTable[hooktype] = hptr;
	return;
}

void delHook(int hooktype, int (*ifunc)(), void (*vfunc)())
{
	Hook *hptr;
	for (hptr = HookTable[hooktype]; hptr; hptr = hptr->next) {
		if ((ifunc && hptr->ifunc == ifunc) || (vfunc && hptr->vfunc == vfunc)) {
			if (hptr->prev) {
				hptr->prev->next = hptr->next;
			}
			else {
				HookTable[hooktype] = hptr->next;
			}
			if (hptr->next) {
				hptr->next->prev = hptr->prev;
			}
			MyFree(hptr);
			return;
		}
	}
	return;
}
