/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/match.c: string functions
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: match.c,v 1.6 2002/04/08 17:05:56 rageircd Exp $
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "memory.h"

#define MAX_CALLS 512
static int calls = 0;

static inline int _match(char *, char *);

char *collapse(char *ptr)
{
	char *s, *s1 = '\0', *t;

	if (BadPtr(ptr)) {
		return ptr;
	}
	for (s = ptr; *s; s++) {
		if (*s == '\\') {
			if (!*(s + 1)) {
				break;
			}
			else {
				s++;
			}
		}
		else if (*s == '*') {
			if (*(t = s1 = s + 1) == '*') {
				while (*t == '*') {
					t++;
				}
			}
			else if (*t == '?') {
				for (t++, s++; *t == '*' || *t == '?'; t++) {
					if (*t == '?') {
						*s1++ = *t;
					}
				}
			}
			while ((*s1++ = *t++));
		}
	}
	return ptr;
}

unsigned char tolowertab[] = {
	0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa,
	0xb, 0xc, 0xd, 0xe, 0xf, 0x10, 0x11, 0x12, 0x13, 0x14,
	0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d,
	0x1e, 0x1f,
	' ', '!', '"', '#', '$', '%', '&', 0x27, '(', ')',
	'*', '+', ',', '-', '.', '/',
	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
	':', ';', '<', '=', '>', '?',
	'@', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
	'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
	't', 'u', 'v', 'w', 'x', 'y', 'z', '{', '|', '}', '~',
	'_',
	'`', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
	'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
	't', 'u', 'v', 'w', 'x', 'y', 'z', '{', '|', '}', '~',
	0x7f,
	0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
	0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f,
	0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99,
	0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f,
	0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9,
	0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,
	0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9,
	0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf,
	0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9,
	0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf,
	0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9,
	0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf,
	0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9,
	0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
	0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9,
	0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
};

unsigned char touppertab[] = {
	0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa,
	0xb, 0xc, 0xd, 0xe, 0xf, 0x10, 0x11, 0x12, 0x13, 0x14,
	0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d,
	0x1e, 0x1f,
	' ', '!', '"', '#', '$', '%', '&', 0x27, '(', ')',
	'*', '+', ',', '-', '.', '/',
	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
	':', ';', '<', '=', '>', '?',
	'@', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I',
	'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S',
	'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '[', '\\', ']', '^',
	0x5f,
	'`', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I',
	'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S',
	'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '[', '\\', ']', '^',
	0x7f,
	0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
	0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f,
	0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99,
	0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f,
	0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9,
	0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,
	0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9,
	0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf,
	0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9,
	0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf,
	0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9,
	0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf,
	0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9,
	0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
	0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9,
	0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
};

unsigned char char_atribs[] =
{
	CNTRL, CNTRL, CNTRL, CNTRL, CNTRL, CNTRL, CNTRL, CNTRL,
	CNTRL, CNTRL | SPACE, CNTRL | SPACE, CNTRL | SPACE, CNTRL | SPACE,
	CNTRL | SPACE, CNTRL, CNTRL,
	CNTRL, CNTRL, CNTRL, CNTRL, CNTRL, CNTRL, CNTRL, CNTRL,
	CNTRL, CNTRL, CNTRL, CNTRL, CNTRL, CNTRL, CNTRL, CNTRL,
	PRINT | SPACE,
	PRINT, PRINT, PRINT, PRINT, PRINT, PRINT, PRINT, PRINT,
	PRINT, PRINT, PRINT, PRINT, PRINT, PRINT, PRINT,
	PRINT | DIGIT, PRINT | DIGIT, PRINT | DIGIT, PRINT | DIGIT,
	PRINT | DIGIT, PRINT | DIGIT, PRINT | DIGIT, PRINT | DIGIT,
	PRINT | DIGIT, PRINT | DIGIT, PRINT, PRINT,
	PRINT, PRINT, PRINT, PRINT,
	PRINT,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT, PRINT,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	PRINT | ALPHA, PRINT | ALPHA, PRINT | ALPHA,
	0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

int mycmp(char *s1, char *s2)
{
	unsigned char *str1 = (unsigned char *)s1;
	unsigned char *str2 = (unsigned char *)s2;

	while (touppertab[*str1] == touppertab[*str2]) {
		if (*str1 == '\0') {
			return 0;
		}
		str1++;
		str2++;
	}
	return -1;
}

int myncmp(char *s1, char *s2, int n)
{
	unsigned char *str1 = (unsigned char *)s1;
	unsigned char *str2 = (unsigned char *)s2;

	while (touppertab[*str1] == touppertab[*str2]) {
		str1++;
		str2++;
		n--;
		if (n == 0 || (*str1 == '\0' && *str2 == '\0')) {
			return 0;
		}
	}
	return -1;
}

int match(char *n, char *m)
{
	calls = 0;
	return _match(n, m);
}

static inline int _match(char *w, char *s)
{
	if (calls++ > MAX_CALLS) {
		return 1;
	}

	if (*w == '*' && *(w + 1) == '!') {
		w += 2;
		while (*s != '!' && *s) {
			s++;
		}
		if (!*s) {
			return 1;
		}
		s++;
	}
	if (*w == '*' && *(w + 1) == '@') {
		w += 2;
		while (*s != '@' && *s) {
			s++;
		}
		if (!*s) {
			return 1;
		}
		s++;
	}
	while (1) {
		if (!*w) {
			return (!*s) ? 0 : 1;
		}
		if (*w == '*') {
			w++;
			while (*w == '*') {
				w++;
			}
			while (*w == '?' && *s) {
				w++;
				s++;
			}
			if (!*w) {
				return 0;
			}
			if (*w == '*') {
				continue;
			}
			for (; *s; s++) {
				if (touppertab[(u_char)*s] == touppertab[(u_char)*w] && !_match(w + 1, s + 1)) {
					return 0;
				}
			}
		}
		if (!*s) {
			return 1;
		}
		if (*w != '?' && touppertab[(u_char)*s] != touppertab[(u_char)*w]) {
			return 1;
		}
		s++;
		w++;
	}
	return 1;
}
