/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/scache.c: server cache
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: scache.c,v 1.4 2002/04/08 16:00:52 rageircd Exp $
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "h.h"
#include "memory.h"

static int hash(char *);
static SCACHE *scache_hash[SCACHE_HASH_SIZE];

void clear_scache_hash_table(void)
{
	memset((char *)scache_hash, '\0', sizeof(scache_hash));
	return;
}

static int hash(char *string)
{
	int hash_value = 0;
	while (*string) {
		hash_value += (*string++ & 0xDF);
	}
	return hash_value % SCACHE_HASH_SIZE;
}

char *find_or_add(char *name)
{
	int hash_index = hash(name);
	SCACHE *newptr, *ptr = scache_hash[hash_index];

	for (ptr = scache_hash[hash_index]; ptr; ptr = ptr->next) {
		if (!mycmp(ptr->name, name)) {
			return ptr->name;
		}
	}

	ptr = scache_hash[hash_index];
	if (ptr) {
		newptr = scache_hash[hash_index] = (SCACHE *)MyMalloc(sizeof(SCACHE));
		strncpyzt(newptr->name, name, HOSTLEN);
		newptr->next = ptr;
		return newptr->name;
	}
	else {
		ptr = scache_hash[hash_index] = (SCACHE *)MyMalloc(sizeof(SCACHE));
		strncpyzt(ptr->name, name, HOSTLEN);
		ptr->next = (SCACHE *)NULL;
		return ptr->name;
	}

	return NULL;
}

void count_scache(int *number_servers_cached, u_long *mem_servers_cached)
{
	SCACHE *scache_ptr;
	int i;

	*number_servers_cached = 0;
	*mem_servers_cached = 0;

	for (i = 0; i < SCACHE_HASH_SIZE; i++) {
		for (scache_ptr = scache_hash[i]; scache_ptr; scache_ptr = scache_ptr->next) {
			*number_servers_cached = *number_servers_cached + 1;
			*mem_servers_cached = *mem_servers_cached + (strlen(scache_ptr->name) + sizeof(SCACHE *));
		}
	}
	return;
}

void list_scache(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int hash_index;
	SCACHE *ptr;

	for (hash_index = 0; hash_index < SCACHE_HASH_SIZE; hash_index++) {
		for (ptr = scache_hash[hash_index]; ptr; ptr = ptr->next) {
			if (ptr->name) {
				sendto_one(sptr, ":%s NOTICE %s :%s", me.name, parv[0], ptr->name);
			}
		}
	}
	return;
}
