#!/bin/sh

echo "Extracting src/version.c..."

if test -r ../Changes
then
	id=`grep '$Id: Changes,v' ../Changes`
	id=`echo $id |sed 's/.* Changes\,v \(.*\) .* Exp .*/\1/'`
else
	id="unknown"
fi
echo "$id"

if test -r version.c.last
then
	generation=`sed -n 's/^char \*generation = \"\(.*\)\";/\1/p' < version.c.last`
if test ! "$generation" ; then generation=0; fi
else
	generation=0
fi

generation=`expr $generation + 1`

uname=`uname -a`

creation=`date | awk '{ if (NF == 6) { print $1 " "  $2 " " $3 " "  $6 " at " $4 " " $5 } else \
	{ print $1 " "  $2 " " $3 " " $7 " at " $4 " " $5 " " $6 }}'`

cat >>version.c<<!SUB!THIS!
/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/version.c.SH: version information
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: version.c.SH,v 1.23 2002/04/08 16:16:26 rageircd Exp $
 */

#include "struct.h"
#include "patchlevel.h"

char *generation = "$generation";
char *creation = "$creation";
char *build = "$id";
char version[128];

char *infotext[] =
{
	"RageIRCd, an advanced Internet Relay Chat daemon (IRCd)"
	"Based on the original code written by Jarkko Oikarinen",
	"Copyright (C) 2002 the past and present IRCd developers, and others.",
	" ",
	"This program is free software; you can redistribute it and/or modify",
	"it under the terms of the GNU General Public License as published by",
	"the Free Software Foundation; either version 2 of the License, or",
	"(at your option) any later version.",
	" ",
	"This version of RageIRCd was brought to you by the following people:",
	" ",
	"^Th3Dud3^         Alasdair McWilliam      rageircd@users.sourceforge.net",
	"Dogbert           Kyle Williamson         admin@kylewilliamson.ath.cx",
	"Rob               David Robson            david.robson@virgin.net",
	" ",
	"With thanks to the following people who have contributed their code,",
	"ideas or support to this version of RageIRCd:",
	" ",
	"Acidic32          Daniel Moss             daniel@ravehosts.co.uk",
	"Asmodai           Paul Ward               asmodai@unixware.org.uk",
	"StrikerST         Robert Krule            strikers@relic.net",
	"Raider            Brian M King            raider@darker.net",
	"^Stinger^         Jacques Weewer          stinger@kippesoep.nl.com",
	"Jamie             Jamie Chetta            jchetta@optonline.net",
	" ",
	"Thanks also goes to the people not listed above who have contributed to",
	"previous versions of RageIRCd.",
	" ",
	"RageIRCd v2 also contains code developed by the following people:",
	" ",
	"ShadowMaster      Thomas Stentas          shadowmaster@shadow-realm.org",
	"RogerY            Roger Yerramsetti       rogery@austnet.org",
	"Stskeeps          Carsten V Munk          stskeeps@unrealircd.com",
	"binary            -                       garryb@binary.islesfan.net",
	"GZ                Maarten van den Bosch   gz@ircsystems.net",
	"codemastr         -                       codemastr@unrealircd.com",
	"skold             Darcy Grexton           skold@habber.net",
	" ",
	"RageIRCd v2 is based upon bahamut-1.4.24. To see the bahamut credits, type",
	"/DALINFO. The official RageIRCd website is http://rageircd.sourceforge.net.",
	" ",
	0,
};

char *dalinfotext[] =
{
	"The following people have contributed code, or support",
	"to the bahamut team:",
	"aClient             Diane Bruce         db@koruna.varner.com",
	"Dakal               David Knepper       dakal@dal.net",
	"Dalvenjah           Sven Nielsen        dalvenjah@dal.net",
	"Epiphani            Aaron Wiebe         epiphani@dal.net",
	"lucas               Lucas Madar         lucas@negaverse.org",
	"MSofty              Mark Salerno        msofty@dal.net",
	"Raistlin_Mejere     Jason Slagle        raistlin@bahamut.net",
	"Rakarra             Ian Westcott        rakarra@dal.net",
	"sedition            David Parton        sedition@dal.net",
	"White_Dragon        Chip Norkus         wd@dal.net",
	" ",
	"The following people have contributed ideas or testing to the",
	"bahamut project:",
	"driz                David Friedman      driz@dal.net",
	"habit               Melinda Thompson    habit@dal.net",
	"Kevin               Kevin Turner        kevin@dal.net",
	" ",
	"Developers and Contributors for previous versions:",
	" ",
	"Chris A. Bongaarts  Chris Behrens       Diane Bruce",
	"Johannes Erdfelt    Roger Espel Llima   Jon Lusky",
	"Michael Pearce      Brian Kraemer       Joan Touzet",
	"Chris Portman       Keith Fralick       Sam Noble",
	"Taner Halicioglu    Chris Behrens       Darren Reed",
	"Markku Savela       Greg Lindahl        Jarkko Oikarinen",
	"Armin Gruner        Matthew Green       Chuck Kane",
	"Matt Lyle           Vesa Ruokonen       Nicolas PIOCH",
	"Stellan Klebom      Dan Goodwin         Mike Bolotski",
	"Ian Frechette       Markku Jarvinen     Kimmo Suominen",
	"Jeff Trim           Vijay Subramaniam   Karl Kleinpaste",
	"Bill Wisner         Tom Davis           Hugo Calendar",
	"Tom Hopkins         Stephen van den Berg",
	"Bo Adler            Michael Sandrof     Jon Solomon",
	"Jan Peterson        Helen Rose          Paul Graham",
	" ",
	"Thanks also goes to those persons not mentioned here who have added",
	"their advice, opinions, and code to IRC.",
	"Thanks also to those who provide the kind sys admins who let me and",
	"others continue to develop IRC.",
	"Thanks also to those who continue to make twinkies and doritos, as they",
	"are the backbone of most programmers' diets and thus make it possible",
	"for code like this to be developed!",
	" ",
	0,
};
!SUB!THIS!
