/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * tools/convertconf.c: configuration file converter
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: convertconf.c,v 1.1 2002/04/08 16:30:04 rageircd Exp $
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#define BadPtr(x) (!(x) || (*(x) == '\0'))

char *mainip = NULL;
char *mainport = NULL;

static char *usage[] = {
	" ",
	"Configuration File Converter for RageIRCd v2",
	"Copyright (C) 2002 the past and present IRCd developers, and others.",
	"Refer to doc/authors/ for full credits and copyright details.",
	" ",
	"This converter can be used to convert any of the old configuration",
	"files (ircd.conf, vhost.conf, etc) from RageIRCd v1.5.x only, to the",
	"latest format in RageIRCd v2.",
	" ",
	"Usage: convertconf <filename> [output]",
	" ",
	"Where <filename> is the name of the file you want converting, and the",
	"optional argument [output] is the name of the output file, or simply",
	"<filename>.new if not specified.",
	" ",
	"If you require any assistance, please read the documentation provided.",
	"If this does not help your situation, please mail us a supprot request",
	"via rageircd-support@lists.sourceforge.net.",
	" ",
	NULL
};

void strip(char *s)
{
	char *p;
	if ((p = strchr(s, '\n')) || (p = strchr(p, '\n'))) {
		*p = '\0';
	}
	return;
}

char *getfield(char *newline)
{
	static char *line = (char *)NULL;
	char *end, *field;

	if (newline != NULL) {
		line = newline;
	}
	if (line == (char *)NULL) {
		return NULL;
	}

	field = line;
	if ((end = strchr(line, ':')) == NULL) {
		line = (char *)NULL;
		if ((end = strchr(field, '\n')) == NULL) {
			end = field + strlen(field);
		}
	}
	else {
		line = end + 1;
	}
	*end = '\0';
	return field;
}

int main(int argc, char *argv[])
{
	FILE *in, *out;
	char buffer[1024], newfile[256];
	char *ptr, *s;

	if (argc < 2) {
		char **text = usage;
		while (*text) {
			fprintf(stderr, "%s\n", *text++);
		}
		fprintf(stderr, "No input file specified.\n");
		return 0;
	}

	in = fopen(argv[1], "r");
	if (in == NULL) {
		fprintf(stderr, "ERROR: failed to open input file %s: %s\n", argv[1], strerror(errno));
		return 0;
	}

	if (argc > 2) {
		sprintf(newfile, "%s", argv[2]);
		out = fopen(argv[2], "w");
	}
	else {
		if (strlen(argv[1]) > 250) {
			fprintf(stderr, "WARNING: filename %s is too large, truncated to ", argv[1]);
			argv[1][250] = '\0';
			fprintf(stderr, "%s\n", argv[1]);
		}
		sprintf(newfile, "%s.new", argv[1]);
		out = fopen(newfile, "w");
	}
	if (out == NULL) {
		fprintf(stderr, "ERROR: failed to open output file %s: %s\n", newfile, strerror(errno));
		return 0;
	}

	while (fgets(buffer, 1024, in)) {
		ptr = buffer;
		while (*ptr == ' ') {
			ptr++;
		}
		if (*ptr == '#' || *ptr == '\n' || *ptr == '\r' || *ptr == '\0') {
			continue;
		}
		strip(ptr);

		if (*(ptr + 1) == ':') {
			switch (*ptr) {
				case 'M':
				case 'm':
					ptr += 2;
					s = getfield(ptr);
					fprintf(out, "servinfo %s", s);
					s = getfield(NULL);
					mainip = strdup(s);
					s = getfield(NULL);
					fprintf(out, " \"%s\"\n", s);
					s = getfield(NULL);
					mainport = strdup(s);
					break;
				case 'A':
					ptr += 2;
					s = getfield(ptr);
					if (!BadPtr(s)) {
						fprintf(out, "admin \"%s\"", s);
						s = getfield(NULL);
						if (!BadPtr(s)) {
							fprintf(out, " \"%s\"", s);
							s = getfield(NULL);
							if (!BadPtr(s)) {
								fprintf(out, " \"%s\"", s);
							}
						}
					}
					fprintf(out, "\n");
					break;
				case 'Y':
				case 'y':
					ptr += 2;
					s = getfield(ptr);
					fprintf(out, "class %s", s);
					s = getfield(NULL);
					fprintf(out, " %s", s);
					s = getfield(NULL);
					fprintf(out, " %s", s);
					s = getfield(NULL);
					fprintf(out, " %s", s);
					s = getfield(NULL);
					fprintf(out, " %s\n", s);
					break;
				case 'I':
				case 'i':
				{
					char *pass, *s2;
					ptr += 2;
					fprintf(out, "allow");
					s = getfield(ptr);
					if ((s2 = strchr(s, '@')) != NULL) {
						s2++;
						s = s2;
					}
					fprintf(out, " %s", s);
					pass = getfield(NULL);
					s = getfield(NULL);
					if ((s2 = strchr(s, '@')) != NULL) {
						s2++;
						s = s2;
					}
					fprintf(out, " %s", s);
					getfield(NULL);
					fprintf(out, " 3");
					s = getfield(NULL);
					fprintf(out, " %s", s);
					if (!BadPtr(pass)) {
						fprintf(out, " %s\n", pass);
					}
					else {
						fprintf(out, "\n");
					}
					break;
				}
				case 'X':
				case 'x':
					ptr += 2;
					s = getfield(ptr);
					fprintf(out, "password %s", !BadPtr(s) ? s : "$");
					s = getfield(NULL);
					fprintf(out, " %s\n", !BadPtr(s) ? s : "$");
					break;
				case 'O':
				case 'o':
				{
					char *host, *pass, *name, *flags, *class;
					ptr += 2;
					host = getfield(ptr);
					pass = getfield(NULL);
					name = getfield(NULL);
					flags = getfield(NULL);
					class = getfield(NULL);
					fprintf(out, "oper %s %s %s %s", name, host, pass, class);
					if (!BadPtr(flags)) {
						fprintf(out, " %s\n", flags);
					}
					else {
						fprintf(out, "\n");
					}
					break;
				}
				case 'U':
				case 'u':
					ptr += 2;
					s = getfield(ptr);
					fprintf(out, "uline %s\n", s);
					break;
				case 'C':
				case 'c':
				{
					char *ip, *pass, *name, *port, *class;
					ptr += 2;
					ip = getfield(ptr);
					pass = getfield(NULL);
					name = getfield(NULL);
					port = getfield(NULL);
					class = getfield(NULL);
					fprintf(out, "link %s %s * %s %s %s\n", name, ip, pass,
						!BadPtr(port) ? port : "7000", class);
					break;
				}
				case 'Q':
				case 'q':
				{
					char *reason;
					ptr += 2;
					getfield(ptr);
					reason = getfield(NULL);
					s = getfield(NULL);
					fprintf(out, "ban nickname \"%s\" \"%s\"\n", s,
						!BadPtr(reason) ? reason : "No reason specified");
					break;
				}
				case 'K':
				case 'k':
				{
					char *host, *reason, mask[128];
					ptr += 2;
					host = getfield(ptr);
					reason = getfield(NULL);
					s = getfield(NULL);
					sprintf(mask, "%s@%s", !BadPtr(s) ? s : "*", host);
					fprintf(out, "ban usermask %s \"%s\"\n", mask,
						!BadPtr(reason) ? reason : "No reason specified");
					break;
				}
				case 'Z':
				case 'z':
					ptr += 2;
					s = getfield(ptr);
					fprintf(out, "ban ip %s", s);
					s = getfield(NULL);
					fprintf(out, " \"%s\"\n", !BadPtr(s) ? s : "No reason specified");
					break;
				case 'B':
				case 'b':
				{
					int len;
					ptr += 2;
					s = getfield(ptr);
					len = strlen(s);
					if (s[len - 1] == '*') {
						s[len - 1] = '\0';
					}
					fprintf(out, "ban file \"%s\" \"transfer of this file is prohibited\"\n", s);
					break;
				}
				case 'n':
					ptr += 2;
					s = getfield(ptr);
					fprintf(out, "ban gcos \"%s\" \"your gcos is prohibited\"\n", s);
					break;
				case 'P':
				case 'p':
					ptr += 2;
					s = getfield(ptr);
					fprintf(out, "listen %s", s);
					getfield(NULL);
					getfield(NULL);
					s = getfield(NULL);
					fprintf(out, " %s\n", s);
					break;
				default:
					fprintf(out, "/* unknown entry: %s */\n", ptr);
					break;
			}
		}
		else {
			if ((s = strtok(ptr, " ")) == NULL) {
				continue;
			}
			if (!strcasecmp(s, "vhost")) {
				char *user = strtok(NULL, " ");
				char *pass = strtok(NULL, " ");
				char *vhost = strtok(NULL, " ");
				if (BadPtr(user) || BadPtr(pass) || BadPtr(vhost)) {
					continue;
				}
				fprintf(out, "vhost %s %s %s *@*\n", vhost, user, pass);
				continue;
			}
			if (!strcasecmp(s, "restrict")) {
				char *chan = strtok(NULL, " ");
				if (BadPtr(chan)) {
					continue;
				}
				fprintf(out, "ban channel %s \"restricted channel\"\n", chan);
				continue;
			}
		}
				
	}
	if (mainip != NULL || mainport != NULL) {
		fprintf(out, "listen %s %s", mainip, mainport);
		free(mainip);
		free(mainport);
	}
	fclose(out);
	return 1;
}
